# Check for the expat library and headers.  This is an Autoconf
# Extension, hence the prefix "AX_".  Note that this macro will set
# ${expat_found} to "yes" if expat was found, and "no" otherwise.
#
# FIXME!! This file should probably be split to "ax_check_expat.m4"
# and "ax_check_xerces_c.m4"

AC_DEFUN([AX_CHECK_EXPAT],
[
  AC_ARG_WITH([expat],
    AC_HELP_STRING([--with-expat],
                   [use expat XML library [[yes]]]),
    [with_expat=${withval}],
    [with_expat=yes])

  if test "X${with_expat}" != "Xno"; then
    if test "X${with_expat}" != "Xyes"; then
      if test -d ${with_expat}/include; then
        CPPFLAGS="-I${with_expat}/include ${CPPFLAGS}"
      fi
      if test -d ${with_expat}/lib; then
        LDFLAGS="-L${with_expat}/lib ${LDFLAGS}"
      fi
    fi
    EXPAT_LIBS="-lexpat"

    AC_LANG_PUSH([C])

    AC_CHECK_HEADERS([expat.h],
                     [expat_headers_found=yes],
                     [expat_headers_found=no])

    AC_CHECK_LIB([expat],
                 [XML_ParserCreate],
                 [expat_lib_found=yes],
                 [expat_lib_found=no])

    expat_found="no"
    if test "X${expat_headers_found}" == "Xyes" \
         -a "X${expat_lib_found}"     == "Xyes"; then
      expat_found="yes"
    fi

    AC_LANG_POP([C])

    AC_SUBST([EXPAT_LIBS])
    AC_SUBST([expat_found])
  fi
])


# Check for the Xerces-C++ library and headers.  This is an Autoconf
# Extension, hence the prefix "AX_".  Note that this macro will set
# ${xerces_c_found} to "yes" if Xerces-C++ was found, and "no"
# otherwise.  This check is very much based on the macro by Ben
# Bornstein and Mike Hucka that is distributed with libSBML.

AC_DEFUN([AX_CHECK_XERCES_C],
[
  AC_ARG_WITH([xerces-c],
    AC_HELP_STRING([--with-xerces-c],
                   [use Xerces-C++ XML library [[yes]]]),
    [with_xerces_c=${withval}],
    [with_xerces_c=yes])

  if test "X${with_xerces_c}" != "Xno"; then
    if test "X${with_xerces_c}" != "Xyes"; then
      if test -d ${with_xerces_c}/include; then
        CPPFLAGS="-I${with_xerces_c}/include ${CPPFLAGS}"
      fi
      if test -d ${with_xerces_c}/include; then
        LDFLAGS="-L${with_xerces_c}/lib ${LDFLAGS}"
      fi
    fi
    XERCES_C_LIBS="-lxerces-c"

    AC_LANG_PUSH([C++])

    AC_CHECK_HEADERS([xercesc/util/XercesDefs.hpp],
                     [xerces_c_headers_found=yes],
                     [xerces_c_headers_found=no])

    AC_MSG_CHECKING([for Initialize in -lxerces-c])
    LIBS_tmp="${LIBS}"
    LIBS="${XERCES_C_LIBS} ${LIBS}"
    AC_TRY_LINK(
      [#include <xercesc/util/XercesDefs.hpp>
       #include <xercesc/util/PlatformUtils.hpp>
       #ifndef XERCES_HAS_CPP_NAMESPACE
       #  define XERCES_CPP_NAMESPACE_QUALIFIER
       #endif],
      [XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();],
      [xerces_c_lib_found=yes],
      [xerces_c_lib_found=no])
    LIBS="${LIBS_tmp}"
    AC_MSG_RESULT([${xerces_c_lib_found}])

    xerces_c_found="no"
    if test "X${xerces_c_headers_found}" == "Xyes" \
         -a "X${xerces_c_lib_found}"     == "Xyes"; then
      xerces_c_found="yes"
    fi

    AC_LANG_POP([C++])

    AC_SUBST([XERCES_C_LIBS])
    AC_SUBST([xerces_c_found])
  fi
])
