/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * The Annotation class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: Annotation.cpp 560 2008-11-09 11:12:14Z davidfange $
 */

  /* For compilation on Windows using the Microsoft Visual C++
   * compiler, replace calls to deprecated functions with calls to the
   * new, secure versions of those functions by defining
   * _CRT_SECURE_CPP_OVERLOAD STANDARD_NAMES as 1.  Turn off warnings
   * about any remaining deprecated functions by defining
   * _CRT_SECURE_NO_WARNINGS as 1.  Additionally, turn off the
   * "assignment operator could not be generated" and "identifier was
   * truncated to '255' characters in the debug information" warnings.
   * These preprocessor directives are needed before the inclusion of
   * any STL headers in order to be useful.
   */
#ifdef _MSC_VER
#  if  _MSC_VER > 1000 && !defined(__INTEL_COMPILER)
#    define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#    define _CRT_SECURE_NO_WARNINGS                 1
#    pragma warning(disable:4512)
#    pragma warning(disable:4786)
#  endif
#endif

  /* For compilation using the Intel C++ compiler, turn off the "value
   * copied to temporary, reference to temporary used" and "operands
   * are evaluated in unspecified order" warnings.
   */
#ifdef __INTEL_COMPILER
#  if  __INTEL_COMPILER >= 800
#    pragma warning(disable:383)
#    pragma warning(disable:981)
#  endif
#endif


#include "Annotation.hpp"
#include "ui/OPrefixStream.hpp"



  /********************
   * EXPAT XML ENGINE *
   ********************/
//#ifdef HAVE_EXPAT
//#  undef HAVE_XERCES_C
//#  include <expat.h>
/*
namespace
{
*/
    /**
     * @brief The ExpatAnnotation class.
     *
     * Handles all the XML parsing using the expat library.  Given a
     * pointer to an SBML base object, it will extract the information
     * from the annotation field into one or more instances of the
     * Annotation class.  The ExpatAnnotation class is a minimal class
     * that should not be used outside this file; hence, its interface
     * is nowhere exported.  The simplicity of this class stems
     * largely from the fact that aside from namespaces, the class
     * only needs to consider XML attributes and XML elements.
     * 
     */
/*class ExpatAnnotation
  {
  private:
    const SBase_t*                   sbase;
    XML_Parser                       parser;
    std::vector<MesoRD::Annotation*> annotations;

    public:*/

      /**
       * @brief The ExpatAnnotation class constructor.
       *
       * The constructor will create an XML parser object.  The parser
       * does the real work in extracting the information needed from
       * XML.  Because we know that SBML documents are always written
       * in UTF-8, the parser is hard-coded to that particular
       * encoding scheme.  Creation of the parser object can fail if
       * there is insufficient memory available.  In such cases, an
       * exception is thrown.
       */
/*inline ExpatAnnotation()
      : sbase(0), parser(0)
      {
        parser = XML_ParserCreateNS("UTF-8",
          MesoRD::Identifier::XML::expat_namespace_separator);
        if (parser == NULL) {
          throw MesoRD::Annotation::InitialisationError(
            "Could not allocate memory for expat parser.");
        }
        XML_SetElementHandler(parser, &start, &end);
      }

*/
      /**
       * @brief The ExpatAnnotation class destructor.
       *
       * Release the parser, if present.  Make sure to set the parser
       * object to <code>null</code>, in order to prevent accidental
       * segmentation faults.
       */
/*  inline ~ExpatAnnotation()
      {
        if (parser)
          XML_ParserFree(parser);
        parser = 0;
      }
*/

      /**
       * @brief End tag handler for the expat XML handler.
       *
       * This function acts the end tag handler for the expat XML
       * parser.  It is called whenever an end tag is encountered.  On
       * being called, the function pops an annotation object from the
       * stack.  This should mean that whenever an end tag is
       * encountered, the top element on the stack is already saved in
       * the child list of some annotation.  The exception is the root
       * annotation element; if we were to pop that element, we would
       * loose all references to that particular annotation.  Then,
       * fromXML() would be unable to retrieve the information we have
       * just parsed.
       *
       * @param data    A pointer to the user data of the parser.
       * @param element A pointer to the name of the element we have
       *                just seen the end of.
       */
/*  static inline void
    end(void* data, const char* element)
      {
        std::string                      e_name;
        std::string                      e_namespace;
        std::stack<MesoRD::Annotation*>* parser_stack =
          static_cast<std::stack<MesoRD::Annotation*>*>(data);
        getNamespace(element, e_name, e_namespace);
        if (parser_stack
          && e_name      != MesoRD::Identifier::SBML_annotation
          && e_namespace == MesoRD::Identifier::XML::xmlns)
          parser_stack -> pop();
      }

*/
      /**
       * @brief Construct an Annotation object from an SBML base object.
       *
       * This function will parse all XML contained in the annotation
       * of an SBML base object and store pointers to the created
       * Annotation objects in the <code>annotations</code> vector.  A
       * pointer to the SBML base object is stored as well.  The point
       * of the <code>annotations</code> vector and the pointer to the
       * SBML base object is that we will not have to redo the
       * parsing, should one decide that one wants to extract another
       * annotation from the same SBML base object.  This function
       * involves XML parsing, which may fail.  If parsing fails, this
       * function will throw an exception defined in the Annotation
       * class.
       *
       * @param s The SBase pointer to extract the Annotation from.
       * @param i The index of the annotation field we are interested
       *          in.  Usually, we care only about the first field.
       * @return  A pointer to the annotation newly created annotation
       *          object.
       */
/*  MesoRD::Annotation*
    fromXML(SBase_t* s, size_t i = 0);

*/
      /**
       * @brief Separates an identifier into name and namespace.
       *
       * In a namespacing expat parser, tokens are returned as
       * namespace identifiers concatenated with the element name.
       * The two entities are separated by a separator character.
       * This function separates the two.  The concatenated string is
       * searched for the separator in reverse order, because the
       * namespace identifier may contain the separator character,
       * while the element name certainly cannot.
       *
       * @param identifier   The concatenated identifier.
       * @param id_name      The name of the identifier.
       * @param id_namespace The namespace of the identifier.
       * @return             True if the identifier specified a
       *                     namespace, false otherwise.
       */
/*  inline static bool
    getNamespace(
      const char* identifier, std::string& id_name, std::string& id_namespace) 
      {
        const std::string id_str(identifier);
        const size_t      sep = id_str.rfind(
          MesoRD::Identifier::XML::expat_namespace_separator);
        if (sep != std::string::npos) {
          id_namespace = id_str.substr(0, sep);
          id_name      = id_str.substr(sep + 1, id_str.size());
          return true;
        }
        id_namespace = "";
        id_name      = id_str;
        return false;
      }
*/

      /**
       * @brief Start tag handler for the expat XML parser.
       *
       * This function acts as the start tag handler for the expat XML
       * parser.  It is called whenever a new start tag is
       * encountered.  On being called, the function pushes a new
       * annotation object, with attributes and everything, to the
       * stack.  The stack is passed as user data.
       *
       * @param data    A pointer to the user data of the parser.
       * @param element A pointer to the name of the element we have
       *                just seen the start of.
       *@param attr    A pointer the list of attributes of the recently
       *                started element.
       */
/*  static void
    start(void* data, const char* element, const char** attr);
  };
*/

    /* Check that we have not already parsed the given SBML base
     * object.  If we have, we can just extract the i:th element from
     * the annotations vector.  Otherwise, we do some sanity checks
     * and call the expat parser on the data from the annotation
     * object.  We must account for the octets per character when
     * indicating the size of the buffer to parse.  The parser will
     * call the start and end tag handlers as appropriate and fail on
     * errors, should they occur.  Finally, what is left on the stack
     * will be the Annotation objects from the root annotation fields.
     * These are put into the annotations vector in reverse order,
     * because that is the way a stack works.
     */
/*MesoRD::Annotation*
  ExpatAnnotation::fromXML(SBase_t* s, size_t i)
  {
    if (parser && s && SBase_isSetAnnotation(s)) {
      if (s != sbase) {
        const char*                      buffer_sbml  =
          SBase_getAnnotation(s);
        const size_t                     buffer_len   =
          strlen(buffer_sbml) * sizeof(char);
        std::stack<MesoRD::Annotation*>* parser_stack =
          new std::stack<MesoRD::Annotation*>;

        sbase = s;
        annotations.clear();
        XML_SetUserData(parser, parser_stack);

        if (!XML_Parse(parser, buffer_sbml, static_cast<int>(buffer_len), 0)) {
          unsigned int line    = XML_GetCurrentLineNumber(parser);
          const char*  message = XML_ErrorString(XML_GetErrorCode(parser));
          throw MesoRD::Annotation::ParseError(line, message);
        }

        while (!parser_stack -> empty()) {
          annotations.resize(annotations.size() + 1);
          for (size_t j = 0; j < annotations.size() - 1; j++)
            annotations[j + 1] = annotations[j];
          annotations[0] = parser_stack -> top();
          parser_stack -> pop();
        }

        delete parser_stack;
      }
    }

    if (s && i < annotations.size())
      return annotations[i];
    return 0;
  }
*/

    /* Check that the name of the element we are about to investigate
     * is defined in the proper namespace.  For the first element, the
     * root SBML annotation element, the proper namespace is the
     * empty, or default, namespace.  For any other elements, the
     * namespace must be the MesoRD namespace, identified by the
     * appropriate URL.  Any non-root elements are added to list of
     * children of the top Annotation object on the stack.  Once this
     * check has been performed, add every attribute defined in the
     * namespace to a newly created Annotation object.  Note that the
     * root element does not contain any attributes we care about.
     */
/*void
  ExpatAnnotation::start(void* data, const char* element, const char** attr)
  {
    std::stack<MesoRD::Annotation*>* parser_stack =
      static_cast<std::stack<MesoRD::Annotation*>*>(data);

    if (attr && element && parser_stack) {
      std::string e_name;
      std::string e_namespace;

      getNamespace(element, e_name, e_namespace);

      if (e_name == MesoRD::Identifier::SBML_annotation
        && e_namespace.length() <= 0) {
        MesoRD::Annotation* root_annotation = new MesoRD::Annotation(e_name);
        parser_stack -> push(root_annotation);

      } else if (e_name.length() > 0
        && e_namespace == MesoRD::Identifier::XML::xmlns) {

        MesoRD::Annotation* child_annotation = new MesoRD::Annotation(e_name);

        for (size_t i = 0; attr[i]; i += 2) {
          std::string attr_name;
          std::string attr_namespace;
          std::string attr_value(attr[i + 1]);

          getNamespace(attr[i], attr_name, attr_namespace);
          if (attr_namespace == MesoRD::Identifier::XML::xmlns)
            child_annotation -> addAttribute(attr_name, attr_value);
        }

        parser_stack -> top() -> addChild(child_annotation);
        parser_stack -> push(child_annotation);
      }
    }
  }
}
//#endif /* ifdef HAVE_EXPAT */



  /*************************
   * XERCES-C++ XML ENGINE *
   *************************/
/*#ifdef HAVE_XERCES_C

#  include <xercesc/framework/MemBufInputSource.hpp>
#  include <xercesc/sax2/DefaultHandler.hpp>
#  include <xercesc/sax2/SAX2XMLReader.hpp>
#  include <xercesc/sax2/XMLReaderFactory.hpp>

namespace
{
  using XERCES_CPP_NAMESPACE_QUALIFIER Attributes;
  using XERCES_CPP_NAMESPACE_QUALIFIER DefaultHandler;
  using XERCES_CPP_NAMESPACE_QUALIFIER MemBufInputSource;
  using XERCES_CPP_NAMESPACE_QUALIFIER SAX2XMLReader;
  using XERCES_CPP_NAMESPACE_QUALIFIER XMLException;
  using XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils;
  using XERCES_CPP_NAMESPACE_QUALIFIER XMLReaderFactory;
  using XERCES_CPP_NAMESPACE_QUALIFIER XMLString;
  using XERCES_CPP_NAMESPACE_QUALIFIER XMLUni;

*/
    /**
     * @brief The XercesCAnnotation class.
     *
     * Handles all the XML parsing using the Xerces-C++ library from
     * the Apache XML project.  Given a pointer to an SBML base
     * object, it will extract the information from the annotation
     * field into one or more instances of the Annotation class.  The
     * XercesCAnnotation class is a minimal class that should not be
     * used outside this file; hence, its interface is nowhere
     * exported.  The simplicity of this class stems largely from the
     * fact that aside from namespaces, the class only needs to
     * consider XML attributes and XML elements.  Note that prior to
     * the definition of this class, relevant portions of the
     * Xerces-C++ namespace are dumped into the MesoRD namespace.
     * This should not cause any problems.  Note also that Xerces-C++
     * is built with RTTI disabled by default, so we have to use
     * static_cast<>() throughout.
     */
  /*class XercesCAnnotation : public DefaultHandler
  {
  private:
    const SBase_t*                   sbase;
    SAX2XMLReader*                   parser;
    std::stack<MesoRD::Annotation*>  parser_stack;
    std::vector<MesoRD::Annotation*> annotations;

  public:
  */
      /**
       * @brief The XercesCAnnotation class constructor.
       *
       * The constructor will initialise the Xerces-C++ library and
       * create a XML Reader object.  The latter is what does the real
       * work in extracting the information needed from XML and is
       * configured to suit present needs here.  If any of these
       * actions should result in exceptions, they are passed on as
       * InitialisationError exceptions of the Annotation class.  We
       * must make sure the parser is released in the destructor.
       * Note that it is OK to call Initialize() several times; it
       * will not effect anything except possibly the locale, if one
       * would want to change those.
       */
  /*inline XercesCAnnotation()
      : sbase(0), parser(0)
      {
        try {
          XMLPlatformUtils::Initialize();
          parser = XMLReaderFactory::createXMLReader();
          if (parser != NULL) {
            parser -> setFeature(XMLUni::fgSAX2CoreNameSpaces,        true);
            parser -> setFeature(XMLUni::fgSAX2CoreNameSpacePrefixes, true);
            parser -> setFeature(XMLUni::fgSAX2CoreValidation,        true);
            parser -> setFeature(XMLUni::fgXercesDynamic,             true);
            parser -> setFeature(XMLUni::fgXercesSchema,              false);
            parser -> setFeature(XMLUni::fgXercesSchemaFullChecking,  false);

            parser -> setContentHandler(this);
            parser -> setErrorHandler(this);

          } else {
            throw MesoRD::Annotation::InitialisationError(
              "Could not allocate memory for Xerces-C++ parser.");
          }
        } catch (const XMLException& e) {
          std::string message = transcode(e.getMessage());
          throw MesoRD::Annotation::InitialisationError(message.c_str());
        }
      }

  */
      /**
       * @brief The XercesCAnnotation class destructor.
       *
       * First delete the parser if present, as this must be done
       * prior to calling Terminate().  Any child nodes allocated will
       * be marked for garbage collection.  Note, however, that the
       * call to Terminate() is optional in current versions of
       * Xerces-C++.
       */
  /*inline ~XercesCAnnotation()
      {
        if (parser)
          delete parser;
        parser = 0;
        XMLPlatformUtils::Terminate();
      }

  */
  	  /* Turn off the "unreferenced formal parameter" warning when
       * compiling with the Microsoft Visual C++ compiler.
       */
  /*#ifdef _MSC_VER
#  if  _MSC_VER > 1000 && !defined(__INTEL_COMPILER)
#    pragma warning(push)
#    pragma warning(disable:4100)
#  endif
#endif
  */
      /**
       * @brief End tag function for the Xerces-C++ XML Reader.
       *
       * This function is called whenever an end tag is encountered.
       * On being called, the function pops an annotation object from
       * the stack.  This should mean that whenever an end tag is
       * encountered, the top element on the stack is already saved in
       * the child list of some annotation.  The exception is the root
       * annotation element; if we were to pop that element, we would
       * loose all references to that particular annotation.  Then,
       * fromXML() would be unable to retrieve the information we have
       * just parsed.
       *
       * @param uri       The URI of the associated namespace for this
       *                  element.
       * @param localname The local part of the element name.
       * @param qname     The qualified name of this element.
       */
  /*void
    endElement(const XMLCh* const uri, const XMLCh* const localname,
      const XMLCh* const qname)
      {
        std::string e_name      = transcode(localname);
        std::string e_namespace = transcode(uri);
        if (e_name != MesoRD::Identifier::SBML_annotation
          && e_namespace == MesoRD::Identifier::XML::xmlns)
          parser_stack.pop();
      }
#ifdef _MSC_VER
#  if  _MSC_VER > 1000 && !defined(__INTEL_COMPILER)
#    pragma warning(pop)
#  endif
#endif

  */
      /**
       * @brief Construct an Annotation object from an SBML base
       * object.
       *
       * This function will parse all XML contained in the annotation
       * of an SBML base object and store pointers to the created
       * Annotation objects in the <code>annotations</code> vector.  A
       * pointer to the SBML base object is stored as well.  The point
       * of the <code>annotations</code> vector and the pointer to the
       * SBML base object is that we will not have to redo the
       * parsing, should one decide that one wants to extract another
       * annotation from the same SBML base object.  This function
       * involves XML parsing, which may fail.  If parsing fails, this
       * function will throw an exception defined in the Annotation
       * class.
       *
       * @param s The SBase pointer to extract the Annotation from.
       * @param i The index of the annotation field we are interested
       *          in.  Usually, we care only about the first field.
       * @return  A pointer to the annotation newly created annotation
       *          object.
       */
  /*MesoRD::Annotation*
    fromXML(SBase_t* s, size_t i = 0);

  */
      /**
       * @brief Start tag function for the Xerces-C++ XML Reader.
       *
       * This function is called whenever a new start tag is
       * encountered.  On being called, the function pushes a new
       * annotation object, with attributes and everything, to the
       * stack.
       *
       * @param data    A pointer to the user data of the parser.
       * @param element A pointer to the name of the element we have
       *                just seen the start of.
       *@param attr    A pointer the list of attributes of the recently
       *                started element.
       */
  /*void
    startElement(const XMLCh* const uri, const XMLCh* const localname,
      const XMLCh* const qname, const Attributes& attributes);

  */
      /**
       * @brief Trans-code a string from XML to native code-page.
       *
       * This functionality is needed so frequently that the creation
       * of the below function is merited.  It will convert a string
       * given in the XML code-page to the native code-page, encoded
       * in a standard C++ string.  The default value, should anything
       * go bad, is the empty string.
       *
       * @param xml A pointer to the string in the XML locale.
       * @return    The text converted to a C++ standard string.
       */
  /*static inline std::string
    transcode(const XMLCh* xml)
      {
        std::string native("");
        if (xml) {
          char* native_char = XMLString::transcode(xml);
          if (native_char) {
            native = std::string(native_char);
            XMLString::release(&native_char);
          }
        }
        return native;
      }
  };

  */
    /* Check that we have not already parsed the given SBML base
     * object.  If we have, we can just extract the i:th element from
     * the annotations vector.  Otherwise, we do some sanity checks
     * and call the reader on a MemBufInputSource created from the
     * buffer containing the XML statements.  We must account for the
     * octets per character when indicating buffer size.  Note that
     * the source will spool from a reference of the data, as opposed
     * from a locally adopted copy of it.  The reader will call the
     * start and end tag functions as appropriate and fail on errors,
     * should they occur.  Finally, what is left on the stack will be
     * the Annotation objects from the root annotation fields.  These
     * are put into the annotations vector in reverse order, because
     * that is the way a stack works.
     */
  /*MesoRD::Annotation*
  XercesCAnnotation::fromXML(SBase_t* s, size_t i)
  {
    if (parser && SBase_isSetAnnotation(s)) {
      if (s != sbase) {
        const XMLByte* const buffer_sbml =
          (const XMLByte*)SBase_getAnnotation(s);
        const size_t         buffer_len  =
          strlen((const char*)buffer_sbml) * sizeof(char);

        sbase = s;
        annotations.clear();

        try {
          MemBufInputSource source(buffer_sbml, buffer_len,
            MesoRD::Identifier::XML::xercesc_buffer_name.c_str(), false);

          while (!parser_stack.empty())
            parser_stack.pop();
          parser -> parse(source);

        } catch (const XMLException& e) {
          std::string  message = transcode(e.getMessage());
          unsigned int line    = e.getSrcLine();
          MesoRD::Annotation::ParseError exception(line, message.c_str());

          throw exception;
        }

        while (!parser_stack.empty()) {
          annotations.resize(annotations.size() + 1);
          for (size_t j = 0; j < annotations.size() - 1; j++)
            annotations[j + 1] = annotations[j];
          annotations[0] = parser_stack.top();
          parser_stack.pop();
        }
      }
    }

    if (s && i < annotations.size())
      return annotations[i];
    return 0;
  }
  */

    /* Turn off the "unreferenced formal parameter" warning when
	   * compiling with the Microsoft * Visual C++ compiler.
	   */
  /*
#ifdef _MSC_VER
#  if  _MSC_VER > 1000 && !defined(__INTEL_COMPILER)
#    pragma warning(push)
#    pragma warning(disable:4100)
#  endif
#endif
  */
    /* Check that the name of the element we are about to investigate
     * is defined in the proper namespace.  For the first element, the
     * root SBML annotation element, the proper namespace is the
     * empty, or default, namespace.  For any other elements, the
     * namespace must be the MesoRD namespace, identified by the
     * appropriate URL.  Any non-root elements are added to list of
     * children of the top Annotation object on the stack.  Once this
     * check has been performed, add every attribute defined in the
     * namespace to a newly created Annotation object.  Note that the
     * root element does not contain any attributes we care about.
     */
  /* void
  XercesCAnnotation::startElement(const XMLCh* const uri,
    const XMLCh* const localname, const XMLCh* const qname,
    const Attributes& attributes)
  {
    std::string e_name      = transcode(localname);
    std::string e_namespace = transcode(uri);

    if (e_name
      == MesoRD::Identifier::SBML_annotation && e_namespace.length() <= 0) {
      MesoRD::Annotation* root_annotation = new MesoRD::Annotation(e_name);
      parser_stack.push(root_annotation);

    } else if (e_name.length() > 0
      && e_namespace == MesoRD::Identifier::XML::xmlns) {
      MesoRD::Annotation* child_annotation = new MesoRD::Annotation(e_name);

      for (size_t i = 0; i < attributes.getLength(); i++) {
        std::string attr_name      = transcode(attributes.getLocalName(i));
        std::string attr_namespace = transcode(attributes.getURI(i));
        std::string attr_value     = transcode(attributes.getValue(i));

        if (attr_namespace == MesoRD::Identifier::XML::xmlns)
          child_annotation -> addAttribute(attr_name, attr_value);
      }

      parser_stack.top() -> addChild(child_annotation);
      parser_stack.push(child_annotation);
    }
  }
}
  */
  /*
#ifdef _MSC_VER
#  if  _MSC_VER > 1000 && !defined(__INTEL_COMPILER)
#    pragma warning(pop)
#  endif
#endif
  */

//#endif /* ifdef HAVE_XERCES_C */


  /*******************************
   * GENERAL ANNOTATION FUNCTIONS *
   *******************************/

  /* In general, we should be able to just pass on the exceptions from
   * the parer.  Here, we just get the first annotation field of the
   * SBML base object, copy it over and delete the original.  We
   * should make sure to clear the children list before we delete the
   * original, so that the current instance will not suffer from
   * dangling pointers.
   */

  /*MesoRD::Annotation::Annotation(const ::XMLNode* n)
  throw(MesoRD::Annotation::InitialisationError,
    MesoRD::Annotation::ParseError)
  : name("")
{*/
  MesoRD::Annotation::Annotation(const ::XMLNode* n) : name("")
{
  attributes.clear();
  children.clear();

  std::string e_name      = n -> getName();
  std::string e_namespace = n -> getPrefix();
  
  if (e_name
    == MesoRD::Identifier::SBML_annotation && e_namespace.length() <= 0) { 
   for (size_t i = 0; i < n -> getNumChildren(); i++) 
      addChild(new MesoRD::Annotation( &(n -> getChild(i))) );

  } else if (e_name.length() > 0
    && e_namespace == MesoRD::Identifier::XML::xmlns_identifier) {
    
    name = e_name;    
    XMLAttributes a = n -> getAttributes();
    for (size_t i = 0; i < (unsigned int)a.getLength(); i++) {
      std::string attr_namespace = a.getURI(i);
      if (attr_namespace == MesoRD::Identifier::XML::xmlns)
        addAttribute(a.getName(i), a.getValue(i));
    }
    for (size_t i = 0; i < n -> getNumChildren(); i++) 
      addChild(new MesoRD::Annotation( &(n -> getChild(i))) );
  }
}

          
          

  
/*  if (s) {
    Annotation* annotation = 0;

#ifdef HAVE_EXPAT
    ExpatAnnotation expat;
    annotation = expat.fromXML(s, 0);
#endif
#ifdef HAVE_XERCES_C
    XercesCAnnotation xerces;
    annotation = xerces.fromXML(s, 0);
#endif

    if (annotation) {
      attributes = annotation -> attributes;
      children   = annotation -> children;
      name       = annotation -> name;

      annotation -> attributes.clear();
      annotation -> children.clear();
      delete annotation;
    }
  }
*/


  /* Output the Annotation.  This is done by writing out the
   * attributes and recursively outputting the children.  Note that we
   * need only emit the namespace definition for the root element.  We
   * can identify the root element by checking the name: if it is
   * equal to the name used by SBML to identify an annotation, we have
   * found the root.  Note that this means that MesoRD can never make
   * use of tags that are called annotation (or whatever the value of
   * Identifier::SBML_annotation is currently set to).
   */
namespace MesoRD
{
  std::ostream&
  operator<<(std::ostream& os, const MesoRD::Annotation& a)
  {
    std::map<std::string, std::string>::const_iterator i;
    std::vector<const Annotation*>::const_iterator     j;
    OPrefixStream opfs_level0(os.rdbuf(),          "");
    OPrefixStream opfs_level1(opfs_level0.rdbuf(), "  ");

    if (a.name == Identifier::SBML_annotation) {
      opfs_level0 << "<" << Identifier::SBML_annotation << " xmlns:"
                  << Identifier::XML::xmlns_identifier  << "=\""
                  << Identifier::XML::xmlns             << "\">"
                  << std::endl;
    } else {
      size_t name_length = a.name.length()
        + Identifier::XML::xmlns_identifier.length() + 3;

      opfs_level0 << "<"
                  << Identifier::XML::xmlns_identifier << ":" << a.name
                  << " ";
      for (i = a.attributes.begin(); i != a.attributes.end(); ) {
        opfs_level0 << Identifier::XML::xmlns_identifier << ":"
                    << i -> first
                    << "=\""
                    << i -> second
                    << "\"";
        if (++i != a.attributes.end()) {
          opfs_level0 << std::endl;
          for (size_t k = 0; k < name_length; k++)
            opfs_level0 << " ";
        }
      }
      opfs_level0 << ">" << std::endl;
    }

    for (j = a.children.begin(); j != a.children.end(); j++)
      opfs_level1 << **j;

    if (a.name == Identifier::SBML_annotation) {
      opfs_level0 << "</" << Identifier::SBML_annotation << ">" << std::endl;
    } else {
      opfs_level0 << "</"
                  << Identifier::XML::xmlns_identifier << ":" << a.name
                  << ">" << std::endl;
    }
    return os;
  }
}
