/* The CubicSubVolume class.
 *
 * Copyright 2004-2005 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: CubicSubVolume.cpp 541 2008-04-12 18:52:41Z davidfange $
 */
#include <iostream>
#include <map>
#include <vector>

#include "Compartment.hpp"
#include "CubicSubVolume.hpp"
#include "csg/CSGPoint.hpp"



  /* This code should actually be very similar to the code for
   * executing diffusion events.  Note that we want to ensure that
   * reactions that have zero probability of happening do not actually
   * happen.
   */
MesoRD::SubVolume*
MesoRD::SubVolume::eventNeighbourReaction(const double rand)
{
  if (R_neighbour_reaction > 0) {
    double       R_cum    = 0;
    const double rand_tot = rand * R_neighbour_reaction;
    
    R_neighbour_reaction = 0; 
    for( size_t i = 0;  i < neighbours_array.size(); i++) {
      std::valarray<double> rates = 
        compartment -> reactionTable.getRatesArray(*getSpeciesArray(), *(neighbours_array[i] -> getSpeciesArray()));
      
      for( unsigned int j = 0; j < rates.size(); j++) {
        R_cum += rates[j]/(2*((double) (neighbours_array.size() + 1)));
          
        if (rates[j] > 0 && rand_tot <= R_cum) {
          std::valarray<int> change =
            compartment -> reactionTable.getSpeciesChangeVector(static_cast<unsigned int>(floor(static_cast<double>(j/2))));
          
          std::valarray<unsigned int> products = compartment -> reactionTable.getProducts(static_cast<unsigned int>(floor(static_cast<double>(j/2))));
          std::valarray<unsigned int> reactants = compartment -> reactionTable.getReactants(static_cast<unsigned int>(floor(static_cast<double>(j/2))));
            
          if( j % 2 == 0 ) {
            neighbours_array[i] -> changeSpecies(reactants[0], change[reactants[0]]);
            changeSpecies(reactants[1], change[reactants[1]]);
            changeSpecies(products[0], change[products[0]]);
            return neighbours_array[i];
          } else {
            neighbours_array[i] -> changeSpecies(reactants[1], change[reactants[1]]);
            changeSpecies(reactants[0], change[reactants[0]]);
            changeSpecies(products[0], change[products[0]]);
            return neighbours_array[i];
          }
        }
      }
    }


    std::valarray<double> rates = 
      compartment -> reactionTable.getRatesArray(*getSpeciesArray(), *getSpeciesArray());  

    for( unsigned int j = 0; j < rates.size(); j++) {
      R_cum += rates[j]/(2*((double) (neighbours_array.size() + 1)));
      
      if (rates[j] > 0 && rand_tot <= R_cum) {
        std::valarray<int> change =
          compartment -> reactionTable.getSpeciesChangeVector((unsigned int)floor(j/2));
        
        std::valarray<unsigned int> products  = compartment -> reactionTable.getProducts(static_cast<unsigned int>(floor(static_cast<double>(j/2))));
        std::valarray<unsigned int> reactants = compartment -> reactionTable.getReactants(static_cast<unsigned int>(floor(static_cast<double>(j/2))));
        
        changeSpecies(reactants[0], change[reactants[0]]);
        changeSpecies(reactants[1], change[reactants[1]]);
        changeSpecies(products[0], change[products[0]]);
        return 0;
      } 
    }
  }
  return 0;
  
}


  /*****************************
   * DIFFUSION VECTOR UPDATING *
   *****************************/

  /* FIXME!! Hattne doesn't understand this!
   */
/*
void
MesoRD::OctahedricSubvolume::updateDiffusionVectorDeterministic()
{
  for (size_t i = 0; i < species.real -> size(); i++) {
    const double speciesNumber = (*species.real)[i] / (size * size);

    for (size_t j = 0; j < neighbours; j++) {
      const double temp_diffusion_rate = compartment
        -> getDiffusion(i, neighbours_array[j] -> getCompartment() -> getId())
        * speciesNumber;

      diffusion_vector[i] -= temp_diffusion_rate;
      neighbours_array[j] -> addDiffusionRate(i, temp_diffusion_rate);
    }
  }
}
*/


  /*************
   * DIFFUSION *
   *************/

  /* We now know that a neighbour can never be zero.  If a subvolume
   * lacks a neighbour in a certain direction, the neighbour must be
   * set up to point to itself.  Should a neighbour nevertheless be
   * <code>null</code>, it is an error which will lead to a
   * segmentation fault.  We do not try to catch that here, because
   * the error is severe enough to motivate complete failure of the
   * program.  Note that we want to ensure that diffusions that have
   * zero probability of happening do not actually happen.
   */
MesoRD::SubVolume*
MesoRD::CubicSubVolume::eventDiffusion(const double rand) 
{
  if (R_diffusion > 0.0) {
    double       D_cum    = 0.0;
    const double rand_tot = rand * R_diffusion * (size * size);

    for (size_t i = 0; i < species.integer -> size(); i++) {
      const unsigned int s = (*species.integer)[i];

      if (s > 0) {
        for (size_t j = 0; j < neighbours; j++) {
          const double D  = static_cast<double>(s)
            * compartment -> getDiffusion(i,
              neighbours_array[j] -> getCompartment() -> getId());
          D_cum          += D;

          if (D > 0.0 && rand_tot <= D_cum) {
            CubicSubVolume* target =
              static_cast<CubicSubVolume*>(neighbours_array[j]);

            changeSpecies(i, int(-1));
            target -> changeSpecies(i, int(+1));
            return target;
          }
        }
      }
    }
  }
  return 0;
}


  /* Looping over all subvolumes calling this function for each
   * species, you will be able to extract a matrix from
   * diffusionMatrixData, diffusionMatrix(Row/Col)Index.  The vectors
   * can be seen as a coordinate compressed sparse matrix.
   */
void
MesoRD::CubicSubVolume::getDiffusionVector(off_t index,
  std::map<CSGPoint, unsigned int>& subVolumeMap,
  std::vector<double>& diffusionMatrixData,
  std::vector<size_t>& diffusionMatrixRowIndex,
  std::vector<size_t>& diffusionMatrixColIndex,
  int numberOfSpecies)
{
  double temp_diffusion_rate = 0.0;
  double acumulatedDiffusionRate = 0.0;

  double sizeSquared = size * size;
  int subVolumeIndex = subVolumeMap[CSGPoint(
                                      this -> getX(),
                                      this -> getY(),
                                      this -> getZ(), 0.5, 0.5, 0.5)];
  int neighbourIndex = -1;

  diffusionMatrixColIndex.push_back(diffusionMatrixRowIndex.size());

    /* Looping over all the neighbours and picking out their indices.
     * Theses indices are used to construct the diffusion matrix.
     */
  for (size_t j = 0; j < neighbours; j++) {
    neighbourIndex = subVolumeMap[
      CSGPoint(neighbours_array[j] -> getX(),
        neighbours_array[j] -> getY(),
        neighbours_array[j] -> getZ(), 0.5, 0.5, 0.5)];

      /* Only non-self neighbours make a contribution to the diffusion
       * matrix.
       */
    if(subVolumeIndex != neighbourIndex) {
      temp_diffusion_rate = (compartment
        -> getDiffusion(index,
          neighbours_array[j] -> getCompartment() -> getId())) / sizeSquared;

      acumulatedDiffusionRate -= temp_diffusion_rate;

      diffusionMatrixData.push_back(temp_diffusion_rate);
      diffusionMatrixRowIndex.push_back(
        neighbourIndex * numberOfSpecies + index);
    }
    neighbourIndex = -1;
  }

  diffusionMatrixRowIndex.push_back(subVolumeIndex * numberOfSpecies + index);
  diffusionMatrixData.push_back(acumulatedDiffusionRate);
}
