/* The SymbolTable class.
 *
 * Copyright 2004-2005 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: SymbolTable.cpp 541 2008-04-12 18:52:41Z davidfange $
 */
#include "SymbolTable.hpp"



  /* For technical reasons one should always provide a non-inline
   * implementation for virtual destructors, even if it is trivial.
   * This will make sure that the compiler generates a vtable.
   * Without a vtable, there will be unresolved symbols when it comes
   * to linking.
   */
MesoRD::SymbolTable::~SymbolTable()
{}


  /* Sort the list of symbols, in order to be able to merge these
   * lists efficiently later on.
   */
std::list<MesoRD::Symbol*>
MesoRD::SymbolTable::getLocalSymbolById(const std::string& id)
{
  std::list<Symbol*>                             l;
  std::map<std::string, Symbol*>::const_iterator i;

  for (i = symbols.begin(); i != symbols.end(); i++) {
    if (i -> second -> isId(id))
      l.push_back(i -> second);
  }

  l.sort();
  return l;
}


  /* Sort the list of symbols, in order to be able to merge these
   * lists efficiently later on.
   */
std::list<MesoRD::Symbol*>
MesoRD::SymbolTable::getLocalSymbolByName(const std::string& name)
{
  std::list<Symbol*>                             l;
  std::map<std::string, Symbol*>::const_iterator i;

  for (i = symbols.begin(); i != symbols.end(); i++) {
    if (i -> second -> isName(name))
      l.push_back(i -> second);
  }

  l.sort();
  return l;
}
