/* The Thread class.
 *
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: Thread.hpp 554 2008-05-25 19:55:15Z davidfange $
 */
#ifndef MESORD_THREAD_HPP
#define MESORD_THREAD_HPP 1

#include "mesord.hpp"

#ifdef WIN32
#  include <windows.h>
#endif

#ifdef HAVE_PTHREAD
#  include <pthread.h>
#  include <sched.h> // Write separate check in configure.ac
#  include <sys/errno.h>
#endif



extern "C" {
  typedef void* MESORD_THREAD_FUNCTION(void*);
}

namespace MesoRD {
  class GetOpt;
  class System;

  

    /**
     * Class for handling different threads in MesoRD.  
     *
     * Abstraction layer that is supposed to hide the fact that
     * different operating systems are solving the same task in
     * differently stupid ways.
     */
  class Thread {
  private:
    bool            running;
    bool            initialised_variable;

#ifdef HAVE_PTHREAD
    pthread_t       handle;
    pthread_attr_t  attributes;
    pthread_cond_t  initialised_condition;
    pthread_mutex_t initialised_mutex;
#endif

#ifdef WIN32
    HANDLE       handle;
	HANDLE       initialised_event;
	//    HANDLE       initialised_mutex;
    DWORD        id;
#endif
  
  public:
    Thread();
    virtual ~Thread();

#ifdef HAVE_PTHREAD
    virtual void cancel();
    virtual void create(MESORD_THREAD_FUNCTION function, void* arguments);
    virtual void* join();
    void setInitialised(bool initialised);
    void waitForInitialisation();
#endif

#ifdef WIN32
    virtual DWORD cancel();
    virtual void create(LPTHREAD_START_ROUTINE function, LPVOID arguments);
    virtual DWORD join();
    void setInitialised(bool initialised);
    void waitForInitialisation();
#endif
  };


    /* ThreadProgress class.
     */
  class ThreadProgress : public Thread {
  public:
    ThreadProgress();
    ~ThreadProgress();

#ifdef HAVE_PTHREAD
    void createProgress(bool graphic, System* system);
#endif
#ifdef WIN32
    void createProgress(
      HINSTANCE hInstance, HWND hwnd, int nCmdShow, System* system);
#endif
  };


    /* ThreadSimulator class. 
	* !! FIXME Pending removal, no need for a simulator thread.
     */
  /*
  class ThreadSimulator : public Thread {
  public:
    ThreadSimulator();
    ~ThreadSimulator();

#ifdef HAVE_PTHREAD
    void createSimulator(System* system);
#endif
#ifdef WIN32
    void createSimulator(System* system);
#endif
  };
*/

    /* ThreadVisualiser class.
     */
#if HAVE_VIEWER
  class ThreadVisualiser : public Thread {
  public:
    ThreadVisualiser();
    ~ThreadVisualiser();

#ifdef HAVE_PTHREAD
    void createVisualiser(GetOpt* getOpt, System* system);
#endif
#ifdef WIN32
    void createVisualiser(HINSTANCE hInstance, HWND hwnd, int nCmdShow,
      GetOpt* getOpt, System* system);
#endif
  };
#endif
    
}

#endif
