/* The UnitTable class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: UnitTable.cpp 558 2008-09-06 15:52:52Z davidfange $
 */
#include <iostream>

#include "definition.hpp"
#include "UnitTable.hpp"

#include "ui/OPrefixStream.hpp"


  /* Try to convert every unit in the table to metres.  If it works,
   * we have found a length unit.
   */
std::list<std::string>
MesoRD::UnitTable::getLengthUnits() const
{
  std::list<std::string> lengths;
  std::map<std::string, Symbol*>::const_iterator i;

  lengths.clear();
  for (i = symbols.begin(); i != symbols.end(); i++) {
    try {
      Unit* u = static_cast<Unit*>(i -> second);
      u -> toUnit(Unit::metre());
      lengths.push_back(u -> getKey());
    } catch(MesoRD::Unit::InvalidConversion&) {
      ;
    }
  }

  return lengths;
}


  /* Look for the unit in the unit table and return it if found.
   * Return note that the otherwise.
   */
MesoRD::Unit*
MesoRD::UnitTable::getUnit(const std::string& key) const
{
  std::map<std::string, Symbol*>::const_iterator i;

  if (key.length() <= 0) {
    return 0;

  } else if ((i = symbols.find(key)) != symbols.end()) {
    return new Unit(*static_cast<Unit*>(i -> second));

  } else {
    const ::Unit             u    = ::Unit(key);
    if( u.getKind() == UNIT_KIND_INVALID) {
      return 0;
    }
    else{
      UnitDefinition ud = UnitDefinition();
      ud.addUnit(&u);
      
      return new Unit(&ud);
    }
  }
  return 0;
}


void
MesoRD::UnitTable::fromSBML(const ListOfUnitDefinitions* uds)
{
  for (unsigned int i = 0; i < uds -> size(); i++) {
    Unit *u = new Unit(dynamic_cast<const UnitDefinition*>(uds -> get(i)));
    u -> fold();
    symbols[u -> getKey()] = u;
  }
}

  /* Output operator.  Should be valid SBML.
   */
namespace MesoRD {
  std::ostream&
  operator<<(std::ostream& os, const UnitTable& ut) {
    OPrefixStream opfs_level0(os.rdbuf(),          "");
    OPrefixStream opfs_level1(opfs_level0.rdbuf(), "  ");
    std::map<std::string, Symbol*>::const_iterator i;

    opfs_level0 << "<listOfUnitDefinitions>" << std::endl;

    for (i = ut.symbols.begin(); i != ut.symbols.end(); i++)
      opfs_level1 << *static_cast<Unit*>(i -> second);

    opfs_level0 << "</listOfUnitDefinitions>" << std::endl;
 
    return os;
  }
}

