/* The CSGPoint class.
 *
 * Copyright 2004-2005 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: CSGPoint.cpp 541 2008-04-12 18:52:41Z davidfange $
 */

#include <math.h>

#include "CSGPoint.hpp"



  /***********************
   * STATIC DATA MEMBERS *
   ***********************/

  /* Initialise the static data members.  Apparently this is not
   * required on some compilers.  However, the ANSI standard says that
   * static data members should always be initialised.
   */
float MesoRD::CSGPoint::sloppy_x = 0.0;
float MesoRD::CSGPoint::sloppy_y = 0.0;
float MesoRD::CSGPoint::sloppy_z = 0.0;



  /************************
   * COMPARISON OPERATORS *
   ************************/

  /* Check the dimensions in turn.  Should all the if-statements fail,
   * the less-than test must fail, because the instances are in fact
   * equal.
   */
bool
MesoRD::CSGPoint::operator<  (const CSGPoint& c) const
{

  if (fabs(x - c.x) > sloppy_x) {
    if (x < c.x)
      return true;
    else
      return false;
  }
  
  if (fabs(y - c.y) > sloppy_y) {
    if (y < c.y)
      return true;
    else
      return false;
  }
  
  if (fabs(z - c.z) > sloppy_z) {
    if (z < c.z)
      return true;
    else
      return false;
  }
  
  return false;
}


  /* Check the dimensions in turn.  Should all the if-statements fail,
   * the greater-than test must fail, because the instances are in
   * fact equal.
   */
bool
MesoRD::CSGPoint::operator>  (const CSGPoint& c) const
{
  
  if (fabs(x - c.x) > sloppy_x) {
    if (x > c.x)
      return true;
    else
      return false;
  }
  
  if (fabs(y - c.y) > sloppy_y) {
    if (y > c.y)
      return true;
    else
      return false;
  }
  
  if (fabs(z - c.z) > sloppy_z) {
    if (z > c.z)
      return true;
    else
      return false;
  }
  
  return false;
}
