/* Definitions of global constants for MesoRD.
 *
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: definition.hpp 556 2008-09-01 18:38:28Z davidfange $
 */
#ifndef MESORD_DEFINITION_H
#define MESORD_DEFINITION_H 1

#include <string>

#include "mesord.hpp"

  /* For some strange reason, we seem to need to include the
   * <code>valarray</code> header before the windows.h header on
   * Windows systems.  Also note that the windows.h header in turn
   * must be included before the <code>GL</code> headers.
   */
#ifdef HAVE_GL
#  include <GL/gl.h> 
#endif
#ifdef HAVE_GLU
#  include <GL/glu.h> 
#endif
#ifdef HAVE_X
#ifdef HAVE_GL
#  include <GL/glx.h>
#endif
#  include <X11/Xlib.h>
#endif
#ifdef WIN32
#  define NOMINMAX
#  include <valarray>
#  include <windows.h>
#  include <windowsx.h>
#  include <GL/gl.h> 
#  include <GL/glu.h> 
#endif



// I think we should avoid including other files in this file.
/*
  #ifdef HAVE_VIEWER
  #  include "Visual.hpp"
  #endif
*/
namespace MesoRD
{
#ifdef HAVE_VIEWER
    /**
     * @brief The VisualColour struct.
     *
     * This struct defines a colour.  Colours are used to determine
     * how molecules are rendered.  Here, the RGB model is used, where
     * floating point values in the range @f$\left[ 0,\; 1 \right]@f$
     * encode the intensities of the respective components.  In
     * addition to the intensities, a colour has a name.  The name is
     * useful when the user is allowed to pick a colour from an
     * interface.  This struct is only available when the program is
     * compiled with support for the viewer.
     */
  typedef struct VisualColour
  {

      /**
       * @brief The red component of a colour.
       *
       * Should be in the range zero to one inclusive.  A negative
       * value means the colour is not defined.
       */
    float red;

      /**
       * @brief The green component of a colour.
       *
       * Should be in the range zero to one inclusive.  A negative
       * value means the colour is not defined.
       */
    float green;

      /**
       * @brief The blue component of a colour.
       *
       * Should be in the range zero to one inclusive.  A negative
       * value means the colour is not defined.
       */
    float blue;

      /**
       * @brief The name of the colour defined by the RGB triplet.
       *
       * The name of the colour should be descriptive of its
       * appearance.  Who would have guessed that <i>fuchsia</i> is a
       * colour?
       */
    std::string name;
  } VisualColour;



  const std::string VIEWER_WINDOW_TITLE = "MesoRD OpenGL";
  const int VIEWER_WINDOW_INITAL_HEIGHT = 500;
  const int VIEWER_WINDOW_INITAL_WIDTH  = 500;
  const int VIEWER_WINDOW_INITAL_X      = 100;
  const int VIEWER_WINDOW_INITAL_Y      = 100;
  const int VIEWER_INITIAL_SLAB_FACTOR  = 4;
  const int VIEWER_INITIAL_ZOOM_FACTOR  = 3;
  
  const float VIEWER_MOUSE_FACTOR_X = 0.5;
  const float VIEWER_MOUSE_FACTOR_Y = 0.5;
  const float VIEWER_MOUSE_FACTOR_Z = 0.5;
  
  const GLint VIEWER_ARROWHEAD_SLICES = 4;
  const GLint VIEWER_ARROWHEAD_STACKS = 1;
  const GLint VIEWER_SPHERE_SLICES    = 4;
  const GLint VIEWER_SPHERE_STACKS    = 2;
  
  const GLuint VIEWER_LIST_ARROWHEAD   = 1;
  const GLuint VIEWER_LIST_BOUNDINGBOX = 2;
  const GLuint VIEWER_LIST_SPHERE      = 3;
  const GLuint VIEWER_LIST_SUBVOLUME   = 4;
//  const GLuint VIEWER_LIST_WIRECUBE = 2;
//  const GLuint VIEWER_LIST_WIREOCTAHEDRON = 3;
  
  
  const GLfloat light[]        = {1.0, 1.0, 1.0, 0.0};
  
  const size_t MESORD_VIEWER_COLOURS = 16;
  
  /* The VC++ compiler thinks that all hardcoded
   * numbers are doubles. Thats why there so 
   * many explicit casts
   */
  const MesoRD::VisualColour MESORD_VIEWER_COLOUR[16] =
  {
    {(float)0.00, (float)1.00, (float)1.00, "Aqua"},
    {(float)0.00, (float)0.00, (float)0.00, "Black"},
    {(float)0.00, (float)0.00, (float)1.00, "Blue"},
    {(float)1.00, (float)0.00, (float)1.00, "Fuchsia"},
    {(float)0.53, (float)0.53, (float)0.53, "Gray"},
    {(float)0.00, (float)0.53, (float)0.00, "Green"},
    {(float)0.00, (float)1.00, (float)0.00, "Lime"},
    {(float)0.53, (float)0.00, (float)0.00, "Maroon"},
    {(float)0.00, (float)0.00, (float)0.53, "Navy"},
    {(float)0.53, (float)0.53, (float)0.00, "Olive"},
    {(float)0.53, (float)0.00, (float)0.53, "Purple"},
    {(float)1.00, (float)0.00, (float)0.00, "Red"},
    {(float)0.80, (float)0.80, (float)0.80, "Silver"},
    {(float)0.00, (float)0.53, (float)0.53, "Teal"},
    {(float)0.00, (float)0.00, (float)0.00, "White"},
    {(float)1.00, (float)1.00, (float)0.00, "Yellow"}
  };
#endif
  
#ifdef WIN32
  const CHAR szAppName[] = "Win OpenGL"; 
#endif
  
  
    /* For reactions
     */
  const int MESORD_REACTION_HASHTABLE_SIZE = 512;
//  const std::string MESORD_XML_ELEMENT_COMPARTMENT = "compartment";
//  const std::string MESORD_XML_COMPARTMENT_ID = "id";
  
  
    /* For output files.
     */
  const std::string MESORD_SPECIES_FILE_PREFIX = "species_";
  const std::string MESORD_SPECIES_FILE_SUFFIX = ".txt";
  const std::string MESORD_GEOMETRYFILE_NAME   = "geometry.txt";
  const std::string MESORD_TIMEFILE_NAME       = "time.txt";

    /* To locate reactions in compartments.
     */
//const std::string MESORD_XML_ELEMENT_LOCATION     = "location";
//const std::string MESORD_XML_REACTION_COMPARTMENT = "compartment";


    /* The different types of subvolumes MesoRD can deal with.
     *
     * Maybe this is not the right place for this enum.
     */
  typedef enum {
    cubic,
    octahedric
  } Subvolume_t;



    /**
     * @brief Physical constants <i>et al.</i>.
     *
     * The Constant namespace holds some physical constants that are
     * specific enough not to be defined in a compiler tool chain.
     * Come to think of it, there are systems that do not even define
     * those constants that <em>are</em> general enough to be defined
     * in a compiler tool chain.
     */
  namespace Constant
  {

      /**
       * @brief Avogadro's number, or Avogadro's constant,
       * @f$N_{A}@f$.
       *
       * This is the number of molecules in one mole.  Named after
       * Amedeo Avogadro.
       */
    const double Avogadro = 6.0221415E+23;


      /**
       * @brief Absolute zero in degrees Celsius.
       *
       * The number of degrees Kelvin that correspond to zero degrees
       * Celsius.
       */
    const double Celsius = 273.15;


      /**
       * @brief @f$\pi@f$, at least quite close...
       *
       * If <code>M_PI</code> is not defined in the system headers,
       * there is a fall-back definition in mesord.hpp.
       */
    const double Pi = M_PI;


      /**
       * @brief @f$e@f$, where @f$\ln \left( e \right) = 1@f$.
       *
       * If <code>M_E</code> is not defined in the system headers,
       * there is a fall-back definition in mesord.hpp.
       *
       * @todo Better named as just <code>e</code>?
       */
    const double Exponentiale = M_E;


      /**
       * @brief The number of iterations that should elapse between
       * subsequent updates of the progress bar in the deterministic
       * case.
       */
    const size_t iterations_progress_deterministic = 5;


      /**
       * @brief The number of iterations that should elapse between
       * subsequent updates of the progress bar in the stochastic
       * case.
       */
    const size_t iterations_progress_stochastic = 10000;
    
    const double P_intra_sv_reaction = 0.5;
    const double P_inter_sv_reaction = 0.5*0.5*((double)1/(double)6);
    
  }



    /**
     * @brief String constants commonly used throughout MesoRD.
     *
     * The Identifier namespace holds a whole bunch of string
     * constants that are used throughout MesoRD.  Most of them deal
     * with the parsing of XML annotations that can be present in SBML
     * base objects.
     */
  namespace Identifier
  {
    
      /**
       * @brief The name of an annotation field according to SBML.
       *
       * This is the tag name of an annotation field that may be
       * present in any SBML base object.  It should only need to be
       * changed if the SBML people decide to call their annotations
       * something else.
       */
    static const std::string SBML_annotation = "annotation";
    
    namespace XML
    {
      
        /**
         * @brief The character used to separate names from namespace
         * for the expat XML engine.
         *
         * The namespacing expat XML engine returns names of elements
         * as strings, concatenated with the appropriate namespace
         * identifier.  The constant below defines what character
         * expat should use in order to separate names from
         * namespaces.  It should be a single character that does not
         * otherwise occur in element names.
         */
      static const char expat_namespace_separator = ':';
      
        /**
         * @brief The name of the buffer for the Xerces-C++ XML
         * engine.
         *
         * This is used by the Xerces-C++ parser to identify the
         * memory buffer in which to do the parsing.  The exact
         * content is probably not so interesting, as long as it is
         * unique in some ill-defined sense.
         */
      static const std::string xercesc_buffer_name = "SBML_XML_Buffer";
      static const std::string xmlns               = "http://www.icm.uu.se";
      static const std::string xmlns_identifier    = "MesoRD";


      namespace Box 
      {
        const std::string name = "box";
        namespace Attributes
        {
          const std::string x = "x";
          const std::string y = "y";
          const std::string z = "z";
          const std::string units = "units";
        }
      }

      namespace Compartment
      {
        const std::string name = "compartment";
        const std::string id = "id";
      }
      
      namespace Cone
      {
        const std::string name = "cone";
        namespace Attributes
        {
          const std::string radius = "bottomradius";
          const std::string height = "height";
          const std::string units = "units";
        }
      }
      
      namespace Cylinder
      {
        const std::string name = "cylinder";
        namespace Attributes
        {
          const std::string radius = "radius";
          const std::string height = "height";
          const std::string units = "units";
        }
      }
      
      namespace Difference
      {
        const std::string name = "difference";
      }
      
      namespace Diffusion
      {
        const std::string name = "diffusion";
        namespace Attributes
        {
          const std::string compartment = "compartment";
          const std::string rate = "rate";
          const std::string units = "units";
        }
      }

      namespace Intersection
      {
        const std::string name = "intersection";
      }

      namespace Mesh
      {
        const std::string name  = "mesh";
        namespace Triangle
        {
          const std::string name = "triangle";
          namespace Vertex
          {
            const std::string name = "vertex";
            namespace Attributes
            {
              const std::string x = "x";
              const std::string y = "y";
              const std::string z = "z";
              const std::string units = "units";
            }
          }
        }
      }

      namespace PBC
      {
        const std::string name= "pbc";
        namespace Attributes
        {
          const std::string x = "x";
          const std::string y = "y";
          const std::string z = "z";
        }
      }
      
      namespace Rotation
      {
        const std::string name = "rotation";
        namespace Attributes
        {
          const std::string angle = "angle";
          const std::string x = "x";
          const std::string y = "y";
          const std::string z = "z";
          const std::string units = "units";
        }
      }
      
      namespace Scale
      {
        const std::string name = "scale";
        namespace Attributes
        {
          const std::string x = "x";
          const std::string y = "y";
          const std::string z = "z";
        }
      }
      
      namespace Shear
      {
        const std::string name = "shear";
        namespace Attributes
        {
          const std::string angle = "angle";
          const std::string x = "x";
          const std::string y = "y";
          const std::string z = "z";
          const std::string units = "units";
        }
      }

      namespace Sphere
      {
        const std::string name = "sphere";
        namespace Attributes
        {
          const std::string radius = "radius";
          const std::string units = "units";
        }
      }
      
      namespace Translation
      {
        const std::string name  = "translation";
        namespace Attributes
        {
          const std::string x = "x";
          const std::string y = "y";
          const std::string z = "z";
          const std::string units = "units";
        }
      }
      
      namespace Union
      {
        const std::string name = "union";
      }
      
      namespace InterSVAssnReactions
      {
        const std::string name = "inter_sv_assn_reaction";
        namespace Attributes
        {
          const std::string state = "state";
        }
      }
      
      namespace InterSVDissReactions
      {
        const std::string name = "inter_sv_diss_reaction";
        namespace Attributes
        {
          const std::string state = "state";
        }
      }
    }
  }


    /* Definitions of the available integration methods in the program.
     */
  const int MESORD_INTEGRATION_METHODS = 5;
  const std::string MESORD_INTEGRATION_METHOD[5] =
  { "Euler",
    "EulerBackward",
    "Trapeziod",
    "RungeKutta4",
    "BDF2"
  };
}
#endif
