/* The Euler class.
 *
 * Copyright 2004 Johan Elf,Johan Hattne
 * 
 * This file is part of MesoRD.  
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: Euler.cpp 541 2008-04-12 18:52:41Z davidfange $
 */


#include "Euler.hpp"
#include "../Derivative.hpp"

#include <string>
#include <stdio.h>
#include <iostream>

// from the sparselib++ package...
#include "../sparselib/comprow_double.hpp"
// from the mv++ package...
#include "../mv/mvvd.hpp"
#include "../mv/mvblasd.hpp"


MesoRD::Euler::Euler() 
  : Integration()
{}


MesoRD::Euler::Euler(Derivative* derivative, 
  CompRow_Mat_double* diffusionMatrix, 
  double timeStep, std::ostream* outstream_log) 
  : Integration(derivative, diffusionMatrix, timeStep, outstream_log)
{
    /* multiply each elemnt with the timestep */
  for(int i = 0; i < diffusionMatrix -> NumNonzeros(); i++){
    diffusionMatrix -> val(i) *= timeStep;
  }
  
}

MesoRD::Euler::~Euler()
{}


double
MesoRD::Euler::updateState(MV_Vector_double& currentState)
{
  rxnDerivativeVector*=0;
  derivative -> getReactionDerivative(currentState, rxnDerivativeVector);
  rxnDerivativeVector *= timeStep;
  
  currentState += 
    ((*diffusionMatrix*currentState) + rxnDerivativeVector);
  
  return timeStep;
}
