/*  The Euler Class.  
 *
 * Copyright 2004 Johan Elf,Johan Hattne
 * 
 * This file is part of MesoRD.  
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: Euler.hpp 541 2008-04-12 18:52:41Z davidfange $
 */
#ifndef __Euler_H
#define __Euler_H

#include "Integration.hpp"

#include "../Derivative.hpp"

#include "../sparselib/comprow_double.hpp"
#include "../mv/mvvd.hpp"

namespace MesoRD
{
  /** 
   * Euler Integration Class.    
   * 
   */
  class Euler : public Integration
  {   
    
  public:
    
    /**
     * Defualt constructor for the Integration class.  
     */
    Euler();
    
    /**
     * Value constructor for the Euler class.
     *
     * Sets the derivative object, the diffusion matrix and the timestep.    
     *
     * @param derivative A derivative object from which the system can 
     *                   be queried for reaction derivatives.  
     * @param diffusionMatrix Matrix specifying the diffusion in all 
     *                        directions.  
     * @param timeStep The timestep used.  
     */
    Euler(Derivative* derivative, CompRow_Mat_double* diffusionMatrix,  
      double timeStep, std::ostream* outstream_log);

    /** 
     * Destructor the the Integration class.  
     */
    ~Euler();
    
    /** Method used to update the current state vector.  
     */
    double updateState(MV_Vector_double& currentState);
    
  };
}

#endif





