/* The Integration class.
 *
 * Copyright 2004 Johan Elf,Johan Hattne
 * 
 * This file is part of MesoRD.  
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: Integration.cpp 541 2008-04-12 18:52:41Z davidfange $
 */


#include "Integration.hpp"
#include "../Derivative.hpp"

#include <string>
#include <stdio.h>
#include <iostream>

// from the sparselib++ package...
#include "../sparselib/compcol_double.hpp"
// from the mv++ package...
#include "../mv/mvvd.hpp"
#include "../mv/mvblasd.hpp"


MesoRD::Integration::Integration()
{
  timeStep = 0.0;

  diffusionMatrix = 0;
  derivative = 0;

}


MesoRD::Integration::Integration(Derivative* derivative, 
  CompRow_Mat_double* diffusionMatrix, 
  double timeStep, std::ostream* outstream_log)
{
  this -> timeStep = timeStep;
  this -> derivative = derivative;
  this -> diffusionMatrix = diffusionMatrix;
  this -> outstream_log = outstream_log;
  
  rxnDerivativeVector = MV_Vector_double(diffusionMatrix -> dim(0), (double)0);

}

MesoRD::Integration::~Integration()
{}



      /******************
       * STATIC METHODS *
       ******************/

const std::string MesoRD::Integration::IntegrationMethods[6] =
{ "Euler",
  "EulerBackward",
  "EulerAdaptive",
  "Trapezoid",
  "RungeKutta4",
  "BDF2"
};

const int MesoRD::Integration::NrOfIntMethods = 6;


MesoRD::Integration::IntegrationMethod MesoRD::Integration::intMethod = Euler;

bool
MesoRD::Integration::setIntegrationMethod(std::string intMethod)
{
  for(int i = 0; i < NrOfIntMethods; i++){
    if(IntegrationMethods[i] == intMethod){
      MesoRD::Integration::intMethod = (IntegrationMethod)i;
      return true;
    }
  }
  return false;
}

    

