/* The RungeKutta4 class.
 *
 * Copyright 2004 Johan Elf,Johan Hattne
 * 
 * This file is part of MesoRD.  
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: RungeKutta4.cpp 541 2008-04-12 18:52:41Z davidfange $
 */


#include "RungeKutta4.hpp"
#include "../Derivative.hpp"

#include <string>
#include <stdio.h>
#include <iostream>

// from the sparselib++ package...
#include "../sparselib/comprow_double.hpp"
// from the mv++ package...
#include "../mv/mvvd.hpp"
#include "../mv/mvblasd.hpp"


MesoRD::RungeKutta4::RungeKutta4() 
  : Integration()
{}


MesoRD::RungeKutta4::RungeKutta4(Derivative* derivative, 
  CompRow_Mat_double* diffusionMatrix, 
  double timeStep, std::ostream* outstream_log) 
  : Integration(derivative, diffusionMatrix, timeStep, outstream_log)
{

    /* allocating space for all the vectors needed by
     * the Runge-Kutta method.  
     */
  k1 = MV_Vector_double(diffusionMatrix -> dim(0));
  k2 = MV_Vector_double(diffusionMatrix -> dim(0));
  k3 = MV_Vector_double(diffusionMatrix -> dim(0));
  k4 = MV_Vector_double(diffusionMatrix -> dim(0));
  
  k1half = MV_Vector_double(diffusionMatrix -> dim(0));
  k2half = MV_Vector_double(diffusionMatrix -> dim(0));
  k3half = MV_Vector_double(diffusionMatrix -> dim(0));
  
    /* multiply each elemnt with the timestep */
  for(int i = 0; i < diffusionMatrix -> NumNonzeros(); i++){
    diffusionMatrix -> val(i) *= timeStep;
  }
  
}

MesoRD::RungeKutta4::~RungeKutta4()
{}


double
MesoRD::RungeKutta4::updateState(MV_Vector_double& currentState)
{
  rxnDerivativeVector*=0;
  derivative -> getReactionDerivative(currentState, rxnDerivativeVector);
  rxnDerivativeVector *= timeStep;
   
  k1 = *diffusionMatrix*currentState + rxnDerivativeVector;
  
  k1half = k1*(1.0/2.0) + currentState;
  
  rxnDerivativeVector*=0;
  derivative -> getReactionDerivative(k1half, rxnDerivativeVector);
  rxnDerivativeVector *= timeStep;

  k2 = *diffusionMatrix*(currentState + k1half) + rxnDerivativeVector;
  
  k2half = k2*(1.0/2.0) + currentState;

  rxnDerivativeVector*=0;
  derivative -> getReactionDerivative(k2half, rxnDerivativeVector);
  rxnDerivativeVector *= timeStep;
  
  k3 = *diffusionMatrix*(currentState + (k2half))+ rxnDerivativeVector;
  
  k3half = currentState + k3;

  rxnDerivativeVector*=0;
  derivative -> getReactionDerivative(k3half, rxnDerivativeVector);
  rxnDerivativeVector *= timeStep;
  
  k4 = *diffusionMatrix*(k3) + rxnDerivativeVector;
  
  currentState += ((k1 + (k2 * 2) + (k3 * 2) + k4) * (1.0/6.0));
 
  
  return timeStep;
}
