/* Defines and such for MesoRD.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: mesord.hpp 560 2008-11-09 11:12:14Z davidfange $
 */
#ifndef MESORD_MESORD_HPP
#define MESORD_MESORD_HPP 1

  /**
   * @mainpage MesoRD -- Mesoscopic Reaction--Diffusion Simulator
   *
   * @section sec_intro This is the mainpage
   *
   * And this is a dummy introductory section.
   */

  /**
   * @todo Make sure to define WIN32 in autotools
   */
#if HAVE_CONFIG_H //|| WIN32
#  include "../config.h"
#endif

 /* For compilation on Windows using the Microsoft Visual C++
   * compiler, replace calls to deprecated functions with calls to the
   * new, secure versions of those functions by defining
   * _CRT_SECURE_CPP_OVERLOAD STANDARD_NAMES as 1.  Turn off warnings
   * about any remaining deprecated functions by defining
   * _CRT_SECURE_NO_WARNINGS as 1.  Additionally, turn off the
   * "identifier was truncated to '255' characters in the debug
   * information" warning.  These preprocessor directives are needed
   * before the inclusion of any STL headers in order to be useful.
   */
#ifdef _MSC_VER
#  if  _MSC_VER > 1000 && !defined(__INTEL_COMPILER)
#    ifndef _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES
#	   define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#    endif
#    define _CRT_SECURE_NO_WARNINGS                 1
#    pragma warning(push)
#    pragma warning(once:4786)
#    pragma warning(once:4996)
#  endif
#endif


#ifdef WIN32
#  define HAVE_VIEWER 1
#  define NOMINMAX
#  include "windows.h"
#endif

#ifdef WIN32
#  define strcasecmp  _stricmp
#  define strncasecmp _strnicmp
#endif

#ifdef WIN32
#pragma comment (linker, "/SUBSYSTEM:CONSOLE")
//#pragma comment (linker, "/SUBSYSTEM:WINDOWS")
#pragma comment (linker, "/ENTRY:mainCRTStartup")
#endif

  /* For compilation using the Intel C++ compiler, turn off the "value
   * copied to temporary, reference to temporary used" and "operands
   * are evaluated in unspecified order" warnings.
   */
#ifdef __INTEL_COMPILER
#  if  __INTEL_COMPILER >= 800
#    pragma warning(push)
#    pragma warning(disable:383)
#    pragma warning(disable:981)
#  endif
#endif



#include <string>



#ifndef M_E
  /**
   * @f$e@f$, where @f$\ln \left( e \right) = 1@f$.
   *
   * Fall-back definition, if the system headers does not define
   * <code>M_E</code>.
   */
#  define M_E 2.71828182845904523536
#endif



#ifndef M_PI
  /**
   * @f$\pi@f$, at least quite close...
   *
   * Fall-back definition, if the system headers does not define
   * <code>M_PI</code>.
   */
#  define M_PI 3.1415926535897932384626433832795
#endif



#if defined(WIN32) \
  || (!defined(HAVE_RINT) && defined(HAVE_CEIL) && defined(HAVE_FLOOR))
  /**
   * @brief Fall-back definition for <code>rint(3)</code>.
   *
   * In case there is no <code>rint(3)</code> function, this is a
   * fall-back definition using <code>ceil(3)</code> and
   * <code>floor(3)</code>.  The <code>rint(3)</code> function returns
   * its argument rounded to the nearest integral value <em>towards
   * infinity</em>.  The rounding mode is <em>not</em> honoured.
   */
#  include <cmath>
#  define rint(x) (x < 0 ? ceil((x) - 0.5) : floor((x) + 0.5))
#endif

#if defined(WIN32) // || !defined(HAVE_LRINT)
#  define lrint(x) static_cast<long>(rint(x))
#endif


#if defined(WIN32) // || !defined(HAVE_STRTOF)
#  define strtof(nptr, endptr) static_cast<float>(strtod(nptr, endptr))
#endif
namespace MesoRD
{
    /**
     * @brief Return the largest argument.
     *
     * VC++ 6.0 does not seem to follow the STL ANSI standard -- one
     * needs to use <code>std::_cpp_max()</code> and
     * <code>std:_cpp_min()</code> instead of the standard
     * <code>std::max()</code> and <code>std::min()</code> functions
     * respectively.  For portability, MesoRD code should call
     * MesoRD::min() and MesoRD::max() to work around this issue.
     *
     * @todo Would it be possible to extend the <code>std</code>
     * namespace instead so that one can continue to use
     * <code>std::max()</code> and <code>std::min()</code>?
     *
     * YES IT WOULD, so go do it!  FIXME!!
     *
     * @param a First argument to the max() function.
     * @param b Second argument to the max() function.
     * @return  @f$ \max \left( a,\; b \right) @f$.
     */
  template<class T>
  inline T
  max(const T& a, const T& b)
  {
#ifdef WIN32
    return std::_cpp_max(a, b);
#else
    return std::max(a, b);
#endif
  }


    /**
     * @brief Return the smallest argument.
     *
     * @copydoc MesoRD::max()
     *
     * @param a First argument to the min() function.
     * @param b Second argument to the min() function.
     * @return  @f$ \min \left( a,\; b \right) @f$.
     */
  template<class T>
  inline T
  min(const T& a, const T& b)
  {
#ifdef WIN32
    return std::_cpp_min(a, b);
#else
    return std::min(a, b);
#endif
  }
}



#ifdef WIN32
  /**
   * @brief <code>getLastErrorString()</code> implementation for
   * Windows.
   *
   * Because Windows has no <code>strerror()</code> function, one has
   * to roll his/her own.  Note that the default language is used in
   * the <code>FormatMessage()</code> call.
   *
   * @return The error string, in a C++ string.
   */
static inline std::string
getLastErrorString()
{
  std::string errorString = "";
  LPVOID      lpMsgBuf;

  if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER
        | FORMAT_MESSAGE_FROM_SYSTEM
        | FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        GetLastError(),
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR)&lpMsgBuf,
        0,
        NULL)) {
    errorString = std::string(static_cast<char*>(lpMsgBuf));
  }

  LocalFree(lpMsgBuf);
  return errorString;
}
#endif
#endif
