/* The ReactionNode class.
 *
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: ReactionFold.cpp 553 2008-05-09 13:28:22Z davidfange $
 */

#include <limits>
#include <stack>
#include <sstream>
#include <math.h>

#include "../mesord.hpp"

#include "ReactionNode.hpp"

#include "../ParameterTable.hpp"
#include "../SpeciesTable.hpp"



  /********************
   * CONSTANT FOLDING *
   ********************/

MesoRD::ReactionNode*
MesoRD::ReactionNodeAbs::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
      
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);
        
        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCeil::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
      
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);
        
        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeExp::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
      
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);
        
        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }
  
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeFactorial::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
      
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);
        
        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeFloor::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
      
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);
        
        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeLn::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeLog::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodePower::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeRoot::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeAnd::fold()
  throw(MesoRD::ReactionNode::InvalidUnit)
{
  bool                               v = true;
  std::list<ReactionNode*>::iterator i;
    
  
  try {
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);

    for (i = operands.begin(); i != operands.end(); ++i) {
      if (isConstant(*i)) {
        v = v && fabs((*i) -> evaluate(
                        std::valarray<unsigned int>((unsigned int)0, 0)))
          >= std::numeric_limits<double>::min();

        ReactionNode* o1 = (*i);
        operands.erase(i);
        --i;
        delete o1;
      }
    }
  } catch(InvalidUnit& i) {
    throw InvalidUnit(i.what());
  }
  
  if (operands.size() == 0 && v) {
    ReactionNodeConstant* rc =  new ReactionNodeConstant(
      static_cast<double>(true),
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return rc;
  } else if(operands.size() == 0 && !v) {
    ReactionNodeConstant* rc =  new ReactionNodeConstant(
      static_cast<double>(false),
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return rc;
  } else if(operands.size() != 0 && v) {
    operands.push_back(new ReactionNodeConstant(
        static_cast<double>(true),
        Unit::dimensionless(),
        parameterTable,
        speciesTable));
    return this;
  } else if(operands.size() != 0 && !v) {
    operands.push_back(new ReactionNodeConstant(
        static_cast<double>(true),
        Unit::dimensionless(),
        parameterTable,
        speciesTable));
    return this;
  }
  return this;
  
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeNot::fold()
  throw(MesoRD::ReactionNode::InvalidUnit)
{
  if (operand) {
    try {
      operand = refold(operand);

      if (isConstant(operand)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch(InvalidUnit& i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeOr::fold()
  throw(MesoRD::ReactionNode::InvalidUnit)
{
  bool                               v = false;
  std::list<ReactionNode*>::iterator i;
    
  try {
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);

    for (i = operands.begin(); i != operands.end(); ++i) {
      if (isConstant(*i)) {
        v = v || fabs((*i) -> evaluate(
                        std::valarray<unsigned int>((unsigned int)0, 0)))
          >= std::numeric_limits<double>::min();
        ReactionNode* o = (*i);
        operands.erase(i);
        --i;
        delete o;
        
      }
    }
  } catch(InvalidUnit& i) {
    throw InvalidUnit(i.what());
  }

  if (operands.size() == 0 && v) {
    ReactionNodeConstant* rc = new ReactionNodeConstant(
      static_cast<double>(true),
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return rc;
  } else if(operands.size() == 0 && !v) {
    ReactionNodeConstant* rc = new ReactionNodeConstant(
      static_cast<double>(false),
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return rc;
  } else if(operands.size() > 0 && v){
    operands.push_back(new ReactionNodeConstant(
        static_cast<double>(true),
        Unit::dimensionless(),
        parameterTable,
        speciesTable));
    return this;
  } else if(operands.size() > 0 && !v) {
    operands.push_back(new ReactionNodeConstant(
        static_cast<double>(false),
        Unit::dimensionless(),
        parameterTable,
        speciesTable));
    return this;
  }
  return this;
}


  /*Performs pairwise xor 
   */
MesoRD::ReactionNode*
MesoRD::ReactionNodeXor::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  std::list<ReactionNode*>::iterator i;
  std::list<ReactionNode*>::iterator j;
  std::stack<std::list<ReactionNode*>::iterator> stack_false;
  std::stack<std::list<ReactionNode*>::iterator> stack_true;
   
  try {
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);
  } catch( InvalidUnit i) {
    throw InvalidUnit(i.what());
  }
  
  for (i = operands.begin(); i != operands.end(); i++) {
    if (isConstant(*i)) {
      if (stack_true.size() > 0
        && fabs((*i) -> evaluate(
                  std::valarray<unsigned int>((unsigned int)0, 0)))
        < std::numeric_limits<double>::min()) {
        j = stack_true.top();
        stack_true.pop();
        operands.push_back(new ReactionNodeConstant(
                             (double)true,
                             Unit::dimensionless(),
                             parameterTable,
                             speciesTable));
        ReactionNode* o1 = (*i);
        ReactionNode* o2 = (*j);
        operands.erase(j);
        operands.erase(i, ++i);
        delete o1;
        delete o2;
        i--;
      } else if (stack_false.size() > 0
        && fabs((*i) -> evaluate(
                  std::valarray<unsigned int>((unsigned int)0, 0)))
        >= std::numeric_limits<double>::min()) {
        j = stack_false.top();
        stack_false.pop();
        operands.push_back(new ReactionNodeConstant(
                             (double)true,
                             Unit::dimensionless(),
                             parameterTable,
                             speciesTable));
        ReactionNode* o1 = (*i);
        ReactionNode* o2 = (*j);
        operands.erase(j);
        operands.erase(i, ++i);
        delete o1;
        delete o2;
        i--;
      } else if (stack_false.size() > 0
        && fabs((*i) -> evaluate(
                  std::valarray<unsigned int>((unsigned int)0, 0)))
        < std::numeric_limits<double>::min()) {
        j = stack_false.top();
        stack_false.pop();
        operands.push_back(new ReactionNodeConstant(
                             (double)false,
                             Unit::dimensionless(),
                             parameterTable,
                             speciesTable));
        ReactionNode* o1 = (*i);
        ReactionNode* o2 = (*j);
        operands.erase(j);
        operands.erase(i, ++i);
        delete o1;
        delete o2;
        i--;
      } else if (stack_true.size() > 0
        && fabs((*i) -> evaluate(
                  std::valarray<unsigned int>((unsigned int)0, 0)))
        >= std::numeric_limits<double>::min()) {
        j = stack_true.top();
        stack_true.pop();
        operands.push_back(new ReactionNodeConstant(
                             (double)false,
                             Unit::dimensionless(),
                             parameterTable,
                             speciesTable));
        ReactionNode* o1 = (*i);
        ReactionNode* o2 = (*j);
        operands.erase(j);
        operands.erase(i, ++i);
        delete o1;
        delete o2;
        i--;
      }
      
      else {
        if (fabs((*i) -> evaluate(
                   std::valarray<unsigned int>((unsigned int)0, 0)))
          >= std::numeric_limits<double>::min()) {
          stack_true.push(i);
        } else {
          stack_false.push(i); 
        }
      }
    }
  }

  if (operands.size() == 1 && isConstant(*(operands.begin()))) {
    if (fabs((*operands.begin()) -> evaluate(
               std::valarray<unsigned int>((unsigned int)0, 0)))
      < std::numeric_limits<double>::min()) {
      delete this;
      return new ReactionNodeConstant(
        (double)false,
        Unit::dimensionless(),
        parameterTable,
        speciesTable);

    } else {
      delete this;
      return new ReactionNodeConstant(
        (double)true,
        Unit::dimensionless(),
        parameterTable,
        speciesTable);
    }
  }

  return this;
}

  /* Algebraic simplification for the addition operation.  Implements
   * transformations "R1", "R2", "R7" and "R9" from Muchnick.
   * Always using the new operator when folding has been done, to ensure 
   * that refold will fold again.
   */
MesoRD::ReactionNode*
MesoRD::ReactionNodeAddition::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (left && right) {
    ReactionNode*            tmp;
    ReactionNodeAddition*    a;
    ReactionNodeConstant*    c1;
    ReactionNodeConstant*    c2;
    ReactionNodeSubtraction* s;
    
    try {
      left  = refold(left);
      right = refold(right);
      
        /* Transformation A1: t + c -> c + t
         */
      if (!(c1 = isConstant(left)) && (c2 = isConstant(right))) {
        tmp   = left;
        left  = right;
        right = tmp;
        
        left  = refold(left);
        right = refold(right);
        
        tmp = new ReactionNodeAddition(left,right, parameterTable, speciesTable);
        setRightChild(0);
        setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A2: c1 + (c2 + t) -> c1+c2 + t
         */
      if ((c1 = isConstant(left)) && (a = isAddition(right))
        && (c2 = isConstant(a -> getLeftChild()))) {
        
        try {
          tmp = new ReactionNodeConstant(
            (c1 -> getValue()) + (c2 -> getValue()),
            (c1 -> getUnit())  + (c2 -> getUnit()),
            parameterTable,
            speciesTable);
          delete left;
          left = tmp;
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }
        
        tmp = a -> getRightChild();
        a -> setRightChild(0);
        delete right;
        right = tmp;
        
        left  = refold(left);
        right = refold(right);
        
        tmp = new ReactionNodeAddition(left,right, parameterTable, speciesTable);
        setRightChild(0);
        setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A3: c1 + (c2 - t) -> c1+c2 - t
         */
      if ((c1 = isConstant(left)) && (s = isSubtraction(right))
        && (c2 = isConstant(s -> getLeftChild()))) {
        
        try {
          tmp = new ReactionNodeSubtraction(
            new ReactionNodeConstant(
              (c1 -> getValue()) + (c2 -> getValue()),
              (c1 -> getUnit())  + (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            s -> getRightChild(), parameterTable, speciesTable);
        
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }

        s -> setRightChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A4: c1 + (t - c2) -> c1-c2 + t
         */
        /* THIS NODE WILL NEVER BE CALLED DUE TO S1 */
      if ((c1 = isConstant(left)) && (s = isSubtraction(right))
        && (c2 = isConstant(s -> getRightChild()))) {
        
        try {
          tmp = new ReactionNodeAddition(
            new ReactionNodeConstant(
              (c1 -> getValue()) - (c2 -> getValue()),
              (c1 -> getUnit())  - (c2 -> getUnit()),
              parameterTable, speciesTable),
            s -> getLeftChild(), parameterTable, speciesTable);
          
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }
        
        s -> setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A5: (c + t1) + t2 -> c + (t2 + t1)
         */
      if ((a = isAddition(left)) && (isConstant(a -> getLeftChild()))) {
        tmp = a -> getLeftChild();
        a -> setLeftChild(right);
        right = tmp;
        
        tmp   = left;
        left  = right;
        right = tmp;
        
        left  = refold(left);
        right = refold(right);
        
      
        tmp = new ReactionNodeAddition(left,right, parameterTable, speciesTable);
        setRightChild(0);
        setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A6: t1 + (c + t2) -> c + (t1 + t2)
         */
      if ((a = isAddition(right)) && (isConstant(a -> getLeftChild()))) {
        tmp = a -> getLeftChild();
        a -> setLeftChild(left);
        left = tmp;
        
        left  = refold(left);
        right = refold(right);
        
        tmp = new ReactionNodeAddition(left,right, parameterTable, speciesTable);
        setRightChild(0);
        setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A7: (c - t1) + t2 -> c + (t2 - t1)
         */
      if ((s = isSubtraction(left)) && (isConstant(s -> getLeftChild()))) {
        tmp = s -> getLeftChild();
        s -> setLeftChild(right);
        right = tmp;
        
        tmp   = left;
        left  = right;
        right = tmp;
        
        left  = refold(left);
        right = refold(right);
        
        tmp = new ReactionNodeAddition(left,right, parameterTable, speciesTable);
        setRightChild(0);
        setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A8: (t1 - c) + t2 -> -c + (t1 + t2)
         */
        /* THIS WILL NEVER BE CALLED DUE TO S1 */
      if ((s = isSubtraction(left)) && (c1 = isConstant(s -> getRightChild()))) {
        tmp = new ReactionNodeAddition(
          new ReactionNodeConstant(
            -1.0 * (c1 -> getValue()),
            c1 -> getUnit(),
            parameterTable,
            speciesTable),
          new ReactionNodeAddition(
            s -> getLeftChild(), right, parameterTable, speciesTable),
          parameterTable, speciesTable);
        
        s -> setLeftChild(0);
        right = 0;
        delete this;
        return tmp;
      }
      
        /* Transformation A9: t1 + (c - t2) -> c + (t1 - t2)
         */
      if ((s = isSubtraction(right)) && (isConstant(s -> getLeftChild()))) {
        tmp = s -> getLeftChild();
        s -> setLeftChild(left);
        left = tmp;
        
        left  = refold(left);
        right = refold(right);
        
        tmp = new ReactionNodeAddition(left,right, parameterTable, speciesTable);
        setRightChild(0);
        setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A10: t1 + (t2 - c) -> -c + (t2 + t1)
         */
        /* THIS WILL NEVER BE CALLED DUE TO S1 */
      if ((s = isSubtraction(right)) && 
        (c1 = isConstant(s -> getRightChild()))) {
        tmp = new ReactionNodeAddition(
          new ReactionNodeConstant(
            -1.0 * (c1 -> getValue()),
            c1 -> getUnit(),
            parameterTable,
            speciesTable),
          new ReactionNodeAddition(
            s -> getLeftChild(), left, parameterTable, speciesTable),
          parameterTable, speciesTable);
        
        left = 0;
        s -> setLeftChild(0);
        delete this;
        return tmp;
      }
      
        /* Transformation A11: c1 + c2 = c1+c2
         */
      if ((c1 = isConstant(left)) && (c2 = isConstant(right))) {
        try {
          tmp = new ReactionNodeConstant(
            (c1 -> getValue()) + (c2 -> getValue()),
            (c1 -> getUnit())  + (c2 -> getUnit()),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }
      }

        /* Transformation A12: 0 + t -> t
         */
      if ((c1 = isConstant(left))
        && fabs(c1 -> getValue()) < std::numeric_limits<double>::min()) {
        tmp   = right;
        right = 0;
        delete this;
        return tmp;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeDivision::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (left && right) {
    ReactionNode*               tmp;
    ReactionNodeConstant*       c1;
    ReactionNodeConstant*       c2;
    ReactionNodeDivision*       d;
    ReactionNodeMultiplication* m;

    try {
      
      left  = refold(left);
      right = refold(right);

        /* Transformation D1: c1 / (c2 / t) -> c1/c2 * t
         */
      if ((c1 = isConstant(left)) && (d = isDivision(right))
        && (c2 = isConstant(d -> getLeftChild()))) {
        if (fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication(
            new ReactionNodeConstant(
              (c1 -> getValue()) / (c2 -> getValue()),
              (c1 -> getUnit())  / (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            d -> getRightChild(),
            parameterTable, 
            speciesTable);

          d -> setRightChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D2: c1 / (t / c2) -> c1*c2 / t
         */
      if ((c1 = isConstant(left)) && (d = isDivision(right))
        && (c2 = isConstant(d -> getRightChild()))) {

        tmp = new ReactionNodeDivision(
          new ReactionNodeConstant(
            (c1 -> getValue()) * (c2 -> getValue()),
            (c1 -> getUnit())  * (c2 -> getUnit()),
            parameterTable,
            speciesTable),
          d -> getLeftChild(),
          parameterTable,
          speciesTable);

      
        d -> setLeftChild(0);
        delete this;
        return tmp;
      }

        /* Transformation D3: c1 / (c2 * t) -> c1/c2 / t
         */
      if ((c1 = isConstant(left)) && (m = isMultiplication(right))
        && (c2 = isConstant(m -> getLeftChild()))) {

        
        if (fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeDivision(
            new ReactionNodeConstant(
              (c1 -> getValue()) / (c2 -> getValue()),
              (c1 -> getUnit())  / (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            m -> getRightChild(),
            parameterTable,
            speciesTable);
        
          m -> setRightChild(0);
          delete this;
          return tmp;
        
        
        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D4: (c1 / t) / c2 -> c1/c2 / t
         */
      if ((d = isDivision(left)) && (c1 = isConstant(d -> getLeftChild()))
        && (c2 = isConstant(right))) {

        if (fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeDivision(
            new ReactionNodeConstant(
              (c1 -> getValue()) / (c2 -> getValue()),
              (c1 -> getUnit())  / (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            d -> getRightChild(),
            parameterTable,
            speciesTable);
        
          d -> setRightChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D5: (t / c1) / c2 -> t / c1*c2
         */
      if ((d = isDivision(left)) && (c1 = isConstant(d -> getRightChild()))
        && (c2 = isConstant(right))) {
      
        if (fabs(c1 -> getValue()) >= std::numeric_limits<double>::min()
          && fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeDivision(
            d -> getLeftChild(),
            new ReactionNodeConstant(
              (c1 -> getValue()) * (c2 -> getValue()),
              (c1 -> getUnit())  * (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            parameterTable,
            speciesTable);   
        
          d -> setLeftChild(0);
          delete this;
          return tmp;
        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D6: (c1 * t) / c2 -> c1/c2 * t
         */
      if ((c2 = isConstant(right)) && (m = isMultiplication(left))
        && (c1 = isConstant(m -> getLeftChild()))) {

        if (fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication(
            new ReactionNodeConstant(
              (c1 -> getValue()) / (c2 -> getValue()),
              (c1 -> getUnit())  / (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            m -> getRightChild(), parameterTable, speciesTable);
        
          m -> setRightChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }
    
        /* Transformation D7: (c / t1) / t2 -> c / (t1 * t2)
         */
      if ((d = isDivision(left)) && (c1 = isConstant(d -> getLeftChild()))) {
        tmp = new ReactionNodeDivision(
          c1,
          new ReactionNodeMultiplication(
            d -> getRightChild(), 
            right, 
            parameterTable, 
            speciesTable),
          parameterTable,
          speciesTable);
      
        d -> setRightChild(0);
        setRightChild(0);
        setLeftChild(0);
      
        delete this;
        return tmp;
      }

        /* Transformation D8: (t1 / c) / t2 -> 1/c * (t1 / t2)
         */
      if ((d = isDivision(left)) && (c1 = isConstant(d -> getRightChild()))) {
        if (fabs(c1 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication(
            new ReactionNodeConstant(
              1 / (c1 -> getValue()),
              MesoRD::Unit::dimensionless() / (c1 -> getUnit()),
              parameterTable,
              speciesTable),
            new ReactionNodeDivision(
              d -> getLeftChild(), right, parameterTable, speciesTable),
            parameterTable, speciesTable);
        
          right = 0;
          d -> setLeftChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D9: t1 / (c / t2) -> 1/c * (t1 * t2)
         */
      if ((d = isDivision(right)) && (c1 = isConstant(d -> getLeftChild()))) {
        if (fabs(c1 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication(
            new ReactionNodeConstant(
              1 / (c1 -> getValue()),
              MesoRD::Unit::dimensionless() / (c1 -> getUnit()),
              parameterTable,
              speciesTable),
            new ReactionNodeMultiplication(
              left, d -> getRightChild(), parameterTable, speciesTable),
            parameterTable, speciesTable);

          left = 0;
          d -> setRightChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D10: t1 / (t2 / c) -> c * (t1 / t2)
         */
      if ((d = isDivision(right)) && (c1 = isConstant(d -> getRightChild()))) {
        tmp = new ReactionNodeMultiplication(
          d -> getRightChild(),
          new ReactionNodeDivision(
            left, d -> getLeftChild(), parameterTable, speciesTable),
          parameterTable, speciesTable);
      
        setLeftChild(0);
        setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation D11: (c * t1) / t2 -> c * (t1 / t2)
         */
      if ((m = isMultiplication(left))
        && (c1 = isConstant(m -> getLeftChild()))) {

        tmp = new ReactionNodeMultiplication(
          m -> getLeftChild(),
          new ReactionNodeDivision(
            m -> getRightChild(), right, parameterTable, speciesTable),
          parameterTable, speciesTable);
      
        right = 0;
        m -> setRightChild(0);
        m -> setLeftChild(0);
      
        delete this;
        return tmp;
      }

        /* Transformation D12: t1 / (c * t2) -> 1/c * (t1 / t2)
         */
      if ((m = isMultiplication(right))
        && (c1 = isConstant(m -> getLeftChild()))) {

        if (fabs(c1 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication(
            new ReactionNodeConstant(
              1 / (c1 -> getValue()),
              MesoRD::Unit::dimensionless() / (c1 -> getUnit()),
              parameterTable,
              speciesTable),
            new ReactionNodeDivision(
              left, m -> getRightChild(), parameterTable, speciesTable),
            parameterTable, speciesTable);
      
          setLeftChild(0);
          m -> setRightChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D13: c1 / c2 -> c1/c2
         */
      if ((c1 = isConstant(left)) && (c2 = isConstant(right))) {
        if (fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeConstant(
            (c1 -> getValue()) / (c2 -> getValue()),
            (c1 -> getUnit())  / (c2 -> getUnit()),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation D14: 0 / t -> 0
         */
      if ((c1 = isConstant(left))
        && fabs(c1 -> getValue()) < std::numeric_limits<double>::min()) {
        tmp = new ReactionNodeConstant(
          0.0,
          Unit::dimensionless(),
          parameterTable,
          speciesTable);
        delete this;
        return tmp;
      }

        /* Transformation D15: t / 0 -> DIVISION BY ZERO
         */
      if ((c1 = isConstant(right))
        && fabs(c1 -> getValue()) < std::numeric_limits<double>::min()) {
        tmp = new ReactionNodeConstant(
          std::numeric_limits<double>::infinity(),
          this -> getUnit(),
          parameterTable,
          speciesTable);
        delete this;
        return tmp;
      }
 
        /* Transformation D16: t / 1 -> t
         */
      if ((c1 = isConstant(right))
        && fabs(c1 -> getValue() - 1.0)
        < std::numeric_limits<double>::epsilon()
        && c1 -> getUnit() == MesoRD::Unit::dimensionless()) {
        tmp = left;
        left = 0;
        delete this;
        return tmp;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }
  
  return this;
}


  /* Algebraic simplification for the multiplication operation.
   * Implements transformations "R3", "R4", "R8" and "R10" through
   * "R20" from Muchnick.
   */
MesoRD::ReactionNode*
MesoRD::ReactionNodeMultiplication::fold()
  throw( MesoRD::ReactionNode::InvalidUnit)
{
  if (left && right) {
    ReactionNode*               tmp;
    ReactionNodeConstant*       c1;
    ReactionNodeConstant*       c2;
    ReactionNodeDivision*       d;
    ReactionNodeMultiplication* m;

    try {
      
      left  = refold(left);
      right = refold(right);

        /* Transformation M1: t * c -> c * t
         */
      if (!(c1 = isConstant(left)) && (c2 = isConstant(right))) {
        tmp = new ReactionNodeMultiplication(
          c2,
          left,
          parameterTable,
          speciesTable);

        setLeftChild(0);
        setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation M2: c1 * (c2 * t) -> c1*c2 * t
         */
      if ((c1 = isConstant(left)) && (m = isMultiplication(right))
        && (c2 = isConstant(m -> getLeftChild()))) {
      
        tmp = new ReactionNodeMultiplication(
          new ReactionNodeConstant(
            (c1 -> getValue()) * (c2 -> getValue()),
            (c1 -> getUnit())  * (c2 -> getUnit()),
            parameterTable,
            speciesTable),
          m -> getRightChild(),
          parameterTable, speciesTable);
      
        m -> setRightChild(0);
        delete this;
        return tmp;
        
      }

        /* Transformation M3: c1 * (c2 / t) -> c1*c2 / t
         */
      if ((c1 = isConstant(left)) && (d = isDivision(right))
        && (c2 = isConstant(d -> getLeftChild()))) {

        tmp = new ReactionNodeDivision(
          new ReactionNodeConstant(
            (c1 -> getValue()) * (c2 -> getValue()),
            (c1 -> getUnit())  * (c2 -> getUnit()),
            parameterTable,
            speciesTable),
          d -> getRightChild(), parameterTable, speciesTable);

        d -> setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation M4: c1 * (t / c2) -> c1/c2 * t
         */
      if ((c1 = isConstant(left)) && (d = isDivision(right))
        && (c2 = isConstant(d -> getRightChild()))) {

        if (fabs(c2 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication( 
            new ReactionNodeConstant(
              (c1 -> getValue()) / (c2 -> getValue()),
              (c1 -> getUnit())  / (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            d -> getLeftChild(),
            parameterTable,
            speciesTable);
        
          d -> setLeftChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }
    
        /* Transformation M5: (c * t1) * t2 -> c * (t2 * t1)
         */
      if ((m = isMultiplication(left)) && (c1 = isConstant(m -> getLeftChild()))) {
        tmp = new ReactionNodeMultiplication(
          c1,
          new ReactionNodeMultiplication(
            getRightChild(),
            m -> getRightChild(),
            parameterTable,
            speciesTable),
          parameterTable,
          speciesTable);
      
        m -> setLeftChild(0);
        m -> setRightChild(0);
        setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation M6: (c / t1) * t2 -> c * (t2 / t1)
         */
      if ((d = isDivision(left)) && (c1 = isConstant(d -> getLeftChild()))) {
        tmp = new ReactionNodeMultiplication(
          c1,
          new ReactionNodeDivision(
            getRightChild(),
            d -> getRightChild(),
            parameterTable,
            speciesTable),
          parameterTable,
          speciesTable);
      
        setRightChild(0);
        d -> setRightChild(0);
        d -> setLeftChild(0);
      
        delete this;
        return tmp;
      }

        /* Transformation M7: (t1 / c) * t2 -> 1/c * (t1 * t2)
         */
      if ((d = isDivision(left)) && (c1 = isConstant(d -> getRightChild()))) {
        if (fabs(c1 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication( 
            new ReactionNodeConstant(
              1 / (c1 -> getValue()),
              MesoRD::Unit::dimensionless() / (c1 -> getUnit()),
              parameterTable,
              speciesTable),
            new ReactionNodeMultiplication(
              d -> getLeftChild(), 
              right, 
              parameterTable, 
              speciesTable),
            parameterTable,
            speciesTable);
        
          d -> setLeftChild(0);
          setRightChild(0);
          delete this;
          return tmp;

        } else {
          tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }

        /* Transformation M8: t1 * (c * t2) -> c * (t1 * t2)
         */
      if ((m = isMultiplication(right)) && (c1 = isConstant(m -> getLeftChild()))) {
        tmp = new ReactionNodeMultiplication(
          c1,
          new ReactionNodeMultiplication(
            getLeftChild(),
            m -> getRightChild(),
            parameterTable,
            speciesTable),
          parameterTable,
          speciesTable);
      
        setLeftChild(0);
        m -> setLeftChild(0);
        m -> setRightChild(0);
      
        delete this;
        return tmp;
      }

        /* Transformation M9: t1 * (c / t2) -> c * (t1 / t2)
         */
      if ((d = isDivision(right)) && (c1 = isConstant(d -> getLeftChild()))) {
        tmp = new ReactionNodeMultiplication(
          c1,
          new ReactionNodeDivision(
            getLeftChild(),
            d -> getRightChild(),
            parameterTable,
            speciesTable),
          parameterTable,
          speciesTable);
      
        setLeftChild(0);
        d -> setLeftChild(0);
        d -> setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation M10: t1 * (t2 / c) -> 1/c * (t1 * t2)
         */
      if ((d = isDivision(right)) && (c1 = isConstant(d -> getRightChild()))) {
        if (fabs(c1 -> getValue()) >= std::numeric_limits<double>::min()) {
          tmp = new ReactionNodeMultiplication(
            new ReactionNodeConstant(
              1 / (c1 -> getValue()),
              MesoRD::Unit::dimensionless() / (c1 -> getUnit()),
              parameterTable,
              speciesTable),
            new ReactionNodeMultiplication(
              d -> getLeftChild(), 
              left, 
              parameterTable, 
              speciesTable),
            parameterTable,
            speciesTable);
        
          d -> setLeftChild(0);
          setLeftChild(0);
          delete this;
          return tmp;

        }

	tmp = new ReactionNodeConstant(
            std::numeric_limits<double>::infinity(),
            this -> getUnit(),
            parameterTable,
            speciesTable);
	delete this;
	return tmp;
      }

        /* Transformation M11: c1 * c2 -> c1*c2
         */
      if ((c1 = isConstant(left)) && (c2 = isConstant(right))) {
        tmp = new ReactionNodeConstant(
          (c1 -> getValue()) * (c2 -> getValue()),
          (c1 -> getUnit())  * (c2 -> getUnit()),
          parameterTable,
          speciesTable);
        delete this;
        return tmp;
      }

        /* Transformation M12: 0 * t -> 0
         */
      if ((c1 = isConstant(left))
        && fabs(c1 -> getValue()) < std::numeric_limits<double>::min()) {
        tmp = new ReactionNodeConstant(
          0.0,
          Unit::dimensionless(),
          parameterTable,
          speciesTable);
        delete this;
        return tmp;
      }

        /* Transformation M13: 1 * t -> t
         */
      if ((c1 = isConstant(left))
        && fabs(c1 -> getValue() - 1.0)
        < std::numeric_limits<double>::epsilon()
        &&  c1 -> getUnit() == MesoRD::Unit::dimensionless()) {
        tmp   = right;
        right = 0;
        delete this;
        return tmp;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


  /* Algebraic simplification for the addition operation.  Implements
   * transformations "R5" and "R6" from Muchnick.
   */
MesoRD::ReactionNode*
MesoRD::ReactionNodeSubtraction::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (left && right) {
    ReactionNode*            tmp;
    ReactionNodeAddition*    a;
    ReactionNodeConstant*    c1;
    ReactionNodeConstant*    c2;
    ReactionNodeSubtraction* s;

    try {
      
      left  = refold(left);
      right = refold(right);

        /* Transformation S1: t - c -> -c + t
         */
      if (!(c1 = isConstant(left)) && (c2 = isConstant(right))) {
        tmp   = left;
        left  = right;
        right = tmp;

        c2 -> setValue(-1.0 * (c2 -> getValue()));
        tmp = new ReactionNodeAddition(
          left, right, parameterTable, speciesTable);
        left  = 0;
        right = 0;
        delete this;
        return tmp;
      }

        /* Transformation S2: c1 - (c2 - t) -> c1-c2 + t
         */
      if ((c1 = isConstant(left)) && (s = isSubtraction(right))
        && (c2 = isConstant(s -> getLeftChild()))) {
        try {
          tmp = new ReactionNodeAddition(
            new ReactionNodeConstant(
              (c1 -> getValue()) - (c2 -> getValue()),
              (c1 -> getUnit())  - (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            s -> getRightChild(), parameterTable, speciesTable);
       
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }

        s -> setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation S3: c1 - (c2 + t) -> c1-c2 - t
         */
      if ((c1 = isConstant(left)) && (a = isAddition(right))
        && (c2 = isConstant(a -> getLeftChild()))) {
        try {
          tmp = new ReactionNodeSubtraction(
            new ReactionNodeConstant(
              (c1 -> getValue()) - (c2 -> getValue()),
              (c1 -> getUnit())  - (c2 -> getUnit()),
              parameterTable,
              speciesTable),
            a -> getRightChild(), parameterTable, speciesTable);
      
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }
      
        a -> setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation S4: t1 - (c - t2) -> -c + (t1 + t2)
         */
      if ((s = isSubtraction(right))
        && (c1 = isConstant(s -> getLeftChild()))) {

        tmp = new ReactionNodeAddition(
          new ReactionNodeConstant(
            -1.0 * (c1 -> getValue()),
            c1 -> getUnit(),
            parameterTable,
            speciesTable),
          new ReactionNodeAddition(
            left, s -> getRightChild(), parameterTable, speciesTable),
          parameterTable, speciesTable);
      
        left = 0;
        s -> setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation S5: t1 - (c + t2) -> -c + (t1 - t2)
         */
      if ((a = isAddition(right))
        && (c1 = isConstant(a -> getLeftChild()))) {

        tmp = new ReactionNodeAddition(
          new ReactionNodeConstant(
            -1.0 * (c1 -> getValue()),
            c1 -> getUnit(),
            parameterTable,
            speciesTable),
          new ReactionNodeSubtraction(
            left, a -> getRightChild(), parameterTable, speciesTable),
          parameterTable, speciesTable);
      
        left = 0;
        a -> setRightChild(0);
        delete this;
        return tmp;
      }

        /* Transformation S6: c1 - c2 -> c1-c2
         */
      if ((c1 = isConstant(left)) && (c2 = isConstant(right))) {
        try {
          tmp = new ReactionNodeConstant(
            (c1 -> getValue()) - (c2 -> getValue()),
            (c1 -> getUnit())  - (c2 -> getUnit()),
            parameterTable,
            speciesTable);
        
        } catch( MesoRD::Unit::InvalidConversion i) {
          std::stringstream message;
          message << "Unable to add unit \n"
                  << *(i.getFrom())
                  << "to \n"
                  << *(i.getTo());
          throw InvalidUnit((message.str()).c_str());
        }

        delete this;
        return tmp;
      }

        /* Transformation S7: 0 - t -> -1 * t
         */
      if ((c1 = isConstant(left))
        && fabs(c1 -> getValue()) < std::numeric_limits<double>::min()) {
        tmp = new ReactionNodeMultiplication(
          new ReactionNodeConstant(
            -1.0,
            Unit::dimensionless(),
            parameterTable,
            speciesTable),
          right, parameterTable, speciesTable);
      
        right = 0;
        delete this;
        return tmp;
      }

        /* Transformation S8: t - 0 -> t
         */
      if ((c1 = isConstant(right))
        && fabs(c1 -> getValue()) < std::numeric_limits<double>::min()) {
        tmp  = left;
        left = 0;
        delete this;
        return tmp;
      }
    
        /* Transformation S9: ( c + t1 ) - t2 -> c + ( t1 - t2 ) 
         */
      if((a = isAddition(left)) && (c1 = isConstant(a -> getLeftChild()))) {
        tmp = new ReactionNodeAddition(
          new ReactionNodeConstant(
            (c1 -> getValue()),
            c1 -> getUnit(),
            parameterTable,
            speciesTable),
          new ReactionNodeSubtraction(
            a -> getRightChild(), right, parameterTable, speciesTable),
          parameterTable, speciesTable);
      
        a -> setRightChild(0);
        right = 0;
      
        delete this;
        return tmp;
      }
    
        /* Transformation S10: ( c - t1 ) - t2 -> c - ( t1 + t2 )
         */
      if((s = isSubtraction(left)) && (c1 = isConstant(s -> getLeftChild()))) {

        tmp = new ReactionNodeSubtraction(
          new ReactionNodeConstant(
            (c1 -> getValue()),
            c1 -> getUnit(),
            parameterTable,
            speciesTable),
          new ReactionNodeAddition(
            s -> getRightChild(), 
            right,
            parameterTable, 
            speciesTable),
          parameterTable, speciesTable);
      
        s -> setRightChild(0);
        right = 0;
      
        delete this;
        return tmp;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeEq::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  bool                               first = true;
  double                             value = 0;
  std::list<ReactionNode*>::iterator i;
  
  try {
    
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);
    
    for (i = operands.begin(); i != operands.end(); ++i) {
      if (isConstant(*i)) {

        if (first) {
          value = (*i) -> 
            evaluate(std::valarray<unsigned int>((unsigned int)0, 0));
          first = false;
        
        } else if (
          fabs(value - (*i) ->
            evaluate(std::valarray<unsigned int>((unsigned int)0, 0)))
          >= std::numeric_limits<double>::epsilon()) {
            
          ReactionNode* tmp = new ReactionNodeConstant(
            (double)false,
            Unit::dimensionless(),
            parameterTable,
            speciesTable);
        
          delete this;
          return tmp;
        
        } else {
          operands.erase(i);
          --i;
        }
      }
    }
  } catch( InvalidUnit i) {
    throw InvalidUnit(i.what());
  }
  
      
  if (!first && operands.size() == 1) {
    ReactionNode* tmp = new ReactionNodeConstant(
      (double)true,
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return tmp;
  } 
    
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeGeq::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  std::list<ReactionNode*>::iterator i;
  std::list<ReactionNode*>::iterator j;

  try {
    
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);

    for (i = j = operands.begin(); i != operands.end(); j = i++) {
      
      if (i != j) {
        if (isConstant(*i) && isConstant(*j)) {
          if (
            ((*j) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0))) >= 
            ((*i) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0)))) {
            ReactionNode* tmp = (*j);
            operands.erase(j);
            delete tmp;
          } else {
            ReactionNode* tmp = new ReactionNodeConstant(
              (double)false,
              Unit::dimensionless(),
              parameterTable,
              speciesTable);
            delete this;
            return tmp;
          }
        }
      }
    }
  } catch( InvalidUnit i) {
    throw InvalidUnit(i.what());
  }
  
  if( operands.size() == 1 && isConstant(*(operands.begin()))) {
    ReactionNode* tmp = new ReactionNodeConstant(
      (double)true,
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return tmp;
  }
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeGt::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  std::list<ReactionNode*>::iterator i;
  std::list<ReactionNode*>::iterator j;
  
  try {
    
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);

    for (i = j = operands.begin(); i != operands.end(); j = i++) {
      if (i != j) {
        if (isConstant(*i) && isConstant(*j)) {
          if (
            ((*j) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0))) >
            ((*i) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0)))) {
            ReactionNode* tmp = (*j);
            operands.erase(j);
            delete tmp;
          } else {
            ReactionNode* tmp = new ReactionNodeConstant(
              (double)false,
              Unit::dimensionless(),
              parameterTable,
              speciesTable);
            delete this;
            return tmp;
          }
        }
      }
    }
  } catch( InvalidUnit i) {
    throw InvalidUnit(i.what());
  }
  
  if( operands.size() == 1 && isConstant(*(operands.begin()))) {
    ReactionNode* tmp = new ReactionNodeConstant(
      (double)true,
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return tmp;
  }
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeLeq::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  std::list<ReactionNode*>::iterator i;
  std::list<ReactionNode*>::iterator j;
  
  try {
    for (i = operands.begin(); i != operands.end(); i++)
      (*i)= refold(*i);
  } catch( InvalidUnit i) {
    throw InvalidUnit(i.what());
  }
  
  for (i = j = operands.begin(); i != operands.end(); j = i++) {
    if (i != j) {
      if (isConstant(*i) && isConstant(*j)) {
        if (
          ((*j) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0))) <=
          ((*i) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0)))) {
          ReactionNode* tmp = (*j);
          operands.erase(j);
          delete tmp;
        } else {
          ReactionNode* tmp = new ReactionNodeConstant(
            (double)false,
            Unit::dimensionless(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }
    }
  }
  if( operands.size() == 1 && isConstant(*(operands.begin()))) {
    ReactionNode* tmp = new ReactionNodeConstant(
      (double)true,
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return tmp;
  }
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeLt::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  std::list<ReactionNode*>::iterator i;
  std::list<ReactionNode*>::iterator j;
  
  try {
    for (i = operands.begin(); i != operands.end(); i++)
      (*i) = refold(*i);
  } catch( InvalidUnit i) {
    throw InvalidUnit(i.what());
  }
  

  for (i = j = operands.begin(); i != operands.end(); j = i++) {
    if (i != j) {
      if (isConstant(*i) && isConstant(*j)) {
        if (
          ((*j) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0))) <
          ((*i) -> evaluate(std::valarray<unsigned int>((unsigned int)0, 0)))) {
          ReactionNode* tmp = (*j);
          operands.erase(j);
          delete tmp;
        } else {
          ReactionNode* tmp = new ReactionNodeConstant(
            (double)false,
            Unit::dimensionless(),
            parameterTable,
            speciesTable);
          delete this;
          return tmp;
        }
      }
    }
  }
  if( operands.size() == 1 && isConstant(*(operands.begin()))) {
    ReactionNode* tmp = new ReactionNodeConstant(
      (double)true,
      Unit::dimensionless(),
      parameterTable,
      speciesTable);
    delete this;
    return tmp;
  }
  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeNeq::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (left && right) {
    try {
      
      left  = refold(left);
      right = refold(right);

      if (isConstant(left) && isConstant(right)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArccos::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArccosh::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArccot::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArccoth::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArccsc::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArccsch::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArcsec::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArcsech::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArcsin::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArcsinh::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArctan::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeArctanh::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCos::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);
    
      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCosh::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCot::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCoth::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCsc::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeCsch::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeSec::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeSech::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeSin::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeSinh::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch(InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeTan::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}


MesoRD::ReactionNode*
MesoRD::ReactionNodeTanh::fold()
  throw( MesoRD::ReactionNode::InvalidUnit )
{
  if (argument) {
    try {
      argument = refold(argument);

      if (isConstant(argument)) {
        ReactionNode* constant = new ReactionNodeConstant(
          evaluate(std::valarray<unsigned int>((unsigned int)0, 0)),
          getUnit(),
          parameterTable,
          speciesTable);

        delete this;
        return constant;
      }
    } catch( InvalidUnit i ) {
      throw InvalidUnit(i.what());
    }
  }

  return this;
}
