/* The IPrefixStream class.
 *
 * Copyright 1995 Dietmar Kuehl
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: IPrefixStream.cpp 541 2008-04-12 18:52:41Z davidfange $
 */
#include "IPrefixStream.hpp"
#include "PrefixBuf.hpp"

  /* The only missing functions are the constructors and the
   * destructors of the classes IPrefixStream (used to read a file
   * stripping of a prefix from every line read) and oprfxstream (used
   * to prepend a prefix to every line written). The constructors are
   * doing exactly the same thing: They initialize their base class
   * with a prfxbuf which is constructed from their arguments.
   */
IPrefixStream::IPrefixStream(std::streambuf *sb, const char *prfx):
  std::istream(new PrefixBuf(sb, prfx))
{
}


  /* Finally, the destructors of the class IPrefixStream and
   * oprfxstream delete the prfxbuf which was constructed in the
   * constructor. The pointer to the prfxbuf is obtained using the
   * ios::rdbuf(): The actual pointer is stored in the ios part of the
   * [io]prfxstream. There is no need to duplicate this pointer.
   */
IPrefixStream::~IPrefixStream()
{
  delete rdbuf();
}

