/* The IPrefixStream class.
 *
 * Copyright 1995 Dietmar Kuehl
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: IPrefixStream.hpp 540 2008-04-12 14:30:36Z davidfange $
 */
#ifndef __IPrefixStream_H
#define __IPrefixStream_H

#include <iostream>

  /**
   * To provide convenient access to the prfxbuf there is an istream and an
   * ostream provided: both create a prfxbuf with the given arguments and
   * initialize their base class (the istream or the ostream) with this
   * object. Both class also take care that the allocate prfxbuf is
   * destructed when the [io]stream is destructed.  It is also possible to
   * perform the same operations "by hand" i.e. to construct a prfxbuf,
   * initialize a corresponding stream with it, and finally, after the
   * stream is no longer needed, destroy the created the constructed
   * prfxbuf. However, it is more convenient and safer to use classes
   * derived from [io]stream.
   *
   * http://www.inf.uni-konstanz.de/~kuehl/c++/iostream/
   */
class IPrefixStream : public std::istream {
  public:
  IPrefixStream(std::streambuf *sb, const char *prfx);
  ~IPrefixStream();
};

#endif
