/* The PrefixBuf class.
 *
 * Copyright 1995 Dietmar Kuehl
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: PrefixBuf.hpp 540 2008-04-12 14:30:36Z davidfange $
 */
#ifndef __PrefixBuf_H
#define __PrefixBuf_H

#include <iostream>


  /**
   * The important part of this file is a class derived from the class
   * streambuf:  The class streambuf defines an interface used by
   * [io]streams to read bytes from an external representation and to write
   * bytes to an external representation. The actual handling of the
   * external representation is implemented in classes derived from
   * streambuf. In this case the "external representation" is a wrapper for
   * another external representation used to prepend a prefix to every line
   * written and to strip off a prefix from every line read. Another
   * streambuf is used by this class to handle a real external
   * representation.
   *
   * http://www.inf.uni-konstanz.de/~kuehl/c++/iostream/
   */
class PrefixBuf : public std::streambuf {
  private:
  std::streambuf	*i_sbuf;	// the actual streambuf used to read and write chars
  unsigned int	i_len;		// the length of the prefix
  char		*i_prfx;	// the prefix
  bool		i_newline;	// remember whether we are at a new line
  int		i_cache;	// may cache a read character

  bool	skip_prefix();

  protected:
  int	overflow(int);
  int	underflow();
  int	uflow();
  int	sync();

  public:
  PrefixBuf(std::streambuf *sb, const char *prfx);
  ~PrefixBuf();
};

#endif
