/* The TextGetOpt class.
 *
 * Copyright 2004-2006 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TextGetOpt.hpp 554 2008-05-25 19:55:15Z davidfange $
 */
#ifndef MESORD_UI_TEXTGETOPT_H
#define MESORD_UI_TEXTGETOPT_H 1

#include "GetOpt.hpp"

namespace MesoRD
{

    /**
     * @brief Class for obtaining command line options.
     *
     * This class is essentially a wrapper around the getopt(3)
     * function.  It reads command line options, and uses the setter
     * functions in GetOpt to propagate the values down the class
     * hierarchy.  Because all programs according to the GNU Coding
     * Standards should support the standard <code>--help</code> and
     * <code>--version</code> options, there are textual
     * implementation of these in this class.
     */
  class TextGetOpt : public GetOpt
  {
  public:

      /**
       * @brief The TextGetOpt class constructor.
       *
       * Parses the options on the command line and sets up the GetOpt
       * members using the respective setter functions.
       *
       * @param argc The <i>argument count</i>, as from main().
       * @param argv The <i>argument vector</i>, as from main().
       */
    TextGetOpt(const int argc, char* const* argv);
      /**
       * @brief The TextGetOpt class destructor.
       *
       * Does nothing.
       */
    inline ~TextGetOpt()
      {}


      /**
       * @brief Display help information.
       *
       * This just prints a summary of all the options and a brief
       * explanation of what they accomplish to <code>stdout</code>.
       */
    void
    displayHelp() const;


      /**
       * @brief Display version information.
       *
       * @bug This implementation is obviously not right, and needs to
       * be fixed some day.
       */
    void
    displayVersion() const
      {
        std::cout << "This is Version" << std::endl;
      }
  };
}
#endif
