/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* Copyright (c) 2004-2007, Johan Elf, David Fange, Johan Hattne
 *
 * Permission to use, copy, modify, and/or distribute this software
 * for any purpose with or without fee is hereby granted, provided
 * that the above copyright notice and this permission notice appear
 * in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $Id: TextProgressBar.cpp 541 2008-04-12 18:52:41Z davidfange $
 */
#include "../mesord.hpp"

#include <cmath>
#include <iomanip>
#include <iostream>

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#include "TextProgressBar.hpp"


  /* This textual progress bar is modelled after the OpenSSH scp
   * textual progress bar.  The bar's title is given by
   * <code>title</code>, the actual progress in the closed interval
   * zero to one by <code>fraction</code> and the total width is
   * <code>width</code>.
   */
void
MesoRD::TextProgressBar::draw()
{
  const int width_minimum   = 6;
  int       width_remaining = width;

    /* The title of the progress bar will be output only if doing so
     * will not use up all available space.  If a title is printed, an
     * extra character will be needed to pad between title and the
     * progress bar itself.
     */
  if (width_remaining > width_minimum + 1) {
    const int width_title = MesoRD::max(
      MesoRD::min(static_cast<int>(title.length()) + 0,
		  width - (width_minimum + 1)),
      MesoRD::min(static_cast<int>(title.length()) + 2,
		  width - (width_minimum + 1)));

    std::cout.fill(' ');
    std::cout.setf(std::ios::right);
    std::cout << std::setw(width_title) << title << ' ';

    width_remaining -= width_title + 1;
  }

    /* The minimum progress indicator is a percentage printing.  This
     * will use up <code>width_minimum</code> characters.
     */
  if (width_remaining >= width_minimum) {
    const int percentage = static_cast<int>(lrint(fraction * float(100)));

    std::cout.fill(' ');
    std::cout.setf(std::ios::right);
    std::cout << std::setw(3) << percentage
              << "% "         << updateSpinner(spinner);
    std::cout.setf(std::ios::left);

    width_remaining -= 4;
  }

    /* The textual bar requires two characters for the left and right
     * delimiters plus one extra character for padding between the
     * percentage display.  Any space left will be used for the actual
     * bar.
     */
  if (width_remaining > 3) {
    const long width_progress =
      lrint(fraction * (width_remaining - 3));

    std::cout << " |";
    for (int i = 0; i < width_remaining - 3; i++)
      std::cout << static_cast<char>((i < width_progress) ? '*' : ' ');
    std::cout << "|";
  }

  std::cout << "\r" << std::flush;
}
