/* The WoeGetOpt class.
 *
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: WoeGetOpt.cpp 559 2008-09-10 19:46:33Z davidfange $
 */
#if defined(_MSC_VER) && _MSC_VER > 1000
#  pragma warning(disable: 4786)
#endif



#include <valarray> // Required before windows.h on windows
#define NOMINMAX
#include <windows.h>
#include <shlobj.h>
#include "../../woe32/resources/resource.h"

//#include <cstdio>
#include <iostream>
#include <sstream>

#include "WoeGetOpt.hpp"

const std::string colour_disabled = "Display Disabled";


 /* I would really like this to be a member function instead of just a friend.
 */
static MesoRD::GetOpt* woeGetOptStaticClass = 0;

template<class T>
inline const std::string
toString(T t)
{
  std::stringstream string;
  string << t;
  return string.str();
}

void
enableCheckpoint(HWND hwnd, bool state)
{
  EnableWindow(
    GetDlgItem(hwnd, IDC_EDIT_CHECKPOINT_ITERATIONS),   state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_EDIT_CHECKPOINT_TIME),         state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_STATIC_CHECKPOINT_ITERATIONS), state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_STATIC_CHECKPOINT_TIME),       state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_CHECK_FULLSTATE_CHECKPOINT),   state);
}

void
enableOpenGL(HWND hwnd, bool state)
{
  EnableWindow(
    GetDlgItem(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR),        state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME),          state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_COMBO_SPECIES_DIMENSION),            state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_STATIC_OPENGL_VISUALISER_COLOUR),    state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_STATIC_OPENGL_VISUALISER_SUBVOLUME), state);
  EnableWindow(
    GetDlgItem(hwnd, IDC_STATIC_OPENGL_TEXT),                 state);
}

void enableDeterministic(HWND hwnd, bool state)
{

    /* disable the opengl options */
  CheckDlgButton(hwnd, IDC_CHECK_OPENGL_VISUALISER, !state);
  EnableWindow(
	GetDlgItem(hwnd, IDC_CHECK_OPENGL_VISUALISER),	 !state);
  enableOpenGL(hwnd, !state);

    /* disable the sparse option */
  CheckDlgButton(hwnd, IDC_CHECK_SPARSE, !state);
  EnableWindow(
	GetDlgItem(hwnd, IDC_CHECK_SPARSE),	 !state);

  EnableWindow(
	GetDlgItem(hwnd, IDC_COMBO_INTEGRATION_METHOD),	 state);
  EnableWindow(
	GetDlgItem(hwnd, IDC_STATIC_INTEGRATION_METHOD), state);
  EnableWindow(
	GetDlgItem(hwnd, IDC_STATIC_TIMESTEP),           state);
  EnableWindow(
	GetDlgItem(hwnd, IDC_EDIT_TIMESTEP),             state);
  EnableWindow(
	       GetDlgItem(hwnd, IDC_STATIC_TOLERANCE_CUTOFF),   state);
  EnableWindow(
	       GetDlgItem(hwnd, IDC_EDIT_TOLERANCE_CUTOFF),     state);
  EnableWindow(
	       GetDlgItem(hwnd, IDC_STATIC_MAX_ITERATIONS), state);
  EnableWindow(
	       GetDlgItem(hwnd, IDC_EDIT_MAX_ITERATIONS),   state);
  EnableWindow(
	       GetDlgItem(hwnd, IDC_STATIC_ITERATIVE_METHODS_HEADER), state);
  EnableWindow(
	       GetDlgItem(hwnd, IDC_COMBO_INTEGRATION_METHOD), state);
}

std::string
getCurrentDirectory()
{
  char buffer[MAX_PATH];
  std::string dir = "";

   /* Get the current working directory: */
  if(GetCurrentDirectory(MAX_PATH, buffer))
     dir = std::string(buffer);

   return dir;
}

double
getDoubleItem(HWND hDlg, int nIDDlgItem) {
  double value = 0.0;
  int    len   = 0;
	
  if((len = GetWindowTextLength(GetDlgItem(hDlg, nIDDlgItem))) > 0) {
    char* buffer;
    char* endptr;

    buffer = (char*)GlobalAlloc(GPTR, len + 1);
    GetDlgItemText(hDlg, nIDDlgItem, buffer, len + 1);

    value = strtod(buffer, &endptr);

	GlobalFree((HANDLE)buffer);
  }

  return value;
}

long int
getLongIntItem(HWND hDlg, int nIDDlgItem) {
  long int value = 0;
  int      len   = 0;
	
  if((len = GetWindowTextLength(GetDlgItem(hDlg, nIDDlgItem))) > 0) {
    char* buffer;
    char* endptr;

    buffer = (char*)GlobalAlloc(GPTR, len + 1);
    GetDlgItemText(hDlg, nIDDlgItem, buffer, len + 1);

    value = strtol(buffer, &endptr, 10);

	GlobalFree((HANDLE)buffer);
  }

  return value;
}

std::string
getStringMessage(HWND hDlg, int nIDDlgItem, unsigned int item)
{
  int         len   = 0;
  std::string value = "";
  
  if ((len = SendDlgItemMessage(hDlg, nIDDlgItem, CB_GETLBTEXTLEN, (WPARAM)item, 0)) > 0) {
    char* buffer;
    
    buffer = (char*)GlobalAlloc(GPTR, len + 1);
    SendDlgItemMessage(hDlg, nIDDlgItem, CB_GETLBTEXT, (WPARAM)item, (LPARAM)buffer);
    
    value = std::string(buffer);
    
    GlobalFree((HANDLE)buffer);
  }
  return value;
}

void
updateName(HWND hwnd) {
  std::string colour = getStringMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
      SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR, CB_GETCURSEL, 0, 0));
  std::string species = getStringMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME,
    SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME, CB_GETCURSEL, 0, 0));
  
  if (colour != colour_disabled)
    woeGetOptStaticClass -> setColour(species, colour);
  else
    woeGetOptStaticClass -> setColour(species, "");
}
void
updateColour(HWND hwnd)
{
  std::string species = getStringMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME,
    SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME, CB_GETCURSEL, 0, 0));
  std::string colour = woeGetOptStaticClass -> getColour(species);
  
  if (colour.length() > 0)
    SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
    CB_SELECTSTRING, 0, (LPARAM)colour.c_str());
  else
    SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
    CB_SELECTSTRING, 0, (LPARAM)colour_disabled.c_str());            
}

void
addTimes(HWND hwnd)
{
  std::list<std::string> times;
  try {
    times = woeGetOptStaticClass -> getTimes();
  } catch( MesoRD::GetOpt::FileNotFoundError& e) {
      MessageBox(NULL, e.what(), "Error!",
        MB_ICONEXCLAMATION | MB_OK);
  }
  
      SendDlgItemMessage(hwnd, IDC_COMBO_OLD_STATE_TIMES,
     CB_RESETCONTENT, 0, 0);

  std::list<std::string>::const_iterator i;
  
  for(i = times.begin(); i != times.end(); i++) {
    SendDlgItemMessage(hwnd, IDC_COMBO_OLD_STATE_TIMES, CB_ADDSTRING, 0, 
      (LPARAM) (*i).c_str() );
  }
}

void enableOldState(HWND hwnd, bool state)
{
  EnableWindow(
	GetDlgItem(hwnd, IDC_COMBO_OLD_STATE_TIMES),	 state);
  if(state) {
    addTimes(hwnd); 
  }
}

  
BOOL CALLBACK MesoRD::OptionDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
//  woeGetOptStaticClass -> viewer   = false;
//  woeGetOptStaticClass -> size     = 0.0;
//  woeGetOptStaticClass -> filename = "";
//  woeGetOptStaticClass -> unit     = "";	


  static int index_species_dimension_1;
  static int index_species_dimension_2;
  static int index_species_dimension_3;
  static int index_species_dimension_4;
  static int index_species_dimension_5;

//  static int index_disabled_colour;

  OPENFILENAME ofn;       // common dialog box structure
  char szFile[MAX_PATH];       // buffer for file name use MAXPATH from windows.h maximum length of a buffer needed to store a filename under Win32
//HWND hwnd = NULL;              // owner window -- should be an argument?
//  HANDLE hf;              // file handle

// Initialize OPENFILENAME
  ZeroMemory(&ofn, sizeof(ofn));


  std::list<std::string> species;
  std::list<std::string>::const_iterator k;

//  std::vector<VisualColour*> colours;

  unsigned int i;

  switch(msg) {
    case WM_INITDIALOG:

        /* Load the icons.
         */
      SendMessage(hwnd, WM_SETICON, ICON_SMALL,
        (LPARAM)LoadIcon(NULL, MAKEINTRESOURCE(IDI_ICON1)));
      SendMessage(hwnd, WM_SETICON, ICON_BIG,
        (LPARAM)LoadIcon(NULL, MAKEINTRESOURCE(IDI_ICON2)));



      CheckDlgButton(hwnd, IDC_CHECK_FULLSTATE_CHECKPOINT,
        ((woeGetOptStaticClass -> getOptionFullStateCheckPoint()) 
          ? BST_CHECKED : BST_UNCHECKED));

        /* Whether to check the checkpointing by default or not.
        */
      if (((woeGetOptStaticClass -> getOptionCheckpointIterations()) > 0)
                || ((woeGetOptStaticClass -> getOptionCheckpointTime()) > 0)) {
       CheckDlgButton(hwnd, IDC_CHECK_CHECKPOINT, BST_CHECKED);
      } else {
       CheckDlgButton(hwnd, IDC_CHECK_CHECKPOINT, BST_UNCHECKED);
      }

        /* Setting the default value for deterministc simulation
         */
      CheckDlgButton(hwnd, IDC_CHECK_DETERMINISTIC,
        ((woeGetOptStaticClass -> getOptionDeterministic())
          ? BST_CHECKED : BST_UNCHECKED));

	  SetDlgItemText(hwnd, IDC_EDIT_TIMESTEP,
	    toString(woeGetOptStaticClass -> getOptionTimeStep()).c_str());
        
	  SetDlgItemText(hwnd, IDC_EDIT_TOLERANCE_CUTOFF,
	      toString(woeGetOptStaticClass -> 
		       getOptionItMetConvergenceTolerance()).c_str());
	  
	  SetDlgItemText(hwnd, IDC_EDIT_MAX_ITERATIONS,
	     toString(woeGetOptStaticClass -> getOptionItMetMaxIterations()).c_str());


        /* Whether to check the progress bar by default or not.
         */
      CheckDlgButton(hwnd, IDC_CHECK_PROGRESS_BAR,
        ((woeGetOptStaticClass -> getOptionProgressBars()) ? BST_CHECKED : BST_UNCHECKED));   
        /* The default number of checkpoint iterations.
         */
      SetDlgItemText(hwnd, IDC_EDIT_CHECKPOINT_ITERATIONS,
        toString(woeGetOptStaticClass -> getOptionCheckpointIterations()).c_str());

        /* The default checkpoint time.
         */
      SetDlgItemText(hwnd, IDC_EDIT_CHECKPOINT_TIME,
        toString(woeGetOptStaticClass -> getOptionCheckpointTime()).c_str());
      
        /* The default maximal number of iterations.
         */
      SetDlgItemText(hwnd, IDC_EDIT_RUN_ITERATIONS,
        toString(woeGetOptStaticClass -> getOptionMaxIterations()).c_str());
      SetDlgItemText(hwnd, IDC_EDIT_RUN_ITERATIONS_SCALE,
        toString(woeGetOptStaticClass -> getOptionMaxIterationsScale()).c_str());

        /* The default maximal simulation time.
         */
      SetDlgItemText(hwnd, IDC_EDIT_RUN_TIME,
        toString(woeGetOptStaticClass -> getOptionMaxTime()).c_str());
      
        /* The default sub volume size.
         */
      SetDlgItemText(hwnd, IDC_EDIT_SUBVOLUME_SIZE,
        toString(woeGetOptStaticClass -> getOptionSizeX()).c_str());

        /* The default file name when no file is selected.
         */
		//if(woeGetOptStaticClass -> getOptionFilename() == ""){
      SetDlgItemText(hwnd, IDC_STATIC_FILENAME,	
        "No input SBML file selected.");
		//} else {
		//	SetDlgItemText(hwnd, IDC_STATIC_FILENAME,	
        //woeGetOptStaticClass->getOptionFilename().c_str());
		//}
        /* The default output directory before one is set.
         */
      SetDlgItemText(hwnd, IDC_STATIC_OUTPUT_DIRECTORY,
        "No output directory set.");
      
        /* default output file value 
         */
      CheckDlgButton(hwnd, IDC_CHECK_OUTPUT_FILE,
		     ((woeGetOptStaticClass -> getOptionLog()) 
                       ? BST_CHECKED : BST_UNCHECKED));
      
        /* default old state read setting 
         */
      CheckDlgButton(hwnd, IDC_CHECK_OLD_STATE,
                    ((woeGetOptStaticClass -> getOptionReadOldState())
                      ? BST_CHECKED : BST_UNCHECKED));
	  
        /* Set up the sub volume geometry combo box.  There is only
         * one kind of sub volume, which is the cube, so add it and
         * make sure it is chosen.
         */
      SendDlgItemMessage(hwnd, IDC_COMBO_SUBVOLUME_GEOMETRY,
        CB_ADDSTRING, 0, (LPARAM)"Cubic");
      SendDlgItemMessage(hwnd, IDC_COMBO_SUBVOLUME_GEOMETRY,
        CB_SELECTSTRING, 0, (LPARAM)"Cubic");
 
        /* Set up the different alternatives as to how many molecules
         * should be displayed in a sub volume.  Select the default
         */
      index_species_dimension_1 = SendDlgItemMessage(hwnd,
        IDC_COMBO_SPECIES_DIMENSION, CB_ADDSTRING, 0, (LPARAM)"1");
      index_species_dimension_2 = SendDlgItemMessage(hwnd,
        IDC_COMBO_SPECIES_DIMENSION, CB_ADDSTRING, 0, (LPARAM)"8");
      index_species_dimension_3 = SendDlgItemMessage(hwnd,
        IDC_COMBO_SPECIES_DIMENSION, CB_ADDSTRING, 0, (LPARAM)"27");
      index_species_dimension_4 = SendDlgItemMessage(hwnd,
        IDC_COMBO_SPECIES_DIMENSION, CB_ADDSTRING, 0, (LPARAM)"64");
      index_species_dimension_5 = SendDlgItemMessage(hwnd,
        IDC_COMBO_SPECIES_DIMENSION, CB_ADDSTRING, 0, (LPARAM)"125");

      switch (woeGetOptStaticClass -> getOptionSpeciesDimension()) {
        case 1:
          SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
            CB_SETCURSEL, (WPARAM)index_species_dimension_1, 0);
          break;

        case 2:
          SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
            CB_SETCURSEL, (WPARAM)index_species_dimension_2, 0);
          break;

        case 3:
          SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
            CB_SETCURSEL, (WPARAM)index_species_dimension_3, 0);
          break;

        case 4:
          SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
            CB_SETCURSEL, (WPARAM)index_species_dimension_4, 0);
          break;

        case 5:
          SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
            CB_SETCURSEL, (WPARAM)index_species_dimension_5, 0);
          break;

        default:
          SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
            CB_SETCURSEL, (WPARAM)index_species_dimension_4, 0);
          break;
      }

      /* adding the integration methods to the dropdown. */
      SendDlgItemMessage(hwnd, IDC_COMBO_INTEGRATION_METHOD,
			 CB_RESETCONTENT, 0, 0);
      
      for(i = 0; i < MESORD_INTEGRATION_METHODS ; i++){
	SendDlgItemMessage(hwnd, IDC_COMBO_INTEGRATION_METHOD,
			   CB_ADDSTRING, 0, (LPARAM) 
			   MESORD_INTEGRATION_METHOD[i].c_str());
      }
      
      SendDlgItemMessage(hwnd, IDC_COMBO_INTEGRATION_METHOD,
      		 CB_SELECTSTRING, 0, (LPARAM) "Euler");
      
      
        /* Disable all the buttons.
         */
      EnableWindow(GetDlgItem(hwnd, IDOK),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_BUTTON_OUTPUT_DIRECTORY),
        false);
        
        /* Disable all the check boxes.
         */
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_CHECKPOINT),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_PROGRESS_BAR),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_DETERMINISTIC),
		false);
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OUTPUT_FILE),
		false);
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OLD_STATE),
		false);
      

        /* Disable all the combo boxes.
         */
      EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SPECIES_DIMENSION),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SUBVOLUME_GEOMETRY),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SUBVOLUME_UNIT),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_COMBO_OLD_STATE_TIMES),
        false);
	  
        /* Disable all the edit boxes.
         */
      EnableWindow(GetDlgItem(hwnd, IDC_EDIT_CHECKPOINT_ITERATIONS),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_EDIT_CHECKPOINT_TIME),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_EDIT_RUN_ITERATIONS),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_EDIT_RUN_ITERATIONS_SCALE),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_EDIT_RUN_TIME),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_EDIT_SUBVOLUME_SIZE),
        false);

        /* Disable all the group boxes.
         */
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_CHECKPOINT),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_OPENGL_VISUALISER),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_RUN),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_SUBVOLUME),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_DETERMINISTIC),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_OLD_STATE),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_OTHER),
        false);

        /* Disable all the static text boxes.
         */
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_CHECKPOINT_ITERATIONS),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_CHECKPOINT_TIME),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_OUTPUT_DIRECTORY),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_RUN_ITERATIONS),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_RUN_ITERATIONS_SCALE),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_RUN_TIME),
        false);      
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_SUBVOLUME_GEOMETRY),
        false);
      EnableWindow(GetDlgItem(hwnd, IDC_STATIC_SUBVOLUME_SIZE),
        false);
	  

        /* Disable the stuff we can toggle.
         */
      enableCheckpoint(hwnd, false);
      enableDeterministic(hwnd, false);
      enableOpenGL(hwnd, false);

      
        /* Taking care of some last changes, the below depend on the
         * enableDeterministic function and therefore needs to
         * be set last....
         */
        /* Whether to check the viewer by default or not.
         */
      CheckDlgButton(hwnd, IDC_CHECK_OPENGL_VISUALISER,
        ((woeGetOptStaticClass -> getOptionViewer()) ? BST_CHECKED : BST_UNCHECKED));
      
        /* default sparse setting 
         */
      CheckDlgButton(hwnd, IDC_CHECK_SPARSE,
                    ((woeGetOptStaticClass -> getOptionSparse())
                      ? BST_CHECKED : BST_UNCHECKED));
      
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_SPARSE),
		false);
      
      EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OPENGL_VISUALISER),
        false);


      

      return TRUE;

    case WM_COMMAND:
      switch(LOWORD(wParam)) {
        case IDC_BUTTON_OUTPUT_DIRECTORY:
          {
            char* buffer = new char[MAX_PATH];
            char* path = new char[MAX_PATH];
            BROWSEINFO bi = {0};
            LPMALLOC pMalloc = NULL;
            SHGetMalloc(&pMalloc);


            bi.hwndOwner = hwnd;
            bi.pidlRoot = NULL; //pidlRoot;
            bi.pszDisplayName = buffer; //pszDisplayName;
            bi.lpszTitle = "Choose a folder";
            bi.ulFlags = 0;
            bi.lpfn = NULL;
            bi.lParam = 0;
            
            SHGetPathFromIDList(SHBrowseForFolder(&bi), path);
            
            pMalloc->Release();

            woeGetOptStaticClass -> setOptionOutputDir(std::string(path + std::string("\\")).c_str());

            SetDlgItemText(hwnd, IDC_STATIC_OUTPUT_DIRECTORY,
              (woeGetOptStaticClass -> getOptionOutputDir()).c_str());
          }
          break;
        
        case IDC_BUTTON_FILE:
          ofn.lStructSize     = sizeof(ofn);
          ofn.hwndOwner       = hwnd;
          ofn.lpstrFile       = szFile;
          //
          // Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
          // use the contents of szFile to initialize itself.
          //
          ofn.lpstrFile[0]    = '\0';
          ofn.nMaxFile        = sizeof(szFile);
          ofn.lpstrFilter     = "All\0*.*\0XML\0*.XML\0";
          ofn.nFilterIndex    = 2; // Use second filter as default
          ofn.lpstrFileTitle  = NULL;
          ofn.lpstrTitle      = "Open MesoRD SBML File";
          ofn.nMaxFileTitle   = 0;
          ofn.lpstrInitialDir = NULL;
          ofn.Flags           = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_READONLY;
          if (GetOpenFileName(&ofn)) {
            std::list<std::string>                 units;
            std::list<std::string>::const_iterator j;

              /* Set the file name to its proper value.  Make sure we
               * store the file name in the getOpt() super class.
               */
            woeGetOptStaticClass -> setOptionFilename(std::string(szFile).c_str());
            woeGetOptStaticClass -> setOptionOutputDir((getCurrentDirectory() + std::string("\\")).c_str());
            try{
              woeGetOptStaticClass -> fromSBML();
            }
            catch(MesoRD::System::ModelError e){
              std::stringstream message;
			  message << "An error occured: "//<< "Cought Exception " 
                      //<< typeid(e).name() 
                      //<< std::endl 
                      << e.what() 
                      << std::endl
                      << std::endl
                      << "Something is exceptionally wrong! "
                      << "Thank you, come again!"
                      << std::endl;
              MessageBox(NULL, (message.str()).c_str(), "Error!",
                MB_ICONEXCLAMATION | MB_OK);
            }
            catch(MesoRD::Species::ModelError e){
              std::stringstream message;
			  message << "An error occured: "//<< "Cought Exception " 
                      //<< typeid(e).name() 
                      //<< std::endl 
                      //<< e.what() 
                      << std::endl
                      << std::endl
                      << "Something is exceptionally wrong! "
                      << "Thank you, come again!"
                      << std::endl;
              MessageBox(NULL, (message.str()).c_str(), "Error!",
                MB_ICONEXCLAMATION | MB_OK);
            }
            
            SetDlgItemText(hwnd, IDC_STATIC_FILENAME,
              (woeGetOptStaticClass -> getOptionFilename()).c_str());
            SetDlgItemText(hwnd, IDC_STATIC_OUTPUT_DIRECTORY,
              (woeGetOptStaticClass -> getOptionOutputDir()).c_str());
            
              /* Disable all the buttons.
            */
            EnableWindow(GetDlgItem(hwnd, IDOK),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_BUTTON_OUTPUT_DIRECTORY),
              true);
            
              /* Enable all the check boxes.
               */
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_CHECKPOINT),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OPENGL_VISUALISER),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_PROGRESS_BAR),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_DETERMINISTIC),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OUTPUT_FILE),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OLD_STATE),
		true);
            EnableWindow(GetDlgItem(hwnd, IDC_CHECK_SPARSE),
		true);
              /* Enable all the combo boxes.
               */
            EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SPECIES_DIMENSION),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SUBVOLUME_GEOMETRY),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_COMBO_SUBVOLUME_UNIT),
              true);

              /* Enable all the edit boxes that we cannot toggle.
               */
            EnableWindow(GetDlgItem(hwnd, IDC_EDIT_RUN_ITERATIONS),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_EDIT_RUN_ITERATIONS_SCALE),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_EDIT_RUN_TIME),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_EDIT_SUBVOLUME_SIZE),
              true);

              /* Enable all the group boxes.
               */
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_CHECKPOINT),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_OPENGL_VISUALISER),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_RUN),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_SUBVOLUME),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_DETERMINISTIC),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_OTHER),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_GROUP_OLD_STATE),
              true);

              /* Enable all the static text boxes.
               */
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_OUTPUT_DIRECTORY),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_RUN_ITERATIONS),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_RUN_ITERATIONS_SCALE),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_RUN_TIME),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_SUBVOLUME_GEOMETRY),
              true);
            EnableWindow(GetDlgItem(hwnd, IDC_STATIC_SUBVOLUME_SIZE),
              true);

              /* Enable the stuff we can toggle, if appropriate.
               */
            if (((woeGetOptStaticClass -> getOptionCheckpointIterations()) > 0)
              || ((woeGetOptStaticClass -> getOptionCheckpointTime()) > 0)) {
             EnableWindow(GetDlgItem(hwnd, IDC_CHECK_CHECKPOINT),
               true);
             enableCheckpoint(hwnd, true);
            }
            
            if (woeGetOptStaticClass -> getOptionViewer()) {
              EnableWindow(GetDlgItem(hwnd, IDC_CHECK_OPENGL_VISUALISER),
                true);
              enableOpenGL(hwnd, true);
            }
            
            if(woeGetOptStaticClass -> getOptionDeterministic()){
              enableDeterministic(hwnd, true);
            }

              /* Enable the OK button.
               */
            EnableWindow(GetDlgItem(hwnd, IDOK),
              true);

              /* Get the length units from the SBML model file and add
               * them to the size unit combo box.  We need to clear
               * the combo box first, in case the model definition is
               * reloaded.  Then select the default length unit.
               */
            SendDlgItemMessage(hwnd, IDC_COMBO_SUBVOLUME_UNIT,
              CB_RESETCONTENT, 0, 0);

            units = woeGetOptStaticClass -> getPossibleUnits();
            for (j = units.begin(); j != units.end(); j++) {
              SendDlgItemMessage(hwnd, IDC_COMBO_SUBVOLUME_UNIT,
                CB_ADDSTRING, 0, (LPARAM)(*j).c_str());
            }
            
            SendDlgItemMessage(hwnd, IDC_COMBO_SUBVOLUME_UNIT,
              CB_SELECTSTRING, 0,
              (LPARAM)((woeGetOptStaticClass -> getOptionUnit()).c_str()));

              /* Add all the possible colours to the species colour
               * combo box.  We need to clear the combo box first, in
               * case the model definition is reloaded.  Make sure to
               * add the special colour which is used for suppressing
               * the rendering of a particular species.
               */ 
            SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
              CB_RESETCONTENT, 0, 0);

            woeGetOptStaticClass -> setDefaultColours();
//            colours = woeGetOptStaticClass -> getColours();
            for (i = 0; i < MESORD_VIEWER_COLOURS; i++) {
              SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
                CB_ADDSTRING, 0,
                (LPARAM)(MESORD_VIEWER_COLOUR[i].name).c_str());
            }
            
            SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
              CB_ADDSTRING, 0, (LPARAM)colour_disabled.c_str());

//            SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_COLOUR,
//              CB_SELECTSTRING, 0, (LPARAM)(colours[0] -> name).c_str());
            

              /* Add all species to the species name combo box.  We
               * need to clear the combo box first, in case the model
               * definition is reloaded.  Select the first species.  Then update the colour
               *  so that it matches the 
               * selected species.
               */
            SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME,
              CB_RESETCONTENT, 0, 0);

            species = woeGetOptStaticClass -> getAllSpecies();
            for (k = species.begin(); k != species.end(); k++) {
              SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME,
                CB_ADDSTRING, 0, (LPARAM)(*k).c_str());
            }
            
            SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_COLOUR_NAME,
              CB_SETCURSEL, 0, 0);

            updateColour(hwnd);
	    
          }
          break;
          
        case IDOK_OPTION:
         woeGetOptStaticClass -> setOptionHelp(
            false);
          woeGetOptStaticClass -> setOptionMaxIterations(
            getLongIntItem(hwnd, IDC_EDIT_RUN_ITERATIONS));
          woeGetOptStaticClass -> setOptionMaxIterationsScale(
            getLongIntItem(hwnd, IDC_EDIT_RUN_ITERATIONS_SCALE));
          woeGetOptStaticClass -> setOptionMaxTime(
            getDoubleItem(hwnd, IDC_EDIT_RUN_TIME));
          woeGetOptStaticClass -> setOptionPedantic(
            false);
          woeGetOptStaticClass -> setOptionProgressBars(
            IsDlgButtonChecked(hwnd, IDC_CHECK_PROGRESS_BAR));
          woeGetOptStaticClass -> setOptionSimulate(
            true);
          woeGetOptStaticClass -> setOptionSizeX(
            getDoubleItem(hwnd, IDC_EDIT_SUBVOLUME_SIZE));          
          woeGetOptStaticClass -> setOptionUnit(
            getStringMessage(hwnd, IDC_COMBO_SUBVOLUME_UNIT,
              SendDlgItemMessage(hwnd, IDC_COMBO_SUBVOLUME_UNIT,
                CB_GETCURSEL, 0, 0)).c_str());
          woeGetOptStaticClass -> setOptionUsage(
            false);
          woeGetOptStaticClass -> setOptionVersion(
            false);
          woeGetOptStaticClass -> setOptionViewer(
            IsDlgButtonChecked(hwnd, IDC_CHECK_OPENGL_VISUALISER));

	  woeGetOptStaticClass -> setOptionLog( 
	    IsDlgButtonChecked(hwnd, IDC_CHECK_OUTPUT_FILE));
          
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_CHECKPOINT)) {
            woeGetOptStaticClass -> setOptionCheckpointIterations(
              getLongIntItem(hwnd, IDC_EDIT_CHECKPOINT_ITERATIONS));
            woeGetOptStaticClass -> setOptionCheckpointTime(
              getDoubleItem(hwnd, IDC_EDIT_CHECKPOINT_TIME));
            if( IsDlgButtonChecked( hwnd, IDC_CHECK_FULLSTATE_CHECKPOINT )) {
              woeGetOptStaticClass -> setOptionFullStateCheckPoint( 
                true);
            } else {
              woeGetOptStaticClass -> setOptionFullStateCheckPoint( 
                false);
            } 
          } else {
            woeGetOptStaticClass -> setOptionCheckpointIterations(
              size_t(0));
            woeGetOptStaticClass -> setOptionCheckpointTime(
              double(0));
            woeGetOptStaticClass -> setOptionFullStateCheckPoint( 
              false);
          }
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_DETERMINISTIC)){
            woeGetOptStaticClass -> setOptionDeterministic(true);
            woeGetOptStaticClass -> setOptionTimeStep( 
              getDoubleItem(hwnd, IDC_EDIT_TIMESTEP));
            woeGetOptStaticClass -> setOptionIntegrationMethod( 
              getStringMessage(
                hwnd, IDC_COMBO_INTEGRATION_METHOD,
                SendDlgItemMessage(hwnd,
                  IDC_COMBO_INTEGRATION_METHOD, 
                  CB_GETCURSEL, 0, 0)).c_str());
            woeGetOptStaticClass -> setOptionItMetMaxIterations( 
              getLongIntItem(hwnd, IDC_EDIT_MAX_ITERATIONS));
            woeGetOptStaticClass -> setOptionItMetConvergenceTolerance(
              getDoubleItem(hwnd, 
                IDC_EDIT_TOLERANCE_CUTOFF));
            woeGetOptStaticClass -> setOptionIntegrationMethod(
              getStringMessage(hwnd, 
                IDC_COMBO_INTEGRATION_METHOD,
                SendDlgItemMessage(hwnd, 
                  IDC_COMBO_INTEGRATION_METHOD,
                  CB_GETCURSEL, 
                  0, 0)).c_str());
          }
          
          if (SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
                CB_GETCURSEL, 0, 0) == index_species_dimension_1) {
            woeGetOptStaticClass -> setOptionSpeciesDimension(1);
          
          } else if (SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
                       CB_GETCURSEL, 0, 0) == index_species_dimension_2) {
            woeGetOptStaticClass -> setOptionSpeciesDimension(2);
          
          } else if (SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
                     CB_GETCURSEL, 0, 0) == index_species_dimension_3) {
            woeGetOptStaticClass -> setOptionSpeciesDimension(3);
          
          } else if (SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
                       CB_GETCURSEL, 0, 0) == index_species_dimension_4) {
            woeGetOptStaticClass -> setOptionSpeciesDimension(4);
          
          } else if (SendDlgItemMessage(hwnd, IDC_COMBO_SPECIES_DIMENSION,
                       CB_GETCURSEL, 0, 0) == index_species_dimension_5) {
            woeGetOptStaticClass -> setOptionSpeciesDimension(5);
          }
          
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_OLD_STATE)){
            woeGetOptStaticClass -> setOptionReadOldState(true);
            woeGetOptStaticClass -> setOptionReadOldStateTime( 
              SendDlgItemMessage(hwnd, 
                IDC_COMBO_OLD_STATE_TIMES, CB_GETCURSEL, 0, 0));
          }
          
          woeGetOptStaticClass -> setOptionSparse(
            IsDlgButtonChecked(hwnd, IDC_CHECK_SPARSE));

          EndDialog(hwnd, IDOK);
          break;
          
        case IDC_CHECK_CHECKPOINT:
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_CHECKPOINT)) {
            enableCheckpoint(hwnd, true);
          } else {
            enableCheckpoint(hwnd, false);
          }
          break;
          
        case IDC_CHECK_OPENGL_VISUALISER:
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_OPENGL_VISUALISER)) {
            enableOpenGL(hwnd, true);
          } else {
            enableOpenGL(hwnd, false);
          }
          break;

        case IDC_CHECK_DETERMINISTIC:
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_DETERMINISTIC)) {
            enableDeterministic(hwnd, true);
          } else {
            enableDeterministic(hwnd, false);
          }
          break;
          
        case IDC_CHECK_OLD_STATE:
          if (IsDlgButtonChecked(hwnd, IDC_CHECK_OLD_STATE)) {
            enableOldState(hwnd, true);
          } else {
            enableOldState(hwnd, false);
          }
          break;
          
          
        case IDC_COMBO_SPECIES_COLOUR_NAME:
          if (HIWORD(wParam) == CBN_CLOSEUP) {
            updateColour(hwnd);
          }
          break;
          
          
        case IDC_COMBO_SPECIES_COLOUR_COLOUR:
          if (HIWORD(wParam) == CBN_CLOSEUP) {
            updateName(hwnd);
          }
           break;
          

        case IDCANCEL_OPTION:
          woeGetOptStaticClass -> setOptionFilename(std::string("").c_str());
          woeGetOptStaticClass -> setOptionSimulate(false);
          EndDialog(hwnd, IDCANCEL);
          break;
      }
      break;
      
    default:
      return FALSE;
  }
  return TRUE;
}



  /****************************
   * CONSTRUCTOR & DESTRUCTOR *
   ****************************/

  /* Default constructor.  Initialise the real and the complex part to
   * zero.
   *
   * Command line options parsing using getopt_long.  According to the
   * GNU coding standards, all programs should support two standard
   * options: "--help" and "--version".
   */
MesoRD::WoeGetOpt::WoeGetOpt(HINSTANCE hInstance, HWND hwnd, int nCmdShow, GetOpt* go)
: GetOpt (*go) 
{
  int ret;
  woeGetOptStaticClass = this;

  if (ret = DialogBox(GetModuleHandle(NULL),
        MAKEINTRESOURCE(IDD_DIALOG_OPTION), hwnd, OptionDialogProc)) {
    ;
  }
}


  /**
   * Destructor.  Do-nothing.
   */
MesoRD::WoeGetOpt::~WoeGetOpt() {}

