/* The WoeGetOpt class.
 *
 * Copyright 2004 Johan Elf, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: WoeProgressBar.cpp 542 2008-04-15 19:35:37Z davidfange $
 */
#include <iostream>
#include <sstream>

#include "../mesord.hpp"
#include "WoeProgressBar.hpp"
#include "../../woe32/resources/resource.h"

 /* Include after "WoeProgressBar.hpp", which in turn includes "windows.h".  This is because of the strange dependencies with the Windows headers.
 */
#include <commctrl.h>

  /* The namespace to use.
   */
//using namespace MesoRD;


 /* I would really like this to be a member function instead of just a friend.
 */
static MesoRD::WoeProgressBar* woeProgressBarStaticClass = 0;

const int MESORD_WOE_PROGRESSBAR_MIN = 0;
const int MESORD_WOE_PROGRESSBAR_MAX = 1000;

BOOL CALLBACK MesoRD::ProgressBarDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{

  switch(msg) {

    case WM_INITDIALOG:
      SendMessage(GetDlgItem(hwnd, IDC_PROGRESS), PBM_SETRANGE, 0, MAKELPARAM(MESORD_WOE_PROGRESSBAR_MIN, MESORD_WOE_PROGRESSBAR_MAX)); 
      return TRUE;
      break;

    case WM_COMMAND:
 	  switch(LOWORD(wParam)) {
 
        case ID_BUTTON_STOP:
          if (woeProgressBarStaticClass)
		    woeProgressBarStaticClass -> alive = false;
          ShowWindow(hwnd, SW_HIDE);
          //EndDialog(hwnd, IDCANCEL);
          break;
	  }
      break;

	default:
      return FALSE;
  }
  return TRUE;
}

  /****************************
   * CONSTRUCTOR & DESTRUCTOR *
   ****************************/

  /* Default constructor.  Initialise the real and the complex part to
   * zero.
   *
   * The progress bar should not be modal (i.e. modeless) so that control returns immediately to the caller.
   */
MesoRD::WoeProgressBar::WoeProgressBar(HINSTANCE hInstance, HWND hwnd, int nCmdShow) 
  : ProgressBar()
{
  alive                     = false;
  handle_progress           = 0;
  woeProgressBarStaticClass = this;

  if ((handle_progress = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_DIALOG_PROGRESS), hwnd, ProgressBarDialogProc))) {
    alive = true;
	ShowWindow(handle_progress, SW_SHOW);
  } else {
    MessageBox(hwnd, "CreateDialog returned NULL", "Warning!",  
                MB_OK | MB_ICONINFORMATION);
  }
}


  /**
   * Destructor.  Do-nothing.
   */
MesoRD::WoeProgressBar::~WoeProgressBar() {
  DestroyWindow(handle_progress);
//  PostQuitMessage(0);
}

  /* Modelled after the OpenSSH scp textual progressbar
   */

  /* Modelled after the OpenSSH scp textual progressbar
   */


void
MesoRD::WoeProgressBar::setFraction(const double fraction)
{
  if ((fraction >= 0.0) && (fraction <= 1.0)) {
    this -> fraction = fraction;
	SendMessage(GetDlgItem(handle_progress, IDC_PROGRESS), PBM_SETPOS, (int)rint(1000 * fraction), 0); 
  }
}

void
MesoRD::WoeProgressBar::setIteration(const std::valarray<size_t>& iteration)
{
  bool              started = false;
  std::stringstream foo;

  this -> iteration = iteration;

  foo << "Iteration ";
  for (int i = (iteration.size() - 1); i >= 0; i--) {
    if (started || (iteration[i] > 0)) {
      started = true;
      foo << iteration[i];
    }
  }

  SetDlgItemText(handle_progress, IDC_STATIC_ITERATION, (foo.str()).c_str());
}

void
MesoRD::WoeProgressBar::setTime(const double time)
{
  if (time > 0) {
	this -> time = time;
	std::stringstream foo;

    foo << "Time " << time << " seconds";
    SetDlgItemText(handle_progress, IDC_STATIC_TIME, (foo.str()).c_str());
  }
}

void
MesoRD::WoeProgressBar::setTitle(const std::string& title)
{
}


bool
MesoRD::WoeProgressBar::isAlive()
{
	return alive;
}
  
	/* Break out the bulk into its own method.
   */
void
MesoRD::WoeProgressBar::run()
{
	MSG Msg;

	while(GetMessage(&Msg, NULL, 0, 0) > 0) {
    if(!IsDialogMessage(handle_progress, &Msg)) {
            
//		MessageBox(NULL, "Processesd progress message", "Warning!",  
//                MB_OK | MB_ICONINFORMATION);

			TranslateMessage(&Msg);
        DispatchMessage(&Msg);
    }
	}
}

void
MesoRD::WoeProgressBar::stop()
{
  ShowWindow(handle_progress, SW_HIDE);
  alive = false;
}

