/* The TestAnnotation class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestParameter.cpp 541 2008-04-12 18:52:41Z davidfange $
 */

#include "TestAnnotation.hpp"
#include "../src/Unit.hpp"
#include "../src/UnitTable.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestAnnotation );


void 
TestAnnotation::setUp()
{
  std::string test_filename = "./sbml_source/TestAnnotation.xml";
  SBMLDocument* d = readSBML(test_filename.c_str());
  m = d -> getModel();
}


void 
TestAnnotation::tearDown()
{}


void 
TestAnnotation::testConstructor()
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_constructor");
  
  CPPUNIT_ASSERT(a1.getName() == "test_constructor");
  
  MesoRD::Annotation a2 = 
    MesoRD::Annotation( m -> getCompartment(0) -> getAnnotation());
  
  CPPUNIT_ASSERT( a2.getName() == "");
  CPPUNIT_ASSERT( a2.getNumChildren() == 1);
  
  const MesoRD::Annotation* a3 = a2.getChild(0);
  
  CPPUNIT_ASSERT( a3 -> getNumChildren() == 1);
  CPPUNIT_ASSERT( a3 -> getName() == "translation");
  CPPUNIT_ASSERT( a3 -> getStringAttribute("x") == "-1");
  CPPUNIT_ASSERT( a3 -> getStringAttribute("x") == "-1");
  CPPUNIT_ASSERT( a3 -> getStringAttribute("y") == "0");
  CPPUNIT_ASSERT( a3 -> getStringAttribute("z") == "-1");
  CPPUNIT_ASSERT( a3 -> getStringAttribute("units") == "um");
  
  const MesoRD::Annotation* a4 = a3 -> getChild(0);
  CPPUNIT_ASSERT( a4 -> getNumChildren() == 0);
  CPPUNIT_ASSERT( a4 -> getName() == "box");
  CPPUNIT_ASSERT( a4 -> getStringAttribute("x") == "0.4");
  CPPUNIT_ASSERT( a4 -> getStringAttribute("y") == "0.4");
  CPPUNIT_ASSERT( a4 -> getStringAttribute("z") == "0.4");
  CPPUNIT_ASSERT( a4 -> getStringAttribute("units") == "um");
  
  MesoRD::Annotation a5 = 
    MesoRD::Annotation( m -> getCompartment(1) -> getAnnotation());
  
  const MesoRD::Annotation* a6 = a5.getChild(0);
  
  CPPUNIT_ASSERT( a6 -> getNumChildren() == 2);
}


void
TestAnnotation::testGetSet()
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");
  
  a1.addAttribute("test1", "true");
  a1.addAttribute("test2", "false");
  
  CPPUNIT_ASSERT(a1.getStringAttribute("test1") == "true");
  CPPUNIT_ASSERT(a1.getStringAttribute("test2") == "false");

  CPPUNIT_ASSERT(a1.getBoolAttribute("test1"));
  CPPUNIT_ASSERT(!a1.getBoolAttribute("test2"));

  a1.addAttribute("test3", "42");
  
  CPPUNIT_ASSERT( a1.getStringAttribute("test3") == "42");
  CPPUNIT_ASSERT_DOUBLES_EQUAL( a1.getDoubleAttribute("test3"), 
    (double) 42.0, std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL( a1.getFloatAttribute("test3"), 
    (float) 42.0, std::numeric_limits<double>::epsilon());
  

  MesoRD::Annotation* a2 = new MesoRD::Annotation("test_anno_child");
  a1.addChild(a2);
  
  CPPUNIT_ASSERT(a1.getNumChildren() == 1);
  CPPUNIT_ASSERT(a1.getChild(0) -> getName() == "test_anno_child");
  
  
}


void
TestAnnotation::testGetBoolAttributeThrow1() 
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");

  a1.addAttribute("test2", "foo");
  
  a1.getBoolAttribute("test2");
  
}


void
TestAnnotation::testGetBoolAttributeThrow2() 
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");

  a1.addAttribute("test2", "foo");
  
  a1.getBoolAttribute("test3");
  
}

void
TestAnnotation::testGetDoubleAttributeThrow1() 
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");

  a1.addAttribute("test2", "foo");
  
  a1.getDoubleAttribute("test2");
  
}


void
TestAnnotation::testGetDoubleAttributeThrow2() 
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");

  a1.addAttribute("test2", "42.42");
  
  a1.getDoubleAttribute("test3");
  
}


void
TestAnnotation::testGetFloatAttributeThrow1() 
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");

  a1.addAttribute("test2", "foo");
  
  a1.getFloatAttribute("test2");
  
}


void
TestAnnotation::testGetFloatAttributeThrow2() 
{
  MesoRD::Annotation a1 = MesoRD::Annotation("test_anno");

  a1.addAttribute("test2", "42.42");
  
  a1.getFloatAttribute("test3");
  
}


/*
void 
TestAnnotation::testIsFunctions()
{}
*/
