/* The TestCompartmentTable class.
 *
 * Copyright 2004-2005 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestCompartmentTable.cpp 553 2008-05-09 13:28:22Z davidfange $
 */

#include "TestCompartmentTable.hpp"
#include "../src/CompartmentTable.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestCompartmentTable );


void 
TestCompartmentTable::setUp()
{
  std::string test_filename 
    = "./sbml_source/TestCompartment.xml";
  SBMLDocument* d = readSBML(test_filename.c_str());
  m = d -> getModel();
  uds = m -> getListOfUnitDefinitions();
  
  std::map<std::string, double> diffConst;
  MesoRD::Species s(diffConst, "A1", "A1","comp", ((unsigned int)14), false);
  st.addSpecies(s);
  MesoRD::Species s1(diffConst, "A2", "A2","comp2", ((unsigned int)14), false);
  st.addSpecies(s1);

  ut = MesoRD::UnitTable(uds);
  MesoRD::Parameter p1(m -> getParameter(0), &ut);
  MesoRD::Parameter p2(m -> getParameter(1), &ut);
  MesoRD::Parameter p3(m -> getParameter(2), &ut);
  MesoRD::Parameter p4(m -> getParameter(3), &ut);
  MesoRD::Parameter p5("volume_subVolume", "volume_subVolume", 
    1e-8, MesoRD::Unit::litre(), 
    true);
  MesoRD::Parameter p6("p6", "p6", 4, MesoRD::Unit::litre(), false);
  MesoRD::Parameter p7("p7", "p7", (1e-8)*MesoRD::Constant::Avogadro, 
    (MesoRD::Unit::litre()/MesoRD::Unit::mole()), true);
  
  pt.addParameter(p1);
  pt.addParameter(p2);
  pt.addParameter(p3);
  pt.addParameter(p4);
  pt.addParameter(p5);
  pt.addParameter(p6);
  pt.addParameter(p7);
}


void 
TestCompartmentTable::tearDown()
{}


void 
TestCompartmentTable::generateSubvolumes() 
{
  MesoRD::CompartmentTable ct;
  
  std::vector<MesoRD::SubVolume*> subvolumes = 
    ct.fromSBML(m, &pt, &ut, 0.1, "um", NULL);

  CPPUNIT_ASSERT_EQUAL((int)subvolumes.size(), (int) 64);
  
}
