/* The TestDerivative class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestDerivative.hpp 550 2008-05-07 14:24:19Z davidfange $
 */
#ifndef MESORD_TESTDERIVATIVE_HPP
#define MESORD_TESTDERIVATIVE_HPP

#include <cppunit/extensions/HelperMacros.h>

#include "../src/Derivative.hpp"
#include "../src/SpeciesTable.hpp"
#include "../src/UnitTable.hpp"
#include "../src/ParameterTable.hpp"
#include "../src/SubVolume.hpp"

#include <sbml/SBMLReader.h>

class TestDerivative : public CppUnit::TestFixture
{

  CPPUNIT_TEST_SUITE( TestDerivative );
  CPPUNIT_TEST( testGetCurrentState );
  CPPUNIT_TEST( testSetCurrentState );
  CPPUNIT_TEST( testGetReactionDerivative );
  CPPUNIT_TEST( testGetDiffusionMatrix );
  CPPUNIT_TEST_SUITE_END();

private:

  Model* m;
  ListOfUnitDefinitions* uds;

  MesoRD::SpeciesTable st;
  MesoRD::ParameterTable pt;
  MesoRD::UnitTable ut;
  MesoRD::CompartmentTable ct;
  MesoRD::Derivative derivative;
  
  std::valarray<unsigned int> val_int;
  std::valarray<double> val_double;

  std::vector<MesoRD::SubVolume*> subvolumes;
  
public:
  void setUp();
  void tearDown();
  void testGetCurrentState();
  void testSetCurrentState();
  void testGetReactionDerivative();
  void testGetDiffusionMatrix();
  
};

#endif // MESORD_TESTDERIVATIVE_HPP
