/* The TestIntegration class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestIntegration.hpp 550 2008-05-07 14:24:19Z davidfange $
 */
#ifndef MESORD_TESTINTEGRATION_HPP
#define MESORD_TESTINTEGRATION_HPP

#include <cppunit/extensions/HelperMacros.h>

#include "../src/Derivative.hpp"
#include "../src/integration/Integration.hpp"
#include "../src/SpeciesTable.hpp"
#include "../src/UnitTable.hpp"
#include "../src/ParameterTable.hpp"
#include "../src/SubVolume.hpp"

#include <sbml/SBMLReader.h>

class TestIntegration : public CppUnit::TestFixture
{

  CPPUNIT_TEST_SUITE( TestIntegration );
  CPPUNIT_TEST( testGetSet );
  CPPUNIT_TEST( testEuler );
  CPPUNIT_TEST( testEulerBackward );
  CPPUNIT_TEST( testTrapezoid );
  CPPUNIT_TEST( testRungeKutta4 );
  CPPUNIT_TEST( testBDF2 );
  CPPUNIT_TEST_SUITE_END();

private:

  Model* m;
  ListOfUnitDefinitions* uds;

  MesoRD::SpeciesTable st;
  MesoRD::ParameterTable pt;
  MesoRD::UnitTable ut;
  MesoRD::CompartmentTable ct;
  MesoRD::Derivative derivative;
  

  std::vector<MesoRD::SubVolume*> subvolumes;
  
public:
  void setUp();
  void tearDown();
  void testGetSet();
  void testEuler();
  void testEulerBackward();
  void testTrapezoid();
  void testRungeKutta4();
  void testBDF2();
  
};

#endif // MESORD_TESTINTEGRATION_H
