/* The TestLinearFactor class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestLinearFactor.cpp 550 2008-05-07 14:24:19Z davidfange $
 */

#include "TestLinearFactor.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestLinearFactor );

void
TestLinearFactor::setUp()
{}

void 
TestLinearFactor::tearDown()
{}

void
TestLinearFactor::testConstructor()
{

  MesoRD::LinearFactor lf;
  CPPUNIT_ASSERT(lf.getExponent() == 0);
  CPPUNIT_ASSERT(lf.getFactor() == 0);
  CPPUNIT_ASSERT(lf.getOffset() == 0);
  
  
  MesoRD::LinearFactor lf2(1, 2.5, 3, 4);
  CPPUNIT_ASSERT(lf2.getExponent() == 1);
  CPPUNIT_ASSERT(lf2.getFactor() == 2.5); 
  CPPUNIT_ASSERT(lf2.getOffset() == 4);
}

void
TestLinearFactor::testGetSet()
{
  
  MesoRD::LinearFactor lf;
  
  lf.setExponent(1);
  lf.setFactor(2.5);
  lf.setOffset(4);
  
  CPPUNIT_ASSERT(lf.getExponent() == 1);
  CPPUNIT_ASSERT(lf.getFactor() == 2.5);
  CPPUNIT_ASSERT(lf.getOffset() == 4);
} 

void
TestLinearFactor::testEvaluate()
{
  
  MesoRD::LinearFactor lf(1, 2.5, 1, 4);
  
  std::valarray<unsigned int> val_int(2);
  std::valarray<double> val_double(2);
  
  val_int[0] = 2;
  val_int[1] = 4;
  val_double[0] = 2;
  val_double[1] = 4;
  
  CPPUNIT_ASSERT(lf.evaluate(val_int) == 20);
  CPPUNIT_ASSERT(lf.evaluate(val_double) == 20);
  
  lf.setExponent(0);
  
  CPPUNIT_ASSERT(lf.evaluate(val_int) == 2.5);
  CPPUNIT_ASSERT(lf.evaluate(val_double) == 2.5);


  MesoRD::LinearFactor lf2(2, 2, 0, 0);
  lf.appendNext(&lf2);
  
  CPPUNIT_ASSERT(lf2.evaluate(val_int) == 8);
  CPPUNIT_ASSERT(lf2.evaluate(val_double) == 8);
  
  CPPUNIT_ASSERT(lf.evaluate(val_int) == 20);
  CPPUNIT_ASSERT(lf.evaluate(val_double) == 20);
  
}

  

