/* The TestLinearTerm class.
 *
 * Copyright 2004-2005 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestLinearTerm.cpp 541 2008-04-12 18:52:41Z davidfange $
 */

#include "TestLinearTerm.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestLinearTerm );

void
TestLinearTerm::setUp()
{}

void 
TestLinearTerm::tearDown()
{}

void
TestLinearTerm::testConstructor()
{

  MesoRD::LinearTerm lt;  
  
  MesoRD::LinearFactor lf(1, 2.5, 3, 4);
  
  MesoRD::LinearTerm lt2((double)3, &lf);

  CPPUNIT_ASSERT(lt.getCoefficient() == 0);
  CPPUNIT_ASSERT(lt.getNext() == 0);

  CPPUNIT_ASSERT(lt2.getCoefficient() == 3);
  
}

void
TestLinearTerm::testGetSet()
{
  MesoRD::LinearTerm lt;
  MesoRD::LinearTerm lt2;
  

  lt.setNext(&lt2);
  lt.setCoefficient((double)3);
  

  CPPUNIT_ASSERT(lt.getCoefficient() == 3);
  CPPUNIT_ASSERT(lt.getNext() == &lt2);
}


void
TestLinearTerm::testEvaluate()
{
  
  MesoRD::LinearFactor lf(1, 2.5, 0, 0);
  
  std::valarray<unsigned int> val_int(2);
  std::valarray<double> val_double(2);
  
  val_int[0] = 2;
  val_int[1] = 4;
  val_double[0] = 2;
  val_double[1] = 4;

  MesoRD::LinearFactor lf2(2, 2, 0, 0);
  
  MesoRD::LinearTerm lt((double)2, &lf);
  MesoRD::LinearTerm lt2((double)1, &lf2);

  CPPUNIT_ASSERT(lt.evaluate(val_int) == 10);
  CPPUNIT_ASSERT(lt.evaluate(val_double) == 10);

  CPPUNIT_ASSERT(lt2.evaluate(val_int) == 8);
  CPPUNIT_ASSERT(lt2.evaluate(val_double) == 8);
  
  lt.appendNext(&lt2);
  
  
  CPPUNIT_ASSERT(lt.evaluate(val_int) == 18);
  CPPUNIT_ASSERT(lt.evaluate(val_double) == 18);
  
}

  

