#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TextTestProgressListener.h>
#include <cppunit/TestResult.h>

#include <string>

class MyProgressListener : public CppUnit::TextTestProgressListener
{
  void startTest(CppUnit::Test *test) {
    std::string name = test->getName();
    
    std::cout << "\tRunning: " << name << std::endl;
  }
  
  void startSuite(CppUnit::Test *test) {
    std::string name = test->getName();
    std::cout << "Running Suite: " << name << std::endl;
  }
  
  void endSuite(CppUnit::Test *test) {
    std::string name = test->getName();
    std::cout << "Finished Suite: " << name << std::endl;
  }
  
};


int main(int argc, char* argv[])
{
  
  
  // Get the top level suite from the registry
  CppUnit::Test *suite = 
    CppUnit::TestFactoryRegistry::getRegistry().makeTest();

  // Adds the test to the list of test to run
  CppUnit::TextUi::TestRunner runner;
  runner.addTest( suite );

  // Change the default outputter to a compiler error format outputter
  CppUnit::CompilerOutputter *compOut = 
    new CppUnit::CompilerOutputter( &runner.result(),std::cerr );

  runner.setOutputter( compOut );
  // Run the tests.

  MyProgressListener progress;
  runner.eventManager().addListener(&progress);
  
  bool wasSucessful = runner.run();

  // Return error code 1 if the one of test failed.
  return wasSucessful ? 0 : 1;
}
