/* The TestParameter class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestParameter.cpp 550 2008-05-07 14:24:19Z davidfange $
 */

#include "TestParameter.hpp"
#include "../src/Unit.hpp"
#include "../src/UnitTable.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestParameter );


void 
TestParameter::setUp()
{
  std::string test_filename = "./sbml_source/TestParameter.xml";
  SBMLDocument* d = readSBML(test_filename.c_str());
  m = d -> getModel();
}


void 
TestParameter::tearDown()
{}


void 
TestParameter::testConstructor()
{

  MesoRD::Parameter p;
  
  CPPUNIT_ASSERT(p.getName() == "");
  CPPUNIT_ASSERT(p.getId() == "");
  CPPUNIT_ASSERT(p.getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(p.getValue() == 0);

  MesoRD::Parameter p2("test_id", "test_name", 45.8, MesoRD::Unit::metre(), 
    false);
  
  CPPUNIT_ASSERT(p2.getName() == "test_name");
  CPPUNIT_ASSERT(p2.getId() == "test_id");
  CPPUNIT_ASSERT(p2.getUnit() == MesoRD::Unit::metre());
  CPPUNIT_ASSERT(p2.getValue() == 45.8);

  MesoRD::UnitTable ut( m -> getListOfUnitDefinitions() );
  
  MesoRD::Parameter p3( m -> getParameter(0) , &ut);
  
  CPPUNIT_ASSERT(p3.getId() == "test_parameter");
  CPPUNIT_ASSERT(p3.getUnit() == MesoRD::Unit::metre());
  CPPUNIT_ASSERT(p3.getValue() == 100);
  
  MesoRD::Parameter p4( m -> getParameter(1), &ut);
  
  CPPUNIT_ASSERT(p4.getId() == "test_dimensionless");
  CPPUNIT_ASSERT(p4.getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(p4.getValue() == 250);

  MesoRD::Parameter p5( m -> getParameter(2), &ut);
    
  CPPUNIT_ASSERT(p5.getId() == "test_empty");
  CPPUNIT_ASSERT(p5.getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(p5.getValue() == 0);
}

void
TestParameter::testGetSet()
{
  
  MesoRD::Parameter p;
  
  p.setId("test_id");
  p.setName("test_name");
  p.setUnit(MesoRD::Unit::centimetreSquaredPerSecond());
  p.setValue(34.7);
  

  CPPUNIT_ASSERT(p.getId() == "test_id");
  CPPUNIT_ASSERT(p.getName() == "test_name");
  CPPUNIT_ASSERT(p.getUnit() == MesoRD::Unit::centimetreSquaredPerSecond());
  CPPUNIT_ASSERT(p.getValue() == 34.7);
}


void 
TestParameter::testIsFunctions()
{
  MesoRD::Parameter p("test_id", "test_name", 45, MesoRD::Unit::metre(), 
    false);
  
  CPPUNIT_ASSERT(p.isId("test_id"));
  CPPUNIT_ASSERT(p.isName("test_name"));
  CPPUNIT_ASSERT(!p.isConstant());
}
