
/* The TestReaction class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestReaction.hpp 550 2008-05-07 14:24:19Z davidfange $
 */
#ifndef MESORD_TESTREACTION_HPP
#define MESORD_TESTREACTION_HPP

#include <cppunit/extensions/HelperMacros.h>

#include "../src/reaction/Reaction.hpp"
#include "../src/SpeciesTable.hpp"
#include "../src/UnitTable.hpp"
#include "../src/ParameterTable.hpp"

#include <sbml/SBMLReader.h>

class TestReaction : public CppUnit::TestFixture
{

  CPPUNIT_TEST_SUITE( TestReaction );
  CPPUNIT_TEST( testGetCompartment );
  CPPUNIT_TEST_EXCEPTION( testGetCompartmentThrow, 
    MesoRD::Reaction::ModelError );
  CPPUNIT_TEST( testFromSBML );
  CPPUNIT_TEST( testGetRate );
  CPPUNIT_TEST( testGetMultiSVReaction );
  CPPUNIT_TEST( testGetReactants );
  

  CPPUNIT_TEST_SUITE_END();

private:

  Model* m;
  ListOfUnitDefinitions* uds;

  MesoRD::SpeciesTable st;
  MesoRD::ParameterTable pt;
  
  std::valarray<unsigned int> val_int;
  std::valarray<double> val_double;

  
public:
  void setUp();
  void tearDown();
  void testGetCompartment();
  void testGetCompartmentThrow();
  void testFromSBML();
  
  void testGetRate();
  void testGetMultiSVReaction();
  void testGetReactants();
  
  
};

#endif // MESORD_TESTREACTION_H
