/* The TestReactionNode class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestReactionNode.hpp 553 2008-05-09 13:28:22Z davidfange $
 */
#ifndef MESORD_TESTREACTIONNODE_H
#define MESORD_TESTREACTIONNODE_H 1

#include <cppunit/extensions/HelperMacros.h>

#include "../src/reaction/ReactionNode.hpp"
#include "../src/SpeciesTable.hpp"
#include "../src/ParameterTable.hpp"

#include <sbml/SBMLReader.h>

class TestReactionNode : public CppUnit::TestFixture
{

  CPPUNIT_TEST_SUITE( TestReactionNode );
  CPPUNIT_TEST( testAbs );
  CPPUNIT_TEST( testCeil );
  CPPUNIT_TEST( testExp );
  CPPUNIT_TEST( testFactorial );
  CPPUNIT_TEST( testFloor );
  CPPUNIT_TEST( testLn );
  CPPUNIT_TEST( testLog );
  CPPUNIT_TEST( testPower );
  CPPUNIT_TEST( testRoot );
  CPPUNIT_TEST( testAnd );
  CPPUNIT_TEST( testNot );
  CPPUNIT_TEST( testOr );
  CPPUNIT_TEST( testXor );
  CPPUNIT_TEST( testAddition );
  CPPUNIT_TEST( testDivision );
  CPPUNIT_TEST( testMultiplication );
  CPPUNIT_TEST( testSubtraction );
  CPPUNIT_TEST( testEq );
  CPPUNIT_TEST( testGeq );
  CPPUNIT_TEST( testGt );
  CPPUNIT_TEST( testLeq );
  CPPUNIT_TEST( testLt );
  CPPUNIT_TEST( testNeq );
  CPPUNIT_TEST( testConcentration );
  CPPUNIT_TEST( testParameterConstant );
  CPPUNIT_TEST( testParameter );
  CPPUNIT_TEST( testFoldAnd );
  CPPUNIT_TEST( testFoldNot );
  CPPUNIT_TEST( testFoldOr );
//  CPPUNIT_TEST( testFoldXor);
  CPPUNIT_TEST( testFoldAdditionA1 );
  CPPUNIT_TEST( testFoldAdditionA2 );
  CPPUNIT_TEST( testFoldAdditionA3 );
  CPPUNIT_TEST( testFoldAdditionA4 );
  CPPUNIT_TEST( testFoldAdditionA5 );
  CPPUNIT_TEST( testFoldAdditionA6 );
  CPPUNIT_TEST( testFoldAdditionA7 );
  CPPUNIT_TEST( testFoldAdditionA8 );
  CPPUNIT_TEST( testFoldAdditionA9 );
  CPPUNIT_TEST( testFoldAdditionA10 );
  CPPUNIT_TEST( testFoldAdditionA11 );
  CPPUNIT_TEST( testFoldAdditionA12 );
  CPPUNIT_TEST( testFoldDivisionD1 );
  CPPUNIT_TEST( testFoldDivisionD2 );
  CPPUNIT_TEST( testFoldDivisionD3 );
  CPPUNIT_TEST( testFoldDivisionD4 );
  CPPUNIT_TEST( testFoldDivisionD5 );
  CPPUNIT_TEST( testFoldDivisionD6 );
  CPPUNIT_TEST( testFoldDivisionD7 );
  CPPUNIT_TEST( testFoldDivisionD8 );
  CPPUNIT_TEST( testFoldDivisionD9 );
  CPPUNIT_TEST( testFoldDivisionD10 );
  CPPUNIT_TEST( testFoldDivisionD11 );
  CPPUNIT_TEST( testFoldDivisionD12 );
  CPPUNIT_TEST( testFoldDivisionD13 );
  CPPUNIT_TEST( testFoldDivisionD14 );
  CPPUNIT_TEST( testFoldDivisionD15 );
  CPPUNIT_TEST( testFoldDivisionD16 );
  CPPUNIT_TEST( testFoldMultiplicationM1 );
  CPPUNIT_TEST( testFoldMultiplicationM2 );
  CPPUNIT_TEST( testFoldMultiplicationM3 );
  CPPUNIT_TEST( testFoldMultiplicationM4 );
  CPPUNIT_TEST( testFoldMultiplicationM5 );
  CPPUNIT_TEST( testFoldMultiplicationM6 );
  CPPUNIT_TEST( testFoldMultiplicationM7 );
  CPPUNIT_TEST( testFoldMultiplicationM8 );
  CPPUNIT_TEST( testFoldMultiplicationM9 );
  CPPUNIT_TEST( testFoldMultiplicationM10 );
  CPPUNIT_TEST( testFoldMultiplicationM11 );
  CPPUNIT_TEST( testFoldMultiplicationM12 );
  CPPUNIT_TEST( testFoldMultiplicationM13 );
  CPPUNIT_TEST( testFoldSubtractionS1 );
  CPPUNIT_TEST( testFoldSubtractionS2 );
  CPPUNIT_TEST( testFoldSubtractionS3 );
  CPPUNIT_TEST( testFoldSubtractionS4 );
  CPPUNIT_TEST( testFoldSubtractionS5 );
  CPPUNIT_TEST( testFoldSubtractionS6 );
  CPPUNIT_TEST( testFoldSubtractionS7 );
  CPPUNIT_TEST( testFoldSubtractionS8 );
  CPPUNIT_TEST( testFoldSubtractionS9 );
  CPPUNIT_TEST( testFoldSubtractionS10 );
  CPPUNIT_TEST( testFoldEq );
  CPPUNIT_TEST( testFoldGeq );
  CPPUNIT_TEST( testFoldGt );
  CPPUNIT_TEST( testFoldLeq );
  CPPUNIT_TEST( testFoldLt );
  CPPUNIT_TEST( testFoldNeq );

  CPPUNIT_TEST_SUITE_END();

private:

  Model* m;
  ListOfUnitDefinitions* uds;

  MesoRD::SpeciesTable st;
  MesoRD::ParameterTable pt;
  
  std::valarray<unsigned int> val_int;
  std::valarray<double> val_double;


public:
  void setUp();
  void tearDown();
  
  void testAbs();
  void testCeil();
  void testExp();
  void testFactorial();
  void testFloor();
  void testLn();
  void testLog();
  void testPower();
  void testRoot();
  void testAnd();
  void testNot();
  void testOr();
  void testXor();
  void testAddition();
  void testDivision();
  void testMultiplication();
  void testSubtraction();
  void testEq();
  void testGeq();
  void testGt();
  void testLeq();
  void testLt();
  void testNeq();
  void testConcentration();
  void testParameterConstant();
  void testParameter();
  void testFoldAnd();
  void testFoldNot();
  void testFoldOr();
//  void testFoldXor();
  void testFoldAdditionA1();
  void testFoldAdditionA2();
  void testFoldAdditionA3();
  void testFoldAdditionA4();
  void testFoldAdditionA5();
  void testFoldAdditionA6();
  void testFoldAdditionA7();
  void testFoldAdditionA8();
  void testFoldAdditionA9();
  void testFoldAdditionA10();
  void testFoldAdditionA11();
  void testFoldAdditionA12();
  void testFoldDivisionD1();
  void testFoldDivisionD2();
  void testFoldDivisionD3();
  void testFoldDivisionD4();
  void testFoldDivisionD5();
  void testFoldDivisionD6();
  void testFoldDivisionD7();
  void testFoldDivisionD8();
  void testFoldDivisionD9();
  void testFoldDivisionD10();
  void testFoldDivisionD11();
  void testFoldDivisionD12();
  void testFoldDivisionD13();
  void testFoldDivisionD14();
  void testFoldDivisionD15();
  void testFoldDivisionD16();
  void testFoldMultiplicationM1();
  void testFoldMultiplicationM2();
  void testFoldMultiplicationM3();
  void testFoldMultiplicationM4();
  void testFoldMultiplicationM5();
  void testFoldMultiplicationM6();
  void testFoldMultiplicationM7();
  void testFoldMultiplicationM8();
  void testFoldMultiplicationM9();
  void testFoldMultiplicationM10();
  void testFoldMultiplicationM11();
  void testFoldMultiplicationM12();
  void testFoldMultiplicationM13();
  void testFoldSubtractionS1();
  void testFoldSubtractionS2();
  void testFoldSubtractionS3();
  void testFoldSubtractionS4();
  void testFoldSubtractionS5();
  void testFoldSubtractionS6();
  void testFoldSubtractionS7();
  void testFoldSubtractionS8();
  void testFoldSubtractionS9();
  void testFoldSubtractionS10();
  void testFoldEq();
  void testFoldGeq();
  void testFoldGt();
  void testFoldLeq();
  void testFoldLt();
  void testFoldNeq();
};

#endif  // MESORD_TESTREACTIONNODE_HPP
