/* The TestReactionNode class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestReactionNodeAlgebraicFolds.cpp 550 2008-05-07 14:24:19Z davidfange $
 */

#include "TestReactionNode.hpp"


/*
  Addition folds (A1-A12)
*/
void
TestReactionNode::testFoldAdditionA1()
{
  Reaction_t* r = Model_getReaction(m,34);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == 6);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == 6);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == 6);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == 6);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
}

void
TestReactionNode::testFoldAdditionA2()
{
  Reaction_t* r = Model_getReaction(m,35);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 10.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 9.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 10.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 9.9999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild()));  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isAddition(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild();
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 10.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 9.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 10.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 9.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
}


void
TestReactionNode::testFoldAdditionA3()
{
  Reaction_t* r = Model_getReaction(m,36);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 8.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 7.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 8.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 7.9999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild()));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn2) -> getLeftChild()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 8.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 7.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 8.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 7.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
}

void
TestReactionNode::testFoldAdditionA4()
{
  Reaction_t* r = Model_getReaction(m,37);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 3.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 2.9999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild()));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn2) -> getRightChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 2.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
}


void
TestReactionNode::testFoldAdditionA5()
{
  Reaction_t* r = Model_getReaction(m,38);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 7.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 6.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 7.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 6.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isAddition(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 7.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 6.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 7.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 6.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
}


void
TestReactionNode::testFoldAdditionA6()
{
  Reaction_t* r = Model_getReaction(m,39);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 7.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 6.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 7.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 6.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isAddition(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 7.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 6.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 7.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 6.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
}


void
TestReactionNode::testFoldAdditionA7()
{
  Reaction_t* r = Model_getReaction(m,40);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 5.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 4.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 4.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());  
}

void
TestReactionNode::testFoldAdditionA8()
{
  Reaction_t* r = Model_getReaction(m,41);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;


  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);

    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getRightChild()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  rn = MesoRD::ReactionNode::refold(rn);


  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());  
}

void
TestReactionNode::testFoldAdditionA9()
{
  Reaction_t* r = Model_getReaction(m,42);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 5.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 4.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getLeftChild()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());


  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 4.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());    
}

void
TestReactionNode::testFoldAdditionA10()
{
  Reaction_t* r = Model_getReaction(m,43);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeAddition*)rn) -> getRightChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeAddition*)rn2) -> getRightChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
}

void
TestReactionNode::testFoldAdditionA11()
{
  Reaction_t* r = Model_getReaction(m,44);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 5.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 4.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getRightChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild())));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 5.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 4.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());    
}


void
TestReactionNode::testFoldAdditionA12()
{
  Reaction_t* r = Model_getReaction(m,45);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAddition));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn) -> getLeftChild())));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());  
}


/*
 * Division folds (D1-D16)
 */
void
TestReactionNode::testFoldDivisionD1()
{
  Reaction_t* r = Model_getReaction(m,46);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) -> getLeftChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
}

void
TestReactionNode::testFoldDivisionD2()
{
  Reaction_t* r = Model_getReaction(m,47);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 25.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 24.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 25.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 24.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) -> getRightChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 25.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 24.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 25.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 24.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild()));
  
}


void
TestReactionNode::testFoldDivisionD3()
{
  Reaction_t* r = Model_getReaction(m,48);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild()));
  
}

void
TestReactionNode::testFoldDivisionD4()
{
  Reaction_t* r = Model_getReaction(m,49);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild()));
  
}

void
TestReactionNode::testFoldDivisionD5()
{
  Reaction_t* r = Model_getReaction(m,50);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= .1001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.09999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.100001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.09999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getRightChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.100001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.099999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.100001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.099999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild()));
  
}


void
TestReactionNode::testFoldDivisionD6()
{
  Reaction_t* r = Model_getReaction(m,51);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= .4001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.39999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.400001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.39999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.400001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.399999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.400001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.399999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
  
}


void
TestReactionNode::testFoldDivisionD7()
{
  Reaction_t* r = Model_getReaction(m,52);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
  
}



void
TestReactionNode::testFoldDivisionD8()
{
  Reaction_t* r = Model_getReaction(m,53);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= .50001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getRightChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
  
}


void
TestReactionNode::testFoldDivisionD9()
{
  Reaction_t* r = Model_getReaction(m,54);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= .50001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));
    
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
  
}




void
TestReactionNode::testFoldDivisionD10()
{
  Reaction_t* r = Model_getReaction(m,55);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getRightChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
                   
}

void
TestReactionNode::testFoldDivisionD11()
{
  Reaction_t* r = Model_getReaction(m,56);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
                   
}

void
TestReactionNode::testFoldDivisionD12()
{
  Reaction_t* r = Model_getReaction(m,57);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.50001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
    
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeDivision*)rn) -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= .500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild()));
                   
}

void 
TestReactionNode::testFoldDivisionD13()
{
  Reaction_t* r = Model_getReaction(m,58);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 3.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 2.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 2.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getLeftChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getRightChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(rn));
}


void 
TestReactionNode::testFoldDivisionD14()
{
  Reaction_t* r = Model_getReaction(m,59);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= -0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= -0.0001);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getLeftChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= -0.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= -0.00001);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(rn));
}

void 
TestReactionNode::testFoldDivisionD15()
{
  Reaction_t* r = Model_getReaction(m,60);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getRightChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == 
    std::numeric_limits<double>::infinity());
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == 
    std::numeric_limits<double>::infinity());
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(rn));
}

void
TestReactionNode::testFoldDivisionD16()
{
 Reaction_t* r = Model_getReaction(m,61);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeDivision));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getRightChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
}




/*
 * Mutiplication folds (M1-M13)
 */

void
TestReactionNode::testFoldMultiplicationM1()
{
 Reaction_t* r = Model_getReaction(m,62);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 2.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 1.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));
  
}

void
TestReactionNode::testFoldMultiplicationM2()
{
 Reaction_t* r = Model_getReaction(m,63);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeMultiplication*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM3()
{
 Reaction_t* r = Model_getReaction(m,64);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM4()
{
 Reaction_t* r = Model_getReaction(m,65);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getRightChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));

}

void
TestReactionNode::testFoldMultiplicationM5()
{
 Reaction_t* r = Model_getReaction(m,66);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM6()
{
 Reaction_t* r = Model_getReaction(m,67);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeDivision*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM7()
{
 Reaction_t* r = Model_getReaction(m,68);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.250001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= .249999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.2500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeMultiplication*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getRightChild())));

  

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.2500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.2500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM8()
{
 Reaction_t* r = Model_getReaction(m,69);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isMultiplication(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeMultiplication*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn2) 
                     -> getLeftChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM9()
{
 Reaction_t* r = Model_getReaction(m,70);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeMultiplication*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 4.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 3.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM10()
{
 Reaction_t* r = Model_getReaction(m,71);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.250001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.2500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isDivision(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getRightChild())));
  
  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeMultiplication*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getRightChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.2500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.2500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.249999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn)
                     -> getLeftChild())));
}

void
TestReactionNode::testFoldMultiplicationM11()
{
Reaction_t* r = Model_getReaction(m,72);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 16.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 15.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 16.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 15.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getRightChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getLeftChild())));

  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 16.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 15.9999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 16.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 15.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(rn));  
}


void
TestReactionNode::testFoldMultiplicationM12()
{
  Reaction_t* r = Model_getReaction(m,73);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= -0.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= -0.00001);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getLeftChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= -0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= -0.00001);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(rn));
}

void
TestReactionNode::testFoldMultiplicationM13()
{
  Reaction_t* r = Model_getReaction(m,74);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeMultiplication));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn) 
                     -> getLeftChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 1.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
}


/*
 * Subtraction nodes (S1-S10)
 */

void
TestReactionNode::testFoldSubtractionS1()
{
  Reaction_t* r = Model_getReaction(m,75);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 0.50001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= .500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= .500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 0.499999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 0.500001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 0.49999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));
}


void
TestReactionNode::testFoldSubtractionS2()
{
  Reaction_t* r = Model_getReaction(m,76);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 3.0001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeSubtraction*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeDivision*)rn2) 
                     -> getLeftChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= 2.99999);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= 3.00001);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= 2.9999);
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));
}


void
TestReactionNode::testFoldSubtractionS3()
{
  Reaction_t* r = Model_getReaction(m,77);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));

  CPPUNIT_ASSERT(MesoRD::ReactionNode::isAddition(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeSubtraction*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn2) 
                     -> getLeftChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));
}


void
TestReactionNode::testFoldSubtractionS4()
{
  Reaction_t* r = Model_getReaction(m,78);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 0;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeSubtraction*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn2) 
                     -> getLeftChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn) 
                     -> getLeftChild())));
}


void
TestReactionNode::testFoldSubtractionS5()
{
  Reaction_t* r = Model_getReaction(m,79);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 2;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isAddition(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeSubtraction*)rn) -> getRightChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn2) 
                     -> getLeftChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn) 
                     -> getLeftChild())));

}


void
TestReactionNode::testFoldSubtractionS6()
{
  Reaction_t* r = Model_getReaction(m,80);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 2;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(rn));
}


void
TestReactionNode::testFoldSubtractionS7()
{
  Reaction_t* r = Model_getReaction(m,81);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = -1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeMultiplication*)rn) 
                     -> getLeftChild())));

}


void
TestReactionNode::testFoldSubtractionS8()
{
 Reaction_t* r = Model_getReaction(m,82);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 1;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getRightChild())));

 
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
    
}


void
TestReactionNode::testFoldSubtractionS9()
{
  Reaction_t* r = Model_getReaction(m,83);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 2;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isAddition(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn2) 
                     -> getLeftChild())));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeAddition*)rn) 
                     -> getLeftChild())));  
}


void
TestReactionNode::testFoldSubtractionS10()
{
  Reaction_t* r = Model_getReaction(m,84);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeSubtraction));

  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  double exact_value = 0;

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isSubtraction(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));

  MesoRD::ReactionNode* rn2 = 
    ((MesoRD::ReactionNodeSubtraction*)rn) -> getLeftChild();
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn2) 
                     -> getLeftChild())));

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(rn -> evaluate(val_int2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) <= (exact_value +
                   std::numeric_limits<double>::epsilon()));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) >= (exact_value -
                   std::numeric_limits<double>::epsilon()));

  CPPUNIT_ASSERT(rn -> getUnit() == MesoRD::Unit::dimensionless());
  
  CPPUNIT_ASSERT(MesoRD::ReactionNode::isConstant(
                   (((MesoRD::ReactionNodeSubtraction*)rn) 
                     -> getLeftChild())));
  
}
