/* The TestReactionNodeException class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestReactionNodeExceptions.cpp 553 2008-05-09 13:28:22Z davidfange $
 */

#include "TestReactionNodeExceptions.hpp"
#include "../src/UnitTable.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestReactionNodeException );


void 
TestReactionNodeException::setUp()
{
  std::string test_filename 
    = "./sbml_source/TestReactionNodeException.xml";
  SBMLDocument* d = readSBML(test_filename.c_str());
  m = d -> getModel();
  uds = m -> getListOfUnitDefinitions();
  
  std::map<std::string, double> diffConst;
  MesoRD::Species s(diffConst, "A1", "A1","comp", ((unsigned int)14), false);
  st.addSpecies(s);

  MesoRD::UnitTable ut(uds);
  MesoRD::Parameter p1(m -> getParameter(0), &ut);
  MesoRD::Parameter p2(m -> getParameter(1), &ut);
  MesoRD::Parameter p3(m -> getParameter(2), &ut);
  MesoRD::Parameter p4(m -> getParameter(3), &ut);
  MesoRD::Parameter p5("volume_subVolume", "volume_subVolume", 
    1e-8, MesoRD::Unit::litre(), 
    true);
  MesoRD::Parameter p6("p6", "p6", 4, MesoRD::Unit::litre(), false);
  MesoRD::Parameter p7("p7", "p7", (1e-8)*MesoRD::Constant::Avogadro, 
    (MesoRD::Unit::litre()/MesoRD::Unit::mole()), true);
  
  pt.addParameter(p1);
  pt.addParameter(p2);
  pt.addParameter(p3);
  pt.addParameter(p4);
  pt.addParameter(p5);
  pt.addParameter(p6);
  pt.addParameter(p7);
}


void 
TestReactionNodeException::tearDown()
{
}
/*
void
TestReactionNodeException::testParameterThrow()
{
  ::Reaction* r = m -> getReaction(0);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = 0;
}

void
TestReactionNodeException::testUnknownThrow()
{
  ::Reaction* r = m -> getReaction(1);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = 0;
}

void
TestReactionNodeException::testAbsGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(2);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testCeilGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(3);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testExpGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(4);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testFloorGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(5);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testFactorialGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(6);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testLnGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(7);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}
*/
void
TestReactionNodeException::testLogGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(0);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testPowerGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(1);
    std::cerr << "reactionid " << r -> getId() << std::endl;
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testRootGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(2);
    std::cerr << "reactionid " << r -> getId() << std::endl;
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testPlusGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(3);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testMinusGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(4);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testEqGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(5);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testGeqGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(6);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testGtGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(7);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testLeqGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(8);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testLtGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(9);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testNeqGetUnitThrow()
{
  ::Reaction* r = m -> getReaction(10);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn -> getUnit();
}

void
TestReactionNodeException::testAbsRefoldThrow()
{
  ::Reaction* r = m -> getReaction(11);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}

void
TestReactionNodeException::testCeilRefoldThrow()
{
  ::Reaction* r = m -> getReaction(12);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st); 
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testExpRefoldThrow()
{
  ::Reaction* r = m -> getReaction(13);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testFactorialRefoldThrow()
{
  ::Reaction* r = m -> getReaction(14);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testFloorRefoldThrow()
{
  ::Reaction* r = m -> getReaction(15);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testLnRefoldThrow()
{
  ::Reaction* r = m -> getReaction(16);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testLogRefoldThrow()
{
  ::Reaction* r = m -> getReaction(17);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testPowerRefoldThrow()
{
  ::Reaction* r = m -> getReaction(18);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testRootRefoldThrow()
{
  ::Reaction* r = m -> getReaction(19);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testAndRefoldThrow()
{
  ::Reaction* r = m -> getReaction(20);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testNotRefoldThrow()
{
  ::Reaction* r = m -> getReaction(21);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testOrRefoldThrow()
{
  ::Reaction* r = m -> getReaction(22);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testXorRefoldThrow()
{
  ::Reaction* r = m -> getReaction(23);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testPlusRefoldThrow()
{
  ::Reaction* r = m -> getReaction(24);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testMinusRefoldThrow()
{
  ::Reaction* r = m -> getReaction(25);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testTimesRefoldThrow()
{
  ::Reaction* r = m -> getReaction(26);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testDivisionRefoldThrow()
{
  ::Reaction* r = m -> getReaction(27);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testEqRefoldThrow()
{
  ::Reaction* r = m -> getReaction(28);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testGeqRefoldThrow()
{
  ::Reaction* r = m -> getReaction(29);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testGtRefoldThrow()
{
  ::Reaction* r = m -> getReaction(30);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}
void
TestReactionNodeException::testLeqRefoldThrow()
{
  ::Reaction* r = m -> getReaction(31);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}


void
TestReactionNodeException::testLtRefoldThrow()
{ 
  ::Reaction* r = m -> getReaction(32);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);
}


void
TestReactionNodeException::testNeqRefoldThrow()
{
  ::Reaction* r = m -> getReaction(33);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}

void
TestReactionNodeException::testArccosRefoldThrow()
{
  ::Reaction* r = m -> getReaction(34);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      (r -> getKineticLaw()) -> getMath(), 
      &pt, &st);
  rn = MesoRD::ReactionNode::refold(rn);
}

