/* The TestReactionNode class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestReactionNodeFolds.cpp 553 2008-05-09 13:28:22Z davidfange $
 */

#include "TestReactionNode.hpp"

void
TestReactionNode::testFoldAnd()
{
  Reaction_t* r = Model_getReaction(m,26);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAnd));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,27);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeAnd));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

}

void
TestReactionNode::testFoldNot()
{
  Reaction* r = m -> getReaction(28);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML((r -> getKineticLaw()) -> getMath(),
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeNot));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
}


void
TestReactionNode::testFoldOr()
{
  Reaction* r = m -> getReaction(29);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML((r -> getKineticLaw()) -> getMath(), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeOr));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  r = m -> getReaction(30);
  rn = MesoRD::ReactionNode::fromSBML((r -> getKineticLaw()) -> getMath(), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeOr));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  

  r = m -> getReaction(31);
  rn = MesoRD::ReactionNode::fromSBML((r -> getKineticLaw()) -> getMath(), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeOr));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);
}
/* !! FIXME
void
TestReactionNode::testFoldXor()
{
  Reaction_t* r = Model_getReaction(m,32);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeXor));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);
  
    // CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,33);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeXor));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);
}
*/
void
TestReactionNode::testFoldEq()
{
  Reaction_t* r = Model_getReaction(m,85);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeEq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,86);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeEq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);
  
  r = Model_getReaction(m,87);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeEq));
  
  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == false);
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeEq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == false);
}

void
TestReactionNode::testFoldGeq()
{
  Reaction_t* r = Model_getReaction(m,88);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGeq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
    
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,89);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGeq));
  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double));
  
  r = Model_getReaction(m,90);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGeq));
  
  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGeq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2));
}

void 
TestReactionNode::testFoldGt()
{
  Reaction_t* r = Model_getReaction(m,91);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGt));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,92);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGt));
  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double));
  
  r = Model_getReaction(m,93);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGt));
  
  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeGt));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2));
}

void 
TestReactionNode::testFoldLeq()
{
  Reaction_t* r = Model_getReaction(m,94);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLeq));
  
    //CPPUNIT_ASSERT((rn -> evaluate(val_int)));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
    
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,95);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLeq));
  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double));

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double));
  
  r = Model_getReaction(m,96);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLeq));
  
  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 3;
  val_double2 = 3;
  
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2));
  CPPUNIT_ASSERT(rn -> evaluate(val_double2));
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLeq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == true);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == true);
}

void
TestReactionNode::testFoldLt()
{
  Reaction_t* r = Model_getReaction(m,97);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLt));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,98);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLt));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);
  
  r = Model_getReaction(m,99);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLt));
  
  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 3;
  val_double2 = 3;
  
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == true);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == true);
  
  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeLt));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == true);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == true);
}

void
TestReactionNode::testFoldNeq()
{
  Reaction_t* r = Model_getReaction(m,100);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeNeq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  
  
  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int));
  CPPUNIT_ASSERT(rn -> evaluate(val_double));
  

  r = Model_getReaction(m,101);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeNeq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);

  rn = MesoRD::ReactionNode::refold(rn);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeConstant));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double) == false);
  
  r = Model_getReaction(m,102);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);

  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeNeq));
  
  std::valarray<unsigned int> val_int2(1);
  std::valarray<double> val_double2(1);
  
  val_int2 = 1;
  val_double2 = 1;
  

  
  CPPUNIT_ASSERT(!rn -> evaluate(val_int2));
  CPPUNIT_ASSERT(!rn -> evaluate(val_double2));

  rn = MesoRD::ReactionNode::refold(rn);
  
  CPPUNIT_ASSERT(typeid(*rn) == typeid(MesoRD::ReactionNodeNeq));
  
  CPPUNIT_ASSERT(rn -> evaluate(val_int2) == false);
  CPPUNIT_ASSERT(rn -> evaluate(val_double2) == false);
}

/* Leaving the constant folding for trig-functions, its all the same.... 
 */
