/* The TestReactionNodeLinearise class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestReactionNodeLinearise.cpp 553 2008-05-09 13:28:22Z davidfange $
 */

#include "TestReactionNodeLinearise.hpp"
#include "../src/reaction/LinearElement.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestReactionNodeLinearise );


void 
TestReactionNodeLinearise::setUp()
{
  std::string test_filename 
    = "./sbml_source/TestReactionNodeLinearise.xml";
  SBMLDocument_t* d = readSBML(test_filename.c_str());
  m = SBMLDocument_getModel(d);
  uds =  m -> getListOfUnitDefinitions();
  
  std::map<std::string, double> diffConst;
  MesoRD::Species s(diffConst, "A1", "A1","comp", ((unsigned int)14), false);
  st.addSpecies(s);
  
  MesoRD::UnitTable ut(uds);
  MesoRD::Parameter p1(Model_getParameter(m, 0), &ut);
  MesoRD::Parameter p2(Model_getParameter(m,1), &ut);
  MesoRD::Parameter p3(Model_getParameter(m,2), &ut);
  MesoRD::Parameter p4(Model_getParameter(m,3), &ut);
  MesoRD::Parameter p5("volume_subVolume", "volume_subVolume", 
    1e-8, MesoRD::Unit::litre(), 
    true);
  MesoRD::Parameter p6("p6", "p6", 4, MesoRD::Unit::litre(), false);
  MesoRD::Parameter p7("p7", "p7", (1e-8)*MesoRD::Constant::Avogadro, 
    (MesoRD::Unit::litre()/MesoRD::Unit::mole()), true);
  
  pt.addParameter(p1);
  pt.addParameter(p2);
  pt.addParameter(p3);
  pt.addParameter(p4);
  pt.addParameter(p5);
  pt.addParameter(p6);
  pt.addParameter(p7);
}


void 
TestReactionNodeLinearise::tearDown()
{}


void
TestReactionNodeLinearise::testAddition()
{

  Reaction_t* r = Model_getReaction(m,0);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  std::valarray<unsigned int> val_int(1);
  std::valarray<double> val_double(1);
  
  val_int[0] = 2;
  val_double[0] = 2;
  
  rn = MesoRD::ReactionNode::refold(rn);  
  MesoRD::LinearElement* le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));

  

  
  r = Model_getReaction(m,1);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));


  r = Model_getReaction(m,3);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();
  
  CPPUNIT_ASSERT(le != 0);
  
  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));




  r = Model_getReaction(m,4);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();
  
  CPPUNIT_ASSERT(le != 0);
  
  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));



  r = Model_getReaction(m,5);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();
  
  CPPUNIT_ASSERT(le != 0);
  
  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));



  r = Model_getReaction(m,6);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();
  
  CPPUNIT_ASSERT(le != 0);
  
  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));

}


void
TestReactionNodeLinearise::testDivision()
{

  Reaction_t* r = Model_getReaction(m,7);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  std::valarray<unsigned int> val_int(1);
  std::valarray<double> val_double(1);
  
  val_int[0] = 2;
  val_double[0] = 2;
  
  rn = MesoRD::ReactionNode::refold(rn);  
  MesoRD::LinearElement* le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  

  r = Model_getReaction(m,8);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  


  r = Model_getReaction(m,9);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  
  

  r = Model_getReaction(m,10);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  

}


void
TestReactionNodeLinearise::testMultiplication()
{

  Reaction_t* r = Model_getReaction(m,11);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  std::valarray<unsigned int> val_int(1);
  std::valarray<double> val_double(1);
  
  val_int[0] = 2;
  val_double[0] = 2;
  
  rn = MesoRD::ReactionNode::refold(rn);  
  MesoRD::LinearElement* le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  

  r = Model_getReaction(m,12);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  


  r = Model_getReaction(m,13);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  
  

  r = Model_getReaction(m,14);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  

}


  
void
TestReactionNodeLinearise::testSubtraction()
{

  Reaction_t* r = Model_getReaction(m,15);
  MesoRD::ReactionNode* rn = 
    MesoRD::ReactionNode::fromSBML(
      KineticLaw_getMath(Reaction_getKineticLaw(r)), 
      &pt, &st);
  
  std::valarray<unsigned int> val_int(1);
  std::valarray<double> val_double(1);
  
  val_int[0] = 2;
  val_double[0] = 2;
  
  rn = MesoRD::ReactionNode::refold(rn);  
  MesoRD::LinearElement* le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  

  r = Model_getReaction(m,16);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  

    /* FIXME This reaction seems like a clear cut case for linearization
  r = Model_getReaction(m,17);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  
  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
*/

  r = Model_getReaction(m,18);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  
  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  
  

  r = Model_getReaction(m,19);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  
  

  r = Model_getReaction(m,20);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  


  r = Model_getReaction(m,21);
  rn = MesoRD::ReactionNode::fromSBML(
    KineticLaw_getMath(Reaction_getKineticLaw(r)), 
    &pt, &st);
  
  rn = MesoRD::ReactionNode::refold(rn);  
  le = rn -> linearise();

  CPPUNIT_ASSERT(le != 0);

  CPPUNIT_ASSERT((rn -> evaluate(val_int)) == (le -> evaluate(val_int)));
  CPPUNIT_ASSERT((rn -> evaluate(val_double)) == (le -> evaluate(val_double)));
  
  


}
