/* The TestSpecies class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestSpecies.cpp 553 2008-05-09 13:28:22Z davidfange $
 */

#include "TestSpecies.hpp"
#include "../src/UnitTable.hpp"
#include "../src/Compartment.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestSpecies );


void 
TestSpecies::setUp()
{
  
  std::string test_filename = "./sbml_source/TestSpecies.xml";
  SBMLDocument* d = readSBML(test_filename.c_str());
  m = d -> getModel();
  uds = m -> getListOfUnitDefinitions();
}


void 
TestSpecies::tearDown()
{
}


void 
TestSpecies::testConstructor()
{
  
  MesoRD::Species s;
  
  std::map<std::string, double> diffConst;
  diffConst["comp1"] = 234;
  diffConst["comp2"] = 235;
  
  
  
  MesoRD::Species s1(diffConst, "test_species_id1", "test_species_name1", 
    "comp_3", ((unsigned int)14), false);
  
  CPPUNIT_ASSERT(s1.getAmount() == 14);
  CPPUNIT_ASSERT(s1.getCompartment() == "comp_3");
  CPPUNIT_ASSERT(s1.getDiffusion() == diffConst);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s1.getDiffusion("comp1"), 
    (double) 234.0, std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s1.getDiffusion("comp2"), (double) 235.0, 
                               std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s1.getId() == "test_species_id1");
  CPPUNIT_ASSERT(s1.getKey() == "test_species_name1");
  CPPUNIT_ASSERT(s1.getName() == "test_species_name1");
  

  MesoRD::Species s2(diffConst, "test_species_id2", "test_species_name2", 
    "comp_4", ((double)15), false);
  
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s2.getAmountDouble(), (double)15.0,
    std::numeric_limits<double>::epsilon() );
  CPPUNIT_ASSERT(s2.getCompartment() == "comp_4");
  CPPUNIT_ASSERT(s2.getDiffusion() == diffConst);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s2.getDiffusion("comp1"), (double)234.0, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s2.getDiffusion("comp2"), (double)235.0,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s2.getId() == "test_species_id2");
  CPPUNIT_ASSERT(s2.getKey() == "test_species_name2");
  CPPUNIT_ASSERT(s2.getName() == "test_species_name2");
  

  MesoRD::Compartment c(NULL);
  c.setVolume(1e-4);
  c.setOriginalUnit("litre");
  
  MesoRD::UnitTable ut(uds);

  MesoRD::Species s3(m -> getSpecies(0), &c, &ut, NULL);

  CPPUNIT_ASSERT(s3.getAmount() == 10);
  CPPUNIT_ASSERT(s3.getCompartment() == "CompartmentOne");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s3.getDiffusion("CompartmentOne"), (double)5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s3.getDiffusion("CompartmentTwo"), (double)5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s3.getId() == "A1");
  CPPUNIT_ASSERT(s3.getKey() == "A");
  CPPUNIT_ASSERT(s3.getName() == "A");

  MesoRD::Species s4(m -> getSpecies(1), &c, &ut, NULL);

  CPPUNIT_ASSERT(s4.getAmount() == 10);
  CPPUNIT_ASSERT(s4.getCompartment() == "CompartmentOne");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s4.getDiffusion("CompartmentOne"), (double)5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s4.getDiffusion("CompartmentTwo"), (double)5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s4.getId() == "A2");
  CPPUNIT_ASSERT(s4.getKey() == "A");
  CPPUNIT_ASSERT(s4.getName() == "A");
  

  MesoRD::Species s5( m -> getSpecies(2), &c, &ut, NULL);

  CPPUNIT_ASSERT(s5.getAmount() == 10);
  CPPUNIT_ASSERT(s5.getCompartment() == "CompartmentOne");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s5.getDiffusion("CompartmentOne"), (double)5e-8, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s5.getDiffusion("CompartmentTwo"), (double)5e-8, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s5.getId() == "A3");
  CPPUNIT_ASSERT(s5.getKey() == "A");
  CPPUNIT_ASSERT(s5.getName() == "A");
  

  MesoRD::Species s6( m ->getSpecies(3), &c, &ut, NULL);

  CPPUNIT_ASSERT(s6.getAmount() == 20);
  CPPUNIT_ASSERT(s6.getCompartment() == "CompartmentOne");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s6.getDiffusion("CompartmentOne"), (double)5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s6.getDiffusion("CompartmentTwo"), 5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s6.getId() == "A4");
  CPPUNIT_ASSERT(s6.getKey() == "A");
  CPPUNIT_ASSERT(s6.getName() == "A");

  c.setOriginalUnit("m3");
  MesoRD::Species s7( m -> getSpecies(4), &c, &ut, NULL);
  
  CPPUNIT_ASSERT(s7.getAmount() == 20);
  CPPUNIT_ASSERT(s7.getCompartment() == "CompartmentOne");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s7.getDiffusion("CompartmentOne"), 5e-8, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s7.getDiffusion("CompartmentTwo"), 5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s7.getId() == "A5");
  CPPUNIT_ASSERT(s7.getKey() == "A");
  CPPUNIT_ASSERT(s7.getName() == "A");

  MesoRD::Species s8( m -> getSpecies(5), &c, &ut, NULL);

  CPPUNIT_ASSERT(s8.getAmount() == 14);
  CPPUNIT_ASSERT(s8.getCompartment() == "CompartmentTwo");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s8.getDiffusion("CompartmentOne"), 5e-8, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s8.getDiffusion("CompartmentTwo"), 5e-8, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s8.getId() == "A6");
  CPPUNIT_ASSERT(s8.getKey() == "A");
  CPPUNIT_ASSERT(s8.getName() == "A");
  
  MesoRD::Species s9( m -> getSpecies(6), &c, &ut, NULL);

  CPPUNIT_ASSERT(s9.getAmount() == 14);
  CPPUNIT_ASSERT(s9.getCompartment() == "CompartmentTwo");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s9.getDiffusion("CompartmentOne"), 5e-8, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s9.getDiffusion("CompartmentTwo"), 5e-8,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s9.getId() == "A7");
  CPPUNIT_ASSERT(s9.getKey() == "A");
  CPPUNIT_ASSERT(s9.getName() == "A");

  
  MesoRD::Species s10( m -> getSpecies(7), &c, &ut, NULL);

  CPPUNIT_ASSERT(s10.getAmount() == 14);
  CPPUNIT_ASSERT(s10.getCompartment() == "CompartmentTwo");
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s10.getDiffusion("CompartmentOne"), 5e-6, 
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s10.getDiffusion("CompartmentTwo"),  2e-6,
    std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT(s10.getId() == "A8");
  CPPUNIT_ASSERT(s10.getKey() == "A");
  CPPUNIT_ASSERT(s10.getName() == "A");  
}


void 
TestSpecies::testConstructorUnknownSubstanceThrow()
{

  MesoRD::Compartment c(NULL);
  MesoRD::UnitTable ut(uds);
  MesoRD::Species s(m -> getSpecies(8), &c, &ut, NULL);
  
}

void 
TestSpecies::testConstructorSubstConvertThrow()
{
  MesoRD::Compartment c(NULL);
  MesoRD::UnitTable ut(uds);
  MesoRD::Species s(m -> getSpecies(9), &c, &ut, NULL);
  
}

void
TestSpecies::testConstructorConcConvertThrow()
{

  MesoRD::Compartment c(NULL);
  c.setOriginalUnit("um");
  MesoRD::UnitTable ut(uds);
  MesoRD::Species s(m -> getSpecies(10), &c, &ut, NULL);
  
  
}



void
TestSpecies::testConstructorDiffUnitThrow() 
{

  MesoRD::Compartment c(NULL);
  MesoRD::UnitTable ut(uds);
  MesoRD::Species s(m -> getSpecies(11), &c, &ut, NULL);
  
  
}

void
TestSpecies::testConstructorDiffUnitConvertThrow()
{
  
  MesoRD::Compartment c(NULL);
  MesoRD::UnitTable ut(uds);
  MesoRD::Species s(m -> getSpecies(12), &c, &ut, NULL);
  
  
}


void
TestSpecies::testGetSets()
{

  MesoRD::Species s;
  

  s.setAmount((unsigned int)345);
  CPPUNIT_ASSERT(s.getAmount() == 345);
  
  s.setAmount((double)345);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s.getAmountDouble(),(double)345,
    std::numeric_limits<double>::epsilon());
  
  s.setCompartment("test_comp");
  CPPUNIT_ASSERT(s.getCompartment() == "test_comp");
  
  std::map<std::string, double> diffConst;
  diffConst["comp1"] = 234;
  diffConst["comp2"] = 235;
  
  s.setDiffusion(diffConst);
  CPPUNIT_ASSERT(s.getDiffusion() == diffConst);
  
  s.setDiffusion("test_comp", 345);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(s.getDiffusion("test_comp"),(double)345,
    std::numeric_limits<double>::epsilon());
  
  s.setId("test_id");
  CPPUNIT_ASSERT(s.getId() == "test_id");
  
  s.setKey("test_key");
  CPPUNIT_ASSERT(s.getKey() == "test_key");
  
  s.setName("test_name");
  CPPUNIT_ASSERT(s.getName() == "test_name");

  MesoRD::Species::setUseDoubleInAmount(true);
  CPPUNIT_ASSERT(MesoRD::Species::getUseDoubleInAmount() == true);
  
}


void
TestSpecies::testIsFunctions()
{
  
  std::map<std::string, double> diffConst;
  diffConst["comp1"] = 234;
  diffConst["comp2"] = 235;
  
  
  
  MesoRD::Species s(diffConst, "test_species_id1", "test_species_name1", 
    "comp_3", ((unsigned int)14), true);
  
  CPPUNIT_ASSERT(s.isCompartment("comp_3"));
  CPPUNIT_ASSERT(s.isConstant());
  CPPUNIT_ASSERT(s.isId("test_species_id1"));
  CPPUNIT_ASSERT(s.isKey("test_species_name1"));
  CPPUNIT_ASSERT(s.isName("test_species_name1"));
}
