/* The TestUnitFactor class.
 *
 * Copyright 2004-2005 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestUnitFactor.cpp 550 2008-05-07 14:24:19Z davidfange $
 */

#include "TestUnitFactor.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestUnitFactor );


void 
TestUnitFactor::setUp()
{
}


void 
TestUnitFactor::tearDown()
{
}


void 
TestUnitFactor::testConstructor()
{

    /* Using the value constructor and then testing it
     */
  MesoRD::UnitFactor ampere=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::ampere, (double) 3.4);
  MesoRD::UnitFactor candela=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::candela, (double) 3.4);
  MesoRD::UnitFactor dimensionless=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::dimensionless,(double) 3.4);
  MesoRD::UnitFactor kelvin=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::kelvin, (double) 3.4);
  MesoRD::UnitFactor kilogram=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::kilogram, (double) 3.4);
  MesoRD::UnitFactor metre=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::metre, (double) 3.4);
  MesoRD::UnitFactor radian=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::radian, (double) 3.4);
  MesoRD::UnitFactor second=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::second, (double) 3.4);
  MesoRD::UnitFactor steradian=MesoRD::UnitFactor(4, 
    MesoRD::UnitFactor::steradian, (double) 3.4);

  CPPUNIT_ASSERT_EQUAL(ampere.getKind(), MesoRD::UnitFactor::ampere);
  CPPUNIT_ASSERT_EQUAL(candela.getKind(), MesoRD::UnitFactor::candela);
  CPPUNIT_ASSERT_EQUAL( dimensionless.getKind(), 
    MesoRD::UnitFactor::dimensionless);
  CPPUNIT_ASSERT_EQUAL(kelvin.getKind(), MesoRD::UnitFactor::kelvin);
  CPPUNIT_ASSERT_EQUAL(kilogram.getKind(), MesoRD::UnitFactor::kilogram);
  CPPUNIT_ASSERT_EQUAL(metre.getKind(), MesoRD::UnitFactor::metre);
  CPPUNIT_ASSERT_EQUAL(radian.getKind(), MesoRD::UnitFactor::radian);
  CPPUNIT_ASSERT_EQUAL(second.getKind(), MesoRD::UnitFactor::second);
  CPPUNIT_ASSERT_EQUAL(steradian.getKind(), MesoRD::UnitFactor::steradian);


  CPPUNIT_ASSERT(ampere.getKindString() == "ampere");
  CPPUNIT_ASSERT(candela.getKindString() == "candela");
  CPPUNIT_ASSERT(dimensionless.getKindString() == "dimensionless");
  CPPUNIT_ASSERT(kelvin.getKindString() == "kelvin");
  CPPUNIT_ASSERT(kilogram.getKindString() == "kilogram");
  CPPUNIT_ASSERT(metre.getKindString() == "metre");
  CPPUNIT_ASSERT(radian.getKindString() == "radian");
  CPPUNIT_ASSERT(second.getKindString() == "second");
  CPPUNIT_ASSERT(steradian.getKindString() == "steradian");


  CPPUNIT_ASSERT(ampere.getExponent() == (int) 4);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(ampere.getMultiplier(), (double) 3.4, 
                              std::numeric_limits<double>::epsilon());
  
}

void 
TestUnitFactor::testGetSets()
{
  MesoRD::UnitFactor uf = MesoRD::UnitFactor(0, MesoRD::UnitFactor::metre, 0);
  
  uf.setExponent(42);
  uf.setMultiplier(32.42);
  uf.setKind(MesoRD::UnitFactor::second);
  
  CPPUNIT_ASSERT_EQUAL(uf.getExponent(), (int) 42);
  CPPUNIT_ASSERT_DOUBLES_EQUAL(uf.getMultiplier(), (double) 32.42,
                              std::numeric_limits<double>::epsilon());
  CPPUNIT_ASSERT_EQUAL(uf.getKind(), MesoRD::UnitFactor::second);
}

void
TestUnitFactor::testOperators()
{
  MesoRD::UnitFactor uf1 = MesoRD::UnitFactor(2, MesoRD::UnitFactor::metre, 0);
  MesoRD::UnitFactor uf2 = MesoRD::UnitFactor(-3, 
    MesoRD::UnitFactor::second, 0);
  
  CPPUNIT_ASSERT(uf1 < uf2);
  CPPUNIT_ASSERT(!(uf1 > uf2));
  CPPUNIT_ASSERT(uf1 != uf2);
  CPPUNIT_ASSERT(!(uf1 == uf2));

  uf2.setExponent(3);

    /* Factors with same signed exponents should be sorted according
     * to the name string
     */
  CPPUNIT_ASSERT(uf1 < uf2);
  CPPUNIT_ASSERT(!(uf1 > uf2));
  CPPUNIT_ASSERT(uf1 != uf2);
  CPPUNIT_ASSERT(!(uf1 == uf2));

  uf2.setKind(MesoRD::UnitFactor::metre);
  uf2.setExponent(2);
  uf2.setMultiplier(42.42);
  uf1.setMultiplier(42.42);
  
  
  CPPUNIT_ASSERT(!(uf1 < uf2));
  CPPUNIT_ASSERT(!(uf1 > uf2));
  CPPUNIT_ASSERT(!(uf1 != uf2));
  CPPUNIT_ASSERT(uf1 == uf2);
  
} 
