/* The TestUnitTable class.
 *
 * Copyright 2004-2008 Johan Elf, David Fange, Johan Hattne
 *
 * This file is part of MesoRD.
 *
 * MesoRD is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MesoRD is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MesoRD; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: TestUnitTable.cpp 550 2008-05-07 14:24:19Z davidfange $
 */

#include "TestUnitTable.hpp"
#include "../src/Unit.hpp"
#include "../src/UnitFactor.hpp"

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( TestUnitTable );


void 
TestUnitTable::setUp()
{
  std::string test_filename = "./sbml_source/TestUnitTable.xml";
  SBMLDocument* d = readSBML(test_filename.c_str());
  m = d -> getModel();
}


void 
TestUnitTable::tearDown()
{}

void 
TestUnitTable::testConstructor()
{
 
  ListOfUnitDefinitions* ud = m -> getListOfUnitDefinitions();

  MesoRD::UnitTable ut(ud);
  
  CPPUNIT_ASSERT((*ut.getUnit("fulmeter")) == MesoRD::Unit::metre());
       
     
  double multiplier = pow(MesoRD::Constant::Avogadro,-2)*1000;

  MesoRD::UnitFactor second = MesoRD::UnitFactor(2, 
    MesoRD::UnitFactor::second, (double) multiplier);

  MesoRD::UnitFactor metre = MesoRD::UnitFactor(-2, 
    MesoRD::UnitFactor::metre, (double) 1);
  
  std::vector<MesoRD::UnitFactor> UnitFactors;
  
  UnitFactors.push_back(second);
  UnitFactors.push_back(metre);
  
  MesoRD::Unit test1(UnitFactors, "test_fold", "test_fold");

  CPPUNIT_ASSERT(ut.getUnit("test_fold") != NULL);
  MesoRD::Unit* test_fold = ut.getUnit("test_fold");

  
  CPPUNIT_ASSERT(*test_fold == test1);

  metre = MesoRD::UnitFactor(1, 
    MesoRD::UnitFactor::metre, (double) 1e-6);
  
  UnitFactors.clear();
  
  UnitFactors.push_back(metre);
  
  MesoRD::Unit test2(UnitFactors, "test_fold", "test_fold");
  
  CPPUNIT_ASSERT(ut.getUnit("um") != NULL);
  CPPUNIT_ASSERT(*(ut.getUnit("um")) == test2);
}

void
TestUnitTable::testGetLengthUnits()
{ 
  MesoRD::UnitTable ut( m -> getListOfUnitDefinitions());
  
  std::list<std::string> luList = ut.getLengthUnits();
  
  CPPUNIT_ASSERT(luList.size() == 2);
}
